package FLAnalyzer;

import ij.ImagePlus;
import ij.gui.PolygonRoi;
import ij.gui.Roi;
import ij.measure.Calibration;
import ij.measure.ResultsTable;
import ij.plugin.filter.Analyzer;
import ij.process.ImageProcessor;
import ij.process.ImageStatistics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Point2D;
import java.text.DecimalFormat;

class CircStats
         {  
int maxImageWidth = 12000;
public FLVars V = null;
public Roi [] Rois = null;
public  String nc="not calculated";                                          
/**Value from {@link FLVars#AREAofHull}*/
public String Area = nc;
/**Value from {@link FLVars#perimeterOfHull}*/
public   String Perimeter = nc;
/**Value from {@link FLVars#circularity}*/
public String Circularity=nc;
/**Value from {@link FLAnalyzer.CircStats#measureHullandCircle 
 * array[3][1]-array[3][0]}*/
public String MargWidth =nc;
/**Value from {@link FLAnalyzer.CircStats#measureHullandCircle 
 * array[3][3]-array[3][2]}*/
public String MargHeight =nc;
/**from {@link FLVars#MaxRadius}*/
public String MaxRadiusStr =nc;
/** from {@link FLVars#CVRadii}*/
public String CVRadiiStr =nc;
/**This string is made from {@link FLVars#MaxOverMinRadii }*/
public String MaxOverMinRadiiStr =nc;
/**from {@link FLVars#MeanRadii}*/
public String MeanOfHullRadii =nc;
/**from {@link FLVars#CircleMiddlex} and
* {@link FLVars#CircleMiddley}*/
public String Circlecentres =nc;
/**From
* {@link FLVars#MassCentrex} and
* {@link FLVars#MassCentrey}*/
public String Masscentres =nc;
/**From the ratio of the greater of 
* {@link FLVars#major} and 
* {@link FLVars#minor}
* and the lesser of them */
public String SpanRatio=nc;
/**From {@link FLVars#MaximumSpanAcrossHull}*/
public String MaxSpanOfPixels=nc;
/**From {@link FLVars#CircleDiameter}*/
public String CircleDiameter=nc;
/**From {@link FLVars#MaxRadiuscircle}*/
public String MaxRadiusCircleStr =nc;
/**From {@link FLVars#CVRadiicircle}*/
public String CVRadiiCircleStr =nc;
/**This string is made from
* {@link FLVars#MaxOverMinRadiicircle }*/
public String MaxOverMinRadiiCircleStr =nc;
/**From {@link FLVars#MeanRadiicircle}*/
public String MeanOfHullCircleRadii =nc;

public CircStats ()
{}
/**Finds the convex hull and minimum bounding circle on the 
 * passed ImageProcessor, then stores internal stats about them.
*@see #measureHullandCircle
*@see #Area
@see #Perimeter
@see #Circularity
@see #SpanRatio
 @param ipk ImageProcessor to work on 
 @param thisslice int for the current slice
 @param lastslice int for the last slice to process
 @param verticalline boolean that switches methods used
 @param horizontalline boolean that switches methods used
 @param isRectangleSubScan boolean for type of scan
 @param ThisRectangleSubScan int for the number of the 
 rectangle or particleanalyzer particle
 @param v FLVars to use for analysis
 @param rois array of Rois
 @return FLVars instance of variables*/
public FLVars getCircStats (
ImageProcessor ipk, 
int thisslice, 
int lastslice,
boolean verticalline, 
boolean horizontalline,
boolean isRectangleSubScan,
int ThisRectangleSubScan, 
FLVars v,
Roi  [] rois)
{
    Rois = rois;
    V=v;
    //==================================
    //                  GET THE CONVEX HULL AND 
    //                  MINIMUM BOUNDING CIRCLE
    //     THIS CALL ALSO DRAWS THEM IF THAT OPTION IS 
    //                        SET IN THE FLVARS
    //===================================
     float [][] dimensions = measureHullandCircle (
             ipk, thisslice, lastslice,
             V.foreground,
             verticalline, horizontalline,                                  
             V.UseSpanMethod, V.UseTriMethod, 
             isRectangleSubScan, 
             ThisRectangleSubScan
             );
     //----------------------------------------------------------------------------
     //         STORE DATA ABOUT THE HULL AND CIRCLE
     //----------------------------------------------------------------------------
     Area = fnum (V.AREAofHull);
     Perimeter = fnum (V.perimeterOfHull);
     Circularity=fnum (V.circularity);
     MargWidth =fnum (dimensions[3][1]-dimensions[3][0]);
     MargHeight = fnum (dimensions[3][3]-dimensions[3][2]);
     MaxRadiusStr = fnum (V.MaxRadius);
     MaxOverMinRadiiStr = fnum (V.MaxOverMinRadii);
     CVRadiiStr = fnum (V.CVRadii);
     MeanOfHullRadii= fnum (V.MeanRadii);
     float maj=Math.max (V.major, V.minor);
     float min=Math.min (V.major, V.minor);
     SpanRatio=fnum (maj/min);
     
     if(    V.ScanSeparateRandomSubAreas   ||
             V.UseParticleAnalyzer  ||
             V.UseRectAnalyzer  ||
             V.UseRoiManager)
     {
         int wOld, hOld, wNew, hNew;
         /*variables for borders*/
         wOld = Rois [ThisRectangleSubScan].getBounds ().width;
         /*get the image's width and height*/
         hOld =  Rois [ThisRectangleSubScan].getBounds ().height;
         wNew = wOld + V.border;
         /* set up a new value for the added border*/
         hNew = hOld + V.border;
         /*set up a new value for the added border*/
         int xOff = (wNew - wOld)/2;
         // offset for centering
         int yOff = (hNew - hOld)/2;
         // offset for centering
         float XXX=
                 (int)Rois[ThisRectangleSubScan].getBounds().getX() -xOff;
         float YYY=
                 (int)Rois[ThisRectangleSubScan].getBounds ().getY ()-yOff;
         Circlecentres = fnum (V.CircleMiddlex+XXX) +","
                 +fnum (V.CircleMiddley+YYY);
         float mXXX=V.MassCentrex+XXX;
         float mYYY=V.MassCentrey+YYY;
         Masscentres = fnum (mXXX) +"," +fnum (mYYY);
     }
     else if(!V.IsIrregularRoi)
     {
         int wOld, hOld, wNew, hNew;
         /*variables for borders*/
         wOld = Rois [ThisRectangleSubScan].getBounds ().width;
         /*get the image's width and height*/
         hOld =  Rois [ThisRectangleSubScan].getBounds ().height;
         wNew = wOld + V.border;
         /* set up a new value for the added border*/
         hNew = hOld + V.border;
         /*set up a new value for the added border*/
         int xOff = (wNew - wOld)/2;
         // offset for centering
         int yOff = (hNew - hOld)/2;
         // offset for centering
         float XXX=
                 (int)Rois[ThisRectangleSubScan].getBounds ().getX ()-xOff;
         float YYY=
                (int)Rois[ThisRectangleSubScan].getBounds ().getY () -yOff;
         Circlecentres = fnum (V.CircleMiddlex+XXX)+"," 
                 +fnum (V.CircleMiddley+YYY);
         float mXXX=V.MassCentrex+XXX;
         float mYYY=V.MassCentrey+YYY;
         Masscentres = fnum (mXXX) +"," +fnum (mYYY);
     }
     else if (V.IsIrregularRoi)
     {
         int wOld, hOld, wNew, hNew;
         /*variables for borders*/
         wOld = Rois[0].getBounds ().width;
         /*get the image's width and height*/
         hOld =  Rois[0]. getBounds ().height;
         wNew = wOld + V.border;
         /* set up a new value for the added border*/
         hNew = hOld + V.border;
         /*set up a new value for the added border*/
         int xOff = (wNew - wOld)/2;	
// offset for centering
         int yOff = (hNew - hOld)/2;	
// offset for centering
         int XXX=(int)Rois[0]. getBounds ().getX ()-xOff;
         int YYY=(int)Rois[0]. getBounds ().getY ()-yOff;
         XXX=XXX+V.NonRectX;
         YYY=YYY+V.NonRectY;
         Circlecentres = fnum (V.CircleMiddlex+XXX)
                 +"," +fnum (V.CircleMiddley+YYY);
         Masscentres = fnum (V.MassCentrex+XXX)  +"," 
                 +fnum (V.MassCentrey+YYY);
     }
     ///            STORE DATA ABOUT THE SPANS                /
     MaxSpanOfPixels= fnum (V.MaximumSpanAcrossHull);
     MaxRadiusCircleStr = fnum (V.MaxRadiuscircle);
     MaxOverMinRadiiCircleStr = fnum (V.MaxOverMinRadiicircle);
     CVRadiiCircleStr = fnum (V.CVRadiicircle);
     MeanOfHullCircleRadii= fnum (V.MeanRadiicircle);
     CircleDiameter=fnum (V.CircleDiameter);
     
     return V;
 }

                     
/**Formats a float to a string using DecimalFormat.
* @param   sum float to format
* @return  string from the passed float*/
public String fnum (float sum)
{
    DecimalFormat myFormatter = 
            new DecimalFormat ("####.####");
    String output = myFormatter.format (sum);
    return output;
}     
                     
/**Gets the {@link #getHull convex hull} for the passed 
 * ImageProcessor, makes a bounding circle for the convex hull, and 
 * stores several variables for these shapes.  Calls the {@link 
 * FLGraphics#drawshapes drawing} method to draw the circle and
 * hull. Calls {@link #GetDimensions GetDimensions} method to 
 * find image coordinates, then calls {@link #getHull getPolygon} 
 * method to make an IJ polygon roi as the convex hull. The bounding
 * circle is made by calling {@link #ChooseBoundingCircle the circle
 * method}.Resets the value for {@link FLVars#CircleDiameter}.
 * Uses the IJ analyzer and ResultsTable to change variables for 
 * {@link FLVars#AREAofHull area} and {@link 
 * FLVars#perimeterOfHull}, then < p>if {@link 
 * FLVars#perimeterOfHull perimeter}=0 {@link 
 * FLVars#circularity}  = 0; otherwise {@link FLVars#circularity}
 * = 4.0*Math.pi*({@link FLVars#AREAofHull}/{@link 
 * FLVars#perimeterOfHull}<sup>2</sup>;</p>(for a circle this
 * ratio= 1.00)<p></p><p>Bounding Circle's {@link 
 * FLVars#CircleDiameter} <p>Some other measures returned in 
 * the results file are found during {@link #getHull the hull} and
 * {@link FLMain#FindMargins margins methods}. In the case that
 * the roi is a horizontal or vertical line, the bounding circle is made 
 * based on the line. (See Costa and Cesar, Shape Analysis and 
 * Classification).
 *@return float [][] that is the hull, bounding circle, and margins
* @param isRectangleSubScan boolean
* @param ThisRectangleSubScan int
* @param ip ImageProcessor of binary image to make a 
 * {@link #getHull convex hull} 
 * @param thisslice int for the current slice number (use 1 if not a 
 * stack)
 * @param lastslice int for the number of slices in this stack (use 1
 * if not a stack)
 * @param FOREGROUND int for foreground (is 255 or 0)
 * @param UseSpanMethod boolean is true
 * if the span method is being forced
 * @param UseTriMethod boolean is true if the triangle method is 
 * being forced
* @param horizontalline boolean true if this image is a single pixel 
 * horizontal line
 * @param verticalline boolean true if the image has a single column
 * of foreground pixels*/
public float [][] measureHullandCircle (
             ImageProcessor ip,
             int thisslice,
             int lastslice,
             int FOREGROUND, 
             boolean verticalline,
             boolean horizontalline,                               
             boolean UseSpanMethod,
             boolean UseTriMethod,
             boolean isRectangleSubScan,
             int ThisRectangleSubScan )
 {
     int [] m=new int [4];
     m=GetDimensions (ip, FOREGROUND);
     if (m==null) return null;
     int  [][]  HullCoords = new int [2] [];
     float [] BoundingCircle=new float[3];

     PolygonRoi polyhull=null;
     if (verticalline)
     {
         HullCoords[0] = new int []{m[0]+1, m[0], m[0], m[0]+1};
         HullCoords[1]= new int [] {m[2], m[2], m[3], m[3]};
         polyhull= new PolygonRoi ( HullCoords[0], HullCoords[1],
                 4, Roi.POLYGON);
         V.MaximumSpanAcrossHull=m[3]-m[2];
         V.MinimumSpanAcrossHull=1;
         BoundingCircle[0]=m[0]-(V.MaximumSpanAcrossHull/2.0f);
         BoundingCircle[1]=m[2];//margtop;
         BoundingCircle[2]=V.MaximumSpanAcrossHull;    
     }
     else if (horizontalline)
     {

         HullCoords[0] = new int []{m[0], m[0], m[1], m[1]};
         HullCoords[1] =new int [] {m[2], m[2]+1, m[2]+1, m[2]};
         polyhull= new PolygonRoi (HullCoords[0], HullCoords[1],
                 4, Roi.POLYGON);
         V.MaximumSpanAcrossHull =(m[1]-m[0]);
         BoundingCircle[0]=m[0];
         BoundingCircle[1]=m[2]-(V.MaximumSpanAcrossHull /2.0f);
         BoundingCircle[2]=V.MaximumSpanAcrossHull ;

         V.MinimumSpanAcrossHull=1;
     }
     else
     {
         HullCoords = getHull (ip, m);
         polyhull = new PolygonRoi (HullCoords[0], HullCoords[1],
                 HullCoords[2][0], null, Roi.POLYGON);
         float []Spans=GetSpans (HullCoords[0], HullCoords[1]);
         V.MaximumSpanAcrossHull=Spans[0];
         V.MinimumSpanAcrossHull=Spans[1];
         BoundingCircle = ChooseBoundingCircle (HullCoords[0], 
                 HullCoords[1],
                 Spans, UseSpanMethod, UseTriMethod);


     }
     V.CircleMiddlex=BoundingCircle[0]+(BoundingCircle[2]/2.0f);
     V.CircleMiddley=BoundingCircle[1]+(BoundingCircle[2]/2.0f);
     V.CircleDiameter=BoundingCircle[2];

     ImagePlus img = new ImagePlus ("nonn", ip);

     if(polyhull!=null) { img.setRoi (polyhull);
       Analyzer ana = new Analyzer ();
//     and analyse the enclosed area          
       int ms = 0;
       ms |= ana.AREA+ana.PERIMETER+ana.ELLIPSE+
               ana.CENTER_OF_MASS;
       ana.setMeasurements (ms);
       ImageStatistics stats = img.getStatistics (ms);

       {Calibration cal = img.getCalibration ();
        cal.pixelWidth = 1.0;
        cal.pixelHeight = 1.0;
        cal.setUnit ("pixel");
        img.setCalibration (cal);
       }
       Roi roi = img.getRoi ();
       ana.saveResults (stats, roi);
       ResultsTable rt =ana.getResultsTable ();
       int cr = rt.getCounter ();
       V.AREAofHull = rt.getValue (rt.AREA, cr-1);
       V.MassCentrex=rt.getValue (rt.X_CENTER_OF_MASS, cr-1);
       V.MassCentrey = rt.getValue (rt.Y_CENTER_OF_MASS, cr-1);

       float [] maxmincircle  = FindDistancesFromCentre (
               V.CircleMiddlex, V.CircleMiddley, 
               IntTofloatArray (HullCoords[0]),
               IntTofloatArray (HullCoords[1]));

       float [] maxmin       = FindDistancesFromCentre (
               V.MassCentrex, V.MassCentrey,
               IntTofloatArray (HullCoords[0]),
               IntTofloatArray (HullCoords[1]));           

       V.MaxOverMinRadii=maxmin[0]/maxmin[1];
       V.MaxRadius=maxmin[0];
       V.CVRadii=maxmin[2]/maxmin[3];
       V.MeanRadii= maxmin[3];
       V.MaxOverMinRadiicircle=maxmincircle[0]/maxmincircle[1];
       V.MaxRadiuscircle=maxmincircle[0];
       V.CVRadiicircle=maxmincircle[2]/maxmincircle[3];
       V.MeanRadiicircle= maxmincircle[3];
       V.perimeterOfHull = rt.getValue (rt.PERIMETER, cr-1);
       V.major = rt.getValue (ResultsTable.MAJOR, cr-1);
       V.minor = rt.getValue (ResultsTable.MINOR, cr-1);
       if(V.AREAofHull!=0)
           V.circularity = (V.perimeterOfHull*
                   V.perimeterOfHull)/V.AREAofHull;

       if (V.perimeterOfHull==0) V.circularity = 0;
       else V.circularity =
               4.0f*((float)Math.PI)*
               (V.AREAofHull/(V.perimeterOfHull*V.perimeterOfHull));


       float [] X=IntTofloatArray (HullCoords[0]);
       float [] Y=IntTofloatArray (HullCoords[1]);
       float [] mm=IntTofloatArray (m);

       //calculate offset

       int wOld, hOld, wNew, hNew;/*variables for borders*/
       wOld =
               Rois
               [ThisRectangleSubScan].getBounds ().width;
       /*get the image's width and height*/
       hOld =  Rois
               [ThisRectangleSubScan].getBounds ().height;
       wNew = wOld + V.border;
       /* set up a new value for the added border*/
       hNew = hOld + V.border;
       /*set up a new value for the added border*/
       int xOff = (wNew - wOld)/2;	// offset for centering
       int yOff = (hNew - hOld)/2;	// offset for centering
       int XXX=(int)Rois
               [ThisRectangleSubScan].getBounds ().getX ()-xOff;
       int YYY=(int)Rois
               [ThisRectangleSubScan].getBounds ().getY ()-yOff;
       int [] xytranslate = new int []{XXX, YYY  };
       //
       if(V.DRAWCircle||V.DRAWHull)
       {
           if(V.UseParticleAnalyzer||V.UseRectAnalyzer||
                   V.ScanSeparateRandomSubAreas||
                   V.IsASingleRectangularRoi||V.UseRoiManager)
//store the PA rois and circles in global arrays
           {                   
               V.PAHulls[ThisRectangleSubScan]=polyhull;
               V.PACircs[ThisRectangleSubScan]=BoundingCircle;
           }
           else FLGraphics.drawshapes (
                   V.DRAWCircle,
                   V.DRAWHull,
                   thisslice, lastslice,
                   polyhull,
                   BoundingCircle,
                   V.originaldrawing,                                            
                   isRectangleSubScan,
                   ThisRectangleSubScan, 
                   xytranslate,
                   V.TitleAndSlice,
                   V);
       }
       return new float [][]{X, Y, BoundingCircle, mm};
     }return null;
 }
                   /**Determine the dimensions of the image,
         and sets some of the important variables.
      * Returns the left, right, top, and bottom 
      * margins of the pixelated
      * part of the passed image considering only foreground pixels.
      * @param ip ImageProcessor
      * @param ForeGround int for foreground pixels
      * (255 or 0 usually)
      * @return int [] of the left, right, top, and
      * bottom margins of the pixelated part of the image
      */
     public int [] GetDimensions (ImageProcessor ip, int ForeGround)
     {
            
         
         int width = ip.getWidth ();/*record the width and height*/
         int height = ip.getHeight ();
         Rectangle roi;
         int[] histogram = new int[256];/*make an array for histogram*/
         ip.setRoi (0,0, width, height);
         histogram=ip.getHistogram ();
         if (histogram[ForeGround]==0) return null;
         //start at the leftmost edge, move to the middle,
         //and find the first coloured pixel for the left edge
         int left = -1;
         do
         {	left++;
                ip.setRoi (left, 0, 1, height);
                histogram = ip.getHistogram ();
                
         } while ( histogram[ForeGround]==0 &&left<width);
         
         //start at the top and find the first pixel
         int top = -1;
         do
         {	top++;
                ip.setRoi (left, top, width, 1);
                histogram = ip.getHistogram ();
                
         } while (histogram[ForeGround]==0&&top<height);
         
         //start at the right and find the first pixel
         int right =width-1;
         do
         {	right--;
                ip.setRoi (right, top, 1,height);
                histogram = ip.getHistogram ();
                
         } while (histogram[ForeGround]==0&&right>left);
         
         //start at the bottom of the image and find the first pixel*/
         int bottom =height;
         do
         {	bottom--;
                ip.setRoi (left, bottom, right, 1);
                histogram = ip.getHistogram ();
                if(bottom==top)break;
         } while (histogram[ForeGround]==0&&bottom>top);
         
         //We have to ensure these regions don't overlap.
         top--;
         bottom++;
         
         return new int[]{left, right, top, bottom};
     }      
     
     /**
* Thomas Roy's algorithm for a convex hull enclosing
* the foreground pixels
* of the passed ImageProcessor, within the passed margins.
* Use PolygonRoi(X, Y, coords.length, null, Roi.POLYGON)
* to convert the returned set of coordinates to a polygon.
* @return array of integer coordinates for a convex hull; to make
* the coordinates into an roi, use 
* ( PolygonRoi(XX, YY, length, null, Roi.POLYGON)
*@param margins the left, right, top, and bottom
* margins of the area to be assessed on the
*passed ImageProcessor
* @param ip the ImageProcessor to assess
*/
                 public int [][] //PolygonRoi
                         getHull (ImageProcessor ip, int [] margins)
                 {
                     int LEFT=margins[0], RIGHT=margins[1], 
                             TOP=margins[2], BOTTOM=margins[3];
                     
                     Point coords[] = new Point[maxImageWidth * 2];
                     for (int i = 0; i < maxImageWidth * 2; i++)
                         coords[i] = new Point ();
                     int[] histogram = new int[256];
                     //int vertCount=0;
                     int y, x;
                     Point farLeft, farRight;
                     
                     //Start by mapping farthest left point
                     x = LEFT;//this.left;
                     y=TOP;//top;
                     do
                     {
                         y++;
                         ip.setRoi (x, y, 1, 1);
                         histogram = ip.getHistogram ();
                     } while (( histogram[ V.foreground ]==0 ) 
                     && y<=BOTTOM);
                     farLeft = new Point (x,y);
                     int index = x;
                     //store the leftmost top point
                     coords[0].x=index;
                     coords[0].y=y;
                     
                     //Find the furthest right, top point
                     x = RIGHT;//this.right;
                     y=TOP;//top;
                     do
                     {
                         y++;
                         ip.setRoi (x, y, 1, 1);
                         histogram = ip.getHistogram ();
                     } while (( histogram[ V.foreground ]==0 ) 
                     && y<=BOTTOM);//bottom);
                     farRight = new Point (x,y);
//store the furthest right top point
                     
                     
                     
                     //     String j="";;
                     int INDEX=1;
                     
                     //Map the top and bottom contours
                     int endPoint = 
                             mapTopaContours (TOP, BOTTOM, 
                             farLeft, farRight, coords,ip);
                     //if the last point in the top lies along the right end and
                     //the first point in the bottom right 
                     //are both along the right end
                     //make sure the first bottom point
                     //on the right is not lost
                     int total = mapBottomaContours (BOTTOM, 
                             TOP, farLeft, farRight, coords,
                             endPoint, ip);
                     String j = "";
                     //        if((coords[total].x==coords[0].x)
                     //&&(coords[total].y == coords[0].y));
                     //        else
                     //if the last point is not already the first point
                     {total++;
                      coords[total].x=coords[0].x;
                      coords[total].y=coords[0].y;
                     }
                     
                     coords = removeContours (coords, total, endPoint);
                     
                     
                     //Produce an array for the output
                     int [] X = new int [coords.length];
                     int [] Y = new int [coords.length];
                     for (int i = 0; i < coords.length; i++)
                     {
                         X[i]=coords[i].x;
                         Y[i]=coords[i].y;
                     }
                     int [] length = new int[1];
                     length[0]=coords.length;
                     return new int [][]{X, Y, length};
//PolygonRoi(X, Y, coords.length, null, Roi.POLYGON);
                 }  
      /**Returns the maximum span, the minimum span, and the
      *indices of the maximum span within the passed convex hull.
      *@param XX float [] of x coordinates for a convex hull
      *@param YY float [] of y coordinates for convex hull
      *@return float [] holding
      *<ol> <li>the maximum span across the
      * hull at spans[0],</li><li>
      *the minimum span across the hull at spans[1], </li><li>
      * and the indices for the maximum span's
      *two points at spans[2] and spans[3]- i.e., XX[spans[2]] 
      * and YY[spans[2]] are one set of endpoints
      *  and XX[spans[3]] with YY[spans[3]] are 
      * the other set of endpoints
      * for the maximum span across the hull
      * </li></ol>*/
     public float [] GetSpans (int [] XX, int [] YY)
     {
         float span=0;
         int max_i = 0;
         int max_i2= 0;
         float MinimumSpanAcrossHull=0f;
         float MaximumSpanAcrossHull=0f;
         //find the greatest distance between two points on the
         //convex hull
         for (int anchor=0; anchor<XX.length; anchor++)
         {
             for (int i = 0; i< XX.length; i++)
             {
                 float dist = (float)Math.abs (Point2D.distance ((float)XX[i],
                         (float)YY[i], (float)XX[anchor], (float)YY[anchor]));
                 if (dist>span)
                 {
                     span=dist;
                     max_i=i;
                     max_i2=anchor;
                 }
                 if (i==0&&anchor==0)
                 {
                     MinimumSpanAcrossHull=dist;
                     MaximumSpanAcrossHull=dist;
                 }
                 
                 MinimumSpanAcrossHull=
                         (float)Math.min (dist, MinimumSpanAcrossHull);
                 MaximumSpanAcrossHull=
                         (float)Math.max (dist, MaximumSpanAcrossHull);
             }
         }
         float [] spans = new float[]{MaximumSpanAcrossHull,
                 MinimumSpanAcrossHull, max_i, max_i2};
                 return spans;
                 
     }
     
      /**
      * Returns a circle specified by the coordinates in
      * the returned array,
      * based on the
      * polygon that is specified as an roi with points XX, YY.  
      * The drawing method
      * Graphics2D.data.drawOval(x, y, width, height) draws an oval
      * that fits in the rectangle specified by the coordinates using
      * x,y for the top left corder.
      * The coordinates are also rounded to integers before
      * being passed to this method by the usual calling function,
      * so the original circle is somewhat modified from the more
      * precise coordinates {@link #measureHullandCircle 
      * calculated} earlier.
      * @param XX int [] of x coordinates for the convex hull
      * @param YY int [] of y coordinates for the convex hull
      * @param spans an array of [0]= maximum and [1]=
      * minimum spans across
      * the hull specified by XX,YY, and [2]=index of one endpoint 
      * for the maximum span,
      * and [3]=the index of the other endpoint for the maximum span
      * @param ForceSpan true forces the result from
      * {@link #FindBoundingCircleFromMaxSpan the span method}
      * @param ForceTriangle true forces the
      * {@link #FindTriangleMethod result fromthe triangle method}
      * as long as ForceSpan is false
      * @return float[] of parameters for making a bounding circle
      * @see #FindBoundingCircleFromMaxSpan
      * @see #FindTriangleMethod
      */
     public float[] ChooseBoundingCircle (
             int[] XX,
             int[] YY,
             float [] spans,
             boolean ForceSpan,
             boolean ForceTriangle)
     {
         
         float [] AA = FindBoundingCircleFromMaxSpan (XX, YY, spans);
         float [] BB=null;
         
         //check to see if the circle is defined by the hull's span
         if((AA[3]==1)
         ||ForceSpan)
         {V.methodused="span";
          return AA;}
         //return the bounding circle defined by
         //the hull span if appropriate
         else
         {//Find the bounding circle using the tri method
             BB=FindTriangleMethod (XX, YY,  spans[0]);
             
             if (BB==null)
             {V.methodused="span";return AA;}
             
             if (ForceTriangle)
             {V.methodused="triangle";return BB;}
         }
         
         if (BB==null&&AA!=null)
         {V.methodused="Span"; return AA;}
         if (AA==null&&BB!=null)
         {V.methodused="Triangle"; return BB;}
         float [] finalroi = ChooseEnclosingCircle (AA, BB, XX, YY);
         return finalroi==AA ? AA : BB;
     }
     
     /**
      * Returns a float array that is one of the  passed two
      * and which defines the minimum bounding circle
      * for the array of X,Y coordinates defining a convex hull.
      *Calls {@link #ContainsAllPoints point checking}
      * method to ensure that
      *both circles contain all the points of the convex hull, 
      * and chooses the smaller
      *of the two that is not null and contains all the points.  
      * Sets the
      *value of V.methodused in the 
      * {@link FLVars variables instance}.
      * @param AA a float array of coordinates for the circle,
      * where the first two elements are the x and y of the rectangle
      * enclosing the
      * bounding circle and the third is the diameter of the circle;
      * thus, the centre is at x+diameter/2.0 and y+diameter/2.0
      * @param BB a float array of coordinates for the circle
      * where the first two elements are the x and y
      * of the rectangle enclosing the
      * bounding circle and the third is the diameter of the circle;
      * thus, the centre is at x+diameter/2.0 and y+diameter/2.0
      * @param XX x coordinates for a convex hull
      * @param YY y coordinates for a convex hull
      * @return float [] that defines the minimal 
      * bounding circle of the two passed
      *
      */
     public float [] ChooseEnclosingCircle (
             float [] AA,
             float [] BB,
             int [] XX,
             int [] YY
             )
     {
         float [] C=null;
         boolean notA=false;
         boolean notB=false;
         if(AA[2]<=0||AA==null)
         {V.methodused="Triangle"; return BB;}
         if(BB[2]<=0||BB==null)
         {V.methodused="span"; return AA;}
         float aradius=AA[2]/2.0f;
         float bradius=BB[2]/2.0f;
         if(!ContainsAllPoints (XX, YY, aradius, 
                 AA[0]+aradius, AA[1]+aradius))notA=true;
         if(!ContainsAllPoints (XX, YY, bradius, 
                 BB[0]+bradius, BB[1]+bradius))notB=true;;
         //Find the smaller of the two circles
         
         boolean Aisbigger=false, Bisbigger=false, SameSize=false;
         if(BB[2]>AA[2])Bisbigger=true;
         if(AA[2]>BB[2])Aisbigger=true;
         if (AA[2]==BB[2])SameSize=true;
         
         if(AA[2]>0 && BB[2]>0)
         {
             if(AA[2]/BB[2]<.5f) C=AA;
             if(BB[2]/AA[2]<.5f) C=BB;
//            if(AA[2]/BB[2] <= 0.93f) C=AA;
//            if(BB[2]/AA[2]<=0.93f  )C=BB;
         }
         
         if (C==null)
         {
             if (SameSize)
             {
                 if(notA) C=BB;
                 else C= AA;
             }
             else if (!SameSize)
             {
                 if(Aisbigger)
                 {
                     if (notA&&!notB)  C= BB;
//if only B contains all the points, return it
                     if (notA && notB)  C= AA;
                     //if neither contains all the points, 
                     //both are too small so return the larger
                     if(!notA && !notB) C= BB;
//if both contain all the points, return the smaller
                     if(!notA && notB)  C= AA;
                     //if only A contains all the points, return it
                     
                 }
                 else if(!Aisbigger)
                 {
                     if (notA && notB)  C= BB;
//if neither contains all the points, return the first method
                     if (notA && !notB)  C= BB;
//if only B contains all the points, return it
                     if( !notA && notB)  C= AA;
                     //if only A contains all the points, return it
                     if( !notA && !notB) C=AA;
//if both contain all the point, return the smaller
                 }
                 
             }
         }
         if(C==AA)V.methodused="span";
         else if (C==BB)V.methodused="triangle";
         if(C==null&&BB==null&&AA!=null)
         {C=AA; V.methodused="forced span";}
         else if (C==null&&AA==null&&BB!=null)
         {
             C=BB; V.methodused="forced triangle";}
         
         return C;
     }
     
     /**
      * Returns true if the passed set of X, Y coordinates are all
      * within the passed radius from the passed
      * point, using Point2D.distance() between
      *  the point at X[i],Y[i] and x, y, and
      * letting any difference greater than 0.001f be 
      * considered not within the radius.
      * @return boolean true if all the points are 
      * within the radius from the passed point
      * @param x float for the centre coordinate agains which to test
      * @param y float for the centre coordinate against which to test
      * @param XX int[] of x coordinates
      * @param YY int [] of y coordinates
      * @param radius float for the distance to test
      */
     public boolean ContainsAllPoints (int[]XX, int[]YY, 
             float radius, float x, float y)
     {
         boolean AllIn=true;
         for (int i = 0; i<XX.length; i++)
         {
             float ds=(float)(Math.abs ((float)
             Point2D.distance ((float )XX[i], (float)YY[i],
                     x, y)));
             //if (ds>radius)AllIn=false;
             if((ds-radius)>0.001f)AllIn=false;
         }
         return AllIn;
         
     }
     
     
     
     
     
     ///////////////////////////////////////////////////////////////////
     ////////////////////////////////////////////////////////////////////
     /**Returns a float array holding three values
      * which are the x coordinate and the y
      * coordinate for the top
      * left corner of the rectangle enclosing the bounding
      * circle and the diameter (width) of the bounding circle.
      * The bounding circle may not be the smallest, so must
      * be compared to the span method first.  Draw
      * it using a function that uses
      * these values, such as DrawOval(x, y, width, width). 
      * The x, y pair is found after the
      * circle's centre is found, by subtracting diameter/2.0
      * from each original coordinate. 
      * Thus, to find the centre of the circle
      *  from the returned float array, 
      * add diameter/2.0 to each coordinate.
      *The passed MaximumSpanAcrossHull can be found using
      *{@link #GetSpans span getter}.
      *Is called by 
      * {@link #ChooseBoundingCircle the bounding circle} method.
      *
      * @param XX an array of x coordinates for convex hull
      * @param YY an array of y coordinates for convex hull
      * @param MaximumSpanAcrossHull float precalculated value
      * @return float [] x, y coordinates for the top left of the 
      * bounding rectangle,
      * and the diameter for the bounding circle of, and the
      * minimum and maximum
      * across the passed convex hull
      *@see #ChooseBoundingCircle
      */
     public float [] FindTriangleMethod
             (int[] XX, int[] YY, float MaximumSpanAcrossHull
             )
     {
         
         {//if the span does not define the bounding circle
             //find a larger bounding circle using the triangle
          
             //initialize the three points to be equal to the hull span
             int max_i = 0,max_i2= 0, max_i3=0;             
           
             //compare all triangle circles,and choose from the ones
             //that are larger than the span 
             //circle and contain all the points
             //the one that is smallest
           
             //at this point we need the biggest diameter and
             //its three points
             //the biggest diameter or hull span was found as sdiameter
             //and the circle corresponding to it is stored already
             //now find each triangle
             boolean ThisIsTheFirstTry=true;
             float Midx, Midy;
             float lastdiameter=0f;
             
             for (int anchor = 0; anchor<XX.length; anchor++)
             {
//iterate through each set of three points, i.e., 
                 //triangle on the hull
                 for (int j = 0; j < XX.length; j++)
                 {
                     for (int i = 0; i < XX.length; i++)
                     {
                         if ((i==j&&j==anchor));
                         else
                             
                         {
                             //make a circle using three 
                             //points that are not the same
                             float [] testcirc =
                                     getCircle (
                                     (float)XX[i], (float)YY[i],
                                     (float)XX[j], (float)YY[j],
                                     (float)XX[anchor],(float)YY[anchor]);
                             
                             float d=testcirc[2];
                             //get that circle's diameter
                             //on the first test, there is no last diameter, so
                             //just keep it if it is greater than the hull span
                             if(d>=MaximumSpanAcrossHull)
                             {
//if the test circle is big enough to be plausible
                                 //see if it has all the points
                                 
                                 Midx=testcirc[0];//-testcirc[2]/2.0f;
                                 Midy=testcirc[1];//-testcirc[2]/2.0f;
                                 
                                 if(ContainsAllPoints (XX, YY, d/2.0f, Midx, Midy))
                                 {
//if the circle contains all the points
                                     //see if it is smaller than the best one so far
                                     if((d<lastdiameter)||ThisIsTheFirstTry)
                                     {
                                         ThisIsTheFirstTry=false;
                                         lastdiameter=d;
                                         max_i=i;
                                         max_i2=j;
                                         max_i3=anchor;
                                     }
                                 }//end if
                             }//end if
                             //if this circle contains all the points and is smaller
                             //than the last one that contained 
                             //all the points, keep it
                         }
                     }
                 }
             }//
             
             //make a circle using the points decided on
             float [] circ =
                     getCircle ((float)XX[max_i], (float)YY[max_i],
                     (float)XX[max_i2], (float)YY[max_i2],
                     (float)XX[max_i3], (float)YY[max_i3]);
             
             float H= circ[0];
             float F = circ[2]/2.0f;
             float K = circ[1];
             
             
             if(ThisIsTheFirstTry) return null;
             return new float []{H-F, K-F, circ[2]
             };
             //boundingcircleVariables);
         }//end find triangle if not forcing the span method
         
     }
     
     /**
      *      Returns the centre coordinates and diameter for
      *      the circle from the passed three points
      *      (a,b), (c,d), and (e,f).  Solves three equations
      *      simultaneously using the basic
      *      equation for a circle.
      *      <ul><li>
      *      (h-x)<sup>2</sup>+(k-y)<sup>2</sup>
      * =r<sup>2</sup></li>
      *     <li>  k = (a<sup>2</sup>+b<sup>2</sup>)*(e-c)
      * + (c<sup>2</sup>+d<sup>2</sup>
      * )*(a-e) + (e<sup>2</sup>+f<sup>2</sup>)*(c-a)) /
      * (2.0*(b*(e-c)+d*(a-e)+f*(c-a))) </li><li>
      * h = ((a<sup>2</sup>+b<sup>2</sup>)*(f-d) +
      * (c<sup>2</sup>+d<sup>2</sup>)*(b-f) +
      * (e<sup>2</sup>+f<sup>2</sup>)*(d-b)) 
      * / (2.0*(a*(f-d)+c*(b-f)+e*(d-b)));
      * </li><li>
      * r = (Math.sqrt((a-h)<sup>2</sup> 
      * + (b-k)<sup>2</sup>))</li>
      *
      * <li>the returned array is {h, -k, 2.0*r}</li></ul>
      *
      *       Does not check for colinearity.
      *      @param a x coordinate of first point
      *      @param b y coordinate of first point
      *      @param c x coordinate of second point
      *      @param d y coordinate of second point
      *      @param e x coordinate of third point
      *      @param f y coordinate of third point
      *      @return float [] containing
      *      the centre coordinates(h, k) and the diameter
      *      of the circle
      */
     public float[] getCircle (float a,
             float b, float c, float d, float e, float f)
     {
         b=-1.0f*b;
         d=-1.0f*d;
         f=-1.0f*f;
         float k = ((sq (a)+sq (b))*(e-c)
         + (sq (c)+sq (d))*(a-e) + (sq (e)+sq (f))*(c-a)) /
                 (2.0f*(b*(e-c)+d*(a-e)+f*(c-a))) ;
         float h = ((sq (a)+sq (b))*(f-d) + (sq (c)+sq (d))*(b-f) +
                 (sq (e)+sq (f))*(d-b)) / (2.0f*(a*(f-d)+c*(b-f)+e*(d-b)));
         
         float r = ((float)Math.sqrt (sq (a-h) + sq (b-k)));
         
         return new float []{h, -1.0f*k, 2.0f*r};
         
     }
     
      /**
                  * Returns a float array from the passed integer array.
                  *
                  * @param   array of integers
                  * @return  float []
                  */
                 public static float[] IntTofloatArray (int[] array)
                 {
                     float [] N = new float[array.length];
                     for (int i = 0; i<array.length; i++)
                         N[i]=(float)array[i];
                     return N;
                 }                                  
                 
                 /**
                  * Returns the square of the passed variable.
                  * @param N a float
                  * @return N*N
                  */
                 public float sq (float N)
                 { return (N*N);}
 
                 
                 /**Thomas Roy's method
                  * 
                  * @param bottom int for bottom
                  * @param top int for top margin
                  * @param farLeft int for left
                  * @param farRight int for right
                  * @param coords Point [] of coordinates
                  * @param vertCount int for vertical count
                  * @param ip ImageProcessor to work on
                  * @return int vertCount
                  */
                 private int mapBottomaContours (int bottom,
                         int top, Point farLeft,
                         Point farRight, Point[] coords,
                         int vertCount, ImageProcessor ip)
                 {
                     //Begin mapping bottom contours, right to left
                     int[] histogram = new int[256];
                     
                     for (int index = farRight.x; index >= farLeft.x; index--)
                     {
                         int y = bottom;
                         do
                         {
                             y--;
                             ip.setRoi (index, y, 1, 1);
                             histogram = ip.getHistogram ();
                         } while (( histogram[ V.foreground ]==0 ) 
                         && y>=top);
                         
                         if (y>top)
                         {
                             vertCount++;
                             coords[vertCount].x = index;
                             coords[vertCount].y=y;
                             // IJ.showMessage("Bottom "+
                             //coords[vertCount].x+" " +coords[vertCount].y);
                         }
                     }
                     return vertCount;
                     
                 }
                 
                 /**Thomas Roy.
                  * 
                  * @param TOP int for top margin
                  * @param BOTTOM int for bottom margin
                  * @param farLeft int for left lmargin
                  * @param farRight int for right
                  * @param coords Point[] array of points
                  * @param ip ImageProcessor to work on
                  * @return int vertCount for vertical count
                  */
                 private int mapTopaContours (int TOP,
                         int BOTTOM, 
                         Point farLeft, Point farRight, Point[] coords,
                         ImageProcessor ip)
                 {
                     //Begin mapping contours left to right
                     int[] histogram = new int[256];
                     int vertCount=0;
                     for (int index = farLeft.x; index <=farRight.x; index++)
                     {
                         
                         int y =TOP;
                         do
                         {
                             y++;
                             ip.setRoi (index, y, 1, 1);
                             histogram = ip.getHistogram ();
                         } while (( histogram[ V.foreground ]==0 ) 
                         && y<=(BOTTOM));
                         
                         if (y<BOTTOM)
                         {
                             //              if(coords[0].x==index&&coords[0].y==y);
                             //              else
                             {
                                 vertCount++;
                                 coords[vertCount].x = index;
                                 coords[vertCount].y = y;
                             }
                             //IJ.showMessage("ytop "
                             //+coords[vertCount].x+" " +coords[vertCount].y);
                         }
                     }
                     return vertCount;
                     
                 }
                 
                 /**Thomas Roy's method.
                  * 
                  * @param coords Point[]
                  * @param length int for length
                  * @param furthest int 
                  * @return Point [] for hull
                  */
                 private Point[] removeContours 
                         (Point[] coords, int length, int furthest)
                 {
     //(Aug 2, 2005)FIXME to deal with the specific case when
     //there are consecutive points with the same x coordinate on
     //the right side
                     int index=0;
                     int N = 0;
                     float rise, run, slope, slope2;
                     
                     //First check the top contours
                     do
                     {
                         float highest = 0;
                         int highestIndex = -1;
                         for(int i = index+1; i <= furthest; i++)
                         {
                             //go through the top contours and check slopes
                             run = coords[i].x - coords[index].x;
                             rise = coords[i].y - coords[index].y;
                             if (run == 0)
                                 slope = 100000;
                             else
                                 slope = rise/run;
                             
                             if (slope <= highest || highestIndex == -1)
                             {
                                 highest = slope;
                                 highestIndex = i;
                             }
                         }
                         coords[N] = coords[index];
                         N++;
                         index = highestIndex;
                         
                     } while (index != -1);
                     
                     //Now check the bottom contours
                     index=furthest;
                     //if the first bottom coordinate
                     //has the same x as the last top coordinate
                     //but has a greater y, then keep it
                     if ((coords[furthest+1].x==coords[furthest].x)&&
                             (coords[furthest+1].y>coords[furthest].y))
                     {
                         index=furthest+1     ;
                         //N++;
                     }
                     do
                     {
                         float highest = 0;
                         int highestIndex = -1;
                         
                         for(int i = index+1; i < length; i++)
                         {
                             run = coords[i].x - coords[index].x;
                             rise = coords[i].y - coords[index].y;
                             if (run == 0)
                                 slope = 100000;
                             else
                                 slope = rise/run;
                             
                             if (slope <= highest || highestIndex == -1)
                             {
                                 highest = slope;
                                 highestIndex = i;
                             }
                         }
                         coords[N] = coords[index];
                         N++;
                         index = highestIndex;
                         
                     } while (index != -1);
                     
                     Point newCoords[] = new Point[N];
                     for (int i = 0; i < N; i++)
                         newCoords[i] = coords[i];
                     
                     return newCoords;
                 }
                 
      /**
      * Finds statistics describing the passed roi based on the passed
      * centre of mass.  Returns the maximum 
      * distance from the centre,
      * the minimum distance from the centre,
      * and the mean and standard deviations of the distances.
      * @return float array of max, min, stddev, 
      * mean of distances from
      * centre to points on passed polygon roi
      * @param cx float for centre x
      * @param cy float for centre y
      * @param XX float array of points for convex hull (roi)
      * @param YY float array of points for convex hull (roi)
      */
     public static float [] FindDistancesFromCentre (float cx,
             float cy, float[] XX, float[] YY)
     {
         
         float max = (float)Math.abs ((float)
         Point2D.distance (XX[0], YY[0], cx, cy));
         float min = (float)Math.abs ((float)
         Point2D.distance (XX[0], YY[0], cx, cy));
         float [] dist= new float[XX.length];
         for (int i = 0; i< XX.length; i++)
         {
             dist[i] = (float)Math.abs
                     ((float)Point2D.distance (XX[i], YY[i], cx, cy));
             max = (float)Math.max (dist[i], max);
             min = (float)Math.min (dist[i], min);
             
         }
         float stdev = StdDev (dist, dist.length);
         float mean = Mean (dist, dist.length);
         return new float []{max, min, stdev, mean};
     }
//      /**
//     * Finds statistics describing the passed roi based on the passed
//     * centre of mass.  Returns the maximum distance
//     * from the centre * 2,
//     * the {@link FLstatistics#mean mean}
//     * and {@link FLstatistics#StdDev standard deviations}
//     * of the distances, 
//     * and the ratio of
//     * the maximum over the minimum distance.
//     * @return double array of max, min, stddev, mean 
//     * of distances from
//     * centre to points on passed polygon roi
//     * @param cx double for centre x
//     * @param cy double for centre y
//     * @param XX double array
//     * @param YY double array
//     */
//    public static float[] FindDistancesFromCentreOfRoi(double cx,
//            double cy, double [] XX,
//            double [] YY) {
//        //find the top, bottom, right, and left coordinates
//        //these are the highest and lowest x and y
//        //measure each point on these lines
//        //store the longest and the shortest
//        // IJ.showMessage("doing");
//        
//        float max =
//                (float)Math.abs(Point2D.distance(XX[0], YY[0], cx, cy));
//        float min =
//                (float) Math.abs(Point2D.distance(XX[0], YY[0], cx, cy));
//        float [] dist= new float[XX.length];
//        for (int i = 0; i< XX.length; i++) {
//            dist[i] = (float)Math.abs(Point2D.distance(XX[i], YY[i], cx, cy));
//            max = (float)Math.max(dist[i], max);
//            min = (float)Math.min(dist[i], min);
//            
//        }
//        float stdev = FLstatistics.StdDev(dist, dist.length);
//        float mean = FLstatistics.Mean(dist, dist.length);
//        return new float []{max, min, stdev, mean};
//    }
     
/**
* Returns the coordinates and diameter of a bounding circle for
* the convex hull defined by the passed set of points
* XX, YY. These values are returned
* as a float array containing x,y, and diameter.
* Compares circles from the maximum span across the
* convex hull and potentially one other point.  
* Calls the {@link #getCircle getCircle} method
*to calculate the circle from two or
* three points. Must be called from
* {@link #ChooseBoundingCircle Bounding Circle } method
*to be accurate; or use {@link #GetSpans the span getter} 
* to calculate the spans.
*      @param XX int array of x coordinates for the convex hull
*      @param YY int array of y coordinates for the convex hull
*@param spans float [] of the maximum [0] 
* and minimum [1]spans across the
*convex hull, and the indices of the maximum[2], [3]
*      @return float []  x and y coordinates for the centre and 
* diameter of the bounding circle;
*the 4th element is 1 if the bounding circle has the same
* diameter as the maximum
*span across the convex hull, or 0 if it does not
*/
public float [] FindBoundingCircleFromMaxSpan
     (int[] XX, int[] YY, float [] spans)
{

 float x1=(float)XX[(int)spans[2]];//(float)XX[max_i];
 float y1=(float)YY[(int)spans[2]];//(float)YY[max_i];
 float x2=(float)XX[(int)spans[3]];//(float)XX[max_i2];
 float y2=(float)YY[(int)spans[3]];//(float)YY[max_i2];
 // ip.drawLine((int)x1, (int)y1, (int)x2, (int)y2);
 // majorcircle=spans[0];//(float)span;
 //finds the midpoint
 float Midx = (x1+x2)/2.0f;
 float Midy =(y1+y2)/2.0f;
 float min=spans[1];//span;
 float maxradius=spans[0]/2.0f;
 float maximumtested=spans[0]/2.0f;
 //find the maximum distance from the midpoint of
 //that line to another point in the convex hull
 int thirdpoint=(int)spans[2];
//max_i;//let the third point equal the first one
//        float testmid = (float)Math.abs((float)Point2D.
 //distance((float)XX[thirdpoint],
//                (float)YY[thirdpoint], Midx, Midy));
 for (int i = 0; i< XX.length; i++)
 {
//check the distance from each point to the middle of the span

     float dist = (float)Math.abs ((float)Point2D.distance 
             ((float)XX[i],
             (float)YY[i], Midx, Midy));
     min = (float)Math.min (dist, min);
     if (dist>maxradius&&dist>maximumtested)
     {//if any point is further from the middle of the span
         //than to its own edges, save that point and distance
         maximumtested=dist;
         thirdpoint=i;
     }

 }//end for to find the distance to the next point from
 //the centre of the span

 boolean isdiameter=false;
 if((thirdpoint==(int)spans[2])//max_i
 ||(thirdpoint==(int)spans[3]//max_i2
         ))
 {
     isdiameter=true;
 }
 //decides if the point is on the current line or radius
 //true if the hull span defines the circle (keep it)

 //minorcircle=(float)min;//store the minor circle

 float [] boundingCircleVariables = new float [4];

 int max_i = (int)spans[2];
 int max_i2=(int)spans[3];
 if (!isdiameter)
 {//if the span does not define the circle, use three points
     //Gets the bounding circle from the three points
     float [] circ = getCircle ((float)XX[max_i], (float)YY[max_i],
             (float)XX[max_i2], (float)YY[max_i2], (float)XX[thirdpoint],
             (float)YY[thirdpoint]);
     V.CircleDiameter=(circ[2]);//store these as they are needed

     boundingCircleVariables[0]=circ[0]-(circ[2]/2.0f);
     boundingCircleVariables[1]= circ[1]-(circ[2]/2.0f);
     boundingCircleVariables[2]= circ[2];
     boundingCircleVariables[3]=0f;
//            if(!ContainsAllPoints(XX, YY, diameter/2.0f, circ[0], circ[1]))
//                IJ.showMessage("ahhhh");

 }
 else
 {//if the span defines the circle, use it
     V.CircleDiameter=spans[0];
     boundingCircleVariables[0]=(Midx-maxradius);
     boundingCircleVariables[1]= (Midy-maxradius);
     boundingCircleVariables[2]= spans[0];
     boundingCircleVariables[3]=1f;

 }

 return boundingCircleVariables;
}
     
      /**
                  * Returns the standard deviation of the passed array, from
                  * the {@link #Calc Calculation} method.
                  * @param x [] floats to assess
                  * @param n int for number of elements in x to assess
                  * @return float for standard deviation of passed array
                  */
                 public static float StdDev (float[] x, int n)
                 {
                     float [] LV = Calc (x, n);
                     return LV[1];
                     
                 }
                 
                             /**
      * Returns the mean of the passed array, as
      * (&#8721;x)/n
      * @param x [] floats to assess
      * @param n int for number of elements in x to assess
      * @return float for the mean
      */
     public static float Mean (float[] x, int n)
     {
         float [] LV = Calc (x, n);
         return LV[2];
     }
                 /**Finds the variance, standard deviation, mean,
                  * and coefficient of variation for the passed
                  * array using n elements
                  * and treating the array as a population
                  * rather than a sample.
                  *Note that variance is calculated as
                  * the ((&#8721;M<sup>2</sup>
                  * -((&#8721;M*&#8721;M)/N))/(N) which
                  * is for a population rather than a sample.;
                  * @param M [] floats to calculate stats for
                  * @param n int for number of elements to use
                  * @return float [] where array[0]=variance,
                  * array[1]=standard deviation, array[2]=mean,
                  * and array[3]=coefficient of variation.*/
                 public static float[] Calc (float[] M, int n)
                 {
                     float SumMsq=0;
                     float SumM=0;
                     float SumMM=0;
                     float SumPsq=0;
                     float N = (float)n;
                     
                     for (int i =0;i<n; i++)
                     {
                         SumM=SumM+M[i];
                         SumMsq=SumMsq+(M[i]*M[i]);
                         
                     }
                     for (int i = 0; i < n; i++)
                     {
                         SumMM = (M[i]*M[i])*(M[i]/SumM);
                     }
                     
                     float [] vals = new float [4];
                     
                     float Variance =((SumMsq-((SumM*SumM)/N))/(N));
                     // float Variance =((SumMsq-((SumM*SumM)/N))/(N-1));
                     vals[0]=Variance;
                     
                     
                     float Stddev = (float)Math.sqrt (Variance);
                     vals[1]=Stddev;
                     
                     float Mean = SumM/N;
                     vals[2] = Mean;
                     
                     float CoefVar=Stddev/Mean;
                     vals[3]=CoefVar;
                     
                     return vals;
                 }
                 
//                  /**
//      * Draws an oval specified by the coordinates in c, and the
//      * polygon that is specified as roi.  The drawing method
//      * Graphics2D.data.drawOval(x, y, width, height) draws an oval
//      * that fits in the rectangle specified by the coordinates using
//      * x,y for the top left corder..
//      * The coordinates are also rounded to integers before
//      * being passed to this method by the usual calling function,
//      * so the original circle is somewhat modified from the more
//      * precise coordinates 
//      * {@link #measureHullandCircle calculated} earlier.
//      * @param xytranslate value for translating drawing
//      * @param DrawCircle true draws the circle
//      * @param DrawHull true draws the hull
//      * @param g2 Graphics2D where the drawing will occur
//      * @param roi convex hull to draw
//      * @param c float [] of x, y for top left corner
//      *  and width (diameter) of rectangle in which
//      *  to draw the circle
//      */
//     public static void DrawPolyandCirc (
//             boolean DrawCircle,
//             boolean DrawHull,            
//             Graphics2D g2,
//             Roi roi,
//             float [] c,
//             int [] xytranslate
//             )
//     {
//         BasicStroke roundStroke = new BasicStroke (0f, 
//                 BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER);
//         int i = 0;
//         g2.setColor (Color.red);
//         g2.setStroke (roundStroke);           
//         if(DrawHull&&roi!=null)
//         {
//             g2.translate (xytranslate[0], xytranslate[1]);
//             g2.drawPolygon (roi.getPolygon ());
//             g2.translate (-xytranslate[0], -xytranslate[1]);
//         }             
//         if(DrawCircle&&c!=null)
//         {
//           g2.translate (xytranslate[0], xytranslate[1]);
//           g2.translate (1, 1);
//           g2.drawOval ((int)c[0], (int)c[1], (int)c[2], (int)c[2]);
//           g2.translate (-1, -1);
//           g2.translate (-xytranslate[0], -xytranslate[1]);
//         }        
//     }
                 
                 }