// Notice
//
// The contents of this file are subject to the
//  same license as the source code for ImageJ.  
//FracLac is in the public domain.
// If you use it to publish research, please cite the source.
// The initial developer of the original
// code is Audrey Karperien, Charles Sturt University.
package FLAnalyzer;
  
import FLUtilities.FLutil;
import ij.IJ;
import ij.ImagePlus;
import ij.ImageStack;
import ij.gui.ImageWindow;
import ij.gui.PlotWindow; 
import ij.gui.PolygonRoi;
import ij.gui.Roi;
import ij.gui.StackWindow;
import ij.process.ImageProcessor;
import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
/**Generates colour-coded images from passed images and arrays
 * specifying rois or rectangles to colour with.
 * Some methods colour only foreground (usually black) pixels
 * in a specified roi of the image
 * using the corresponding colour for 
 * that area. Others colour over each
 * roi with translucent colour.
 * Call {@link #ColourRois ColourRois} 
 * to generate an image based
 * on the parameters passed.
 * The colour schemes are predetermined in the 
 * {@link #makeColourRule colour rule} 
 * and selected by the colourcode passed to
 * that method.
 */
public class FLGraphics
{
   
    /**Default constructor accepts no parameters.*/
    public FLGraphics (){}
    //////////////variables////////////////////
    ///**For colours of each subarea.*/
   // int r,g,b;
    /**The number of colour scales.*/
    public final static int NUMCOLOURCODES =21;
    /**value for main method*/
    public final static int MAKEALL=-9;
    /**Specifies type of drawing for subscans.*/
    public final static int 
    FILL_RECTANGLE=0;
    /**Specifies type of drawing for subscans.*/
//    public final static int  
//    DRAW_RECTANGLE_NO_STRING=1;
    /**Specifies type of drawing for subscans.*/
    public final static int 
    FILL_IRREGULAR_ROI=2;
    /**Specifies type of drawing for subscans.*/
    public final static int  
    REPLACE_FRGRND_IN_IRREG_ROI=3;
    /**Specifies type of drawing for subscans.*/
    public final static int 
    DRAW_RECTANGLE_WRITE_STRING=4;
    /**Specifies type of drawing for subscans.*/
    public final static int  
    DRAW_IRREG_ROI_WRITE_STRNG=5;
    /**Specifies type of drawing for subscans.*/
    public final static int 
    REPLACE_FRGRND_PIX_RECT_ROI=6;
    /**Specifies type of drawing for subscans.*/
    public final static int 
    DRAW_IRREGULAR_ROI_NO_STRING=7;
    /**Specifies type of drawing for subscans.*/
    public final static int 
    NO_COLOUR=8;
    float [] UserColourdfs = FLVars.DefaultColours;
    
    
/**
* Generates a colour determined by the passed integers A and K. A 
* is an integer from 1 to 20, that determines which group of colours
* to use from within one of 20 preset spectra.  Any other value is 
* corrected to a default colour.  K is a value that determines where 
* the returned colour falls in the spectrum specified by A.   In 
* fractal analyses, K is the fractal dimension.  It is assumed that K is 
* never greater than 3 or less than 0.  The value is divided by 3 to 
* map it onto the colour scheme.
* @return Color to draw with that corresponds to A and K
*  @param K float for the fractal dimension
*  @param A int for preset spectrum to use (from 1 to 20)*/
public Color makeColourRule (float K,  int A)
    //a = colour code, K the colour
{
        float n=.84f, m=.137f, r=0f, g=0f;        
        //k has to be converted to a value between 0 and 1
        //K has a value from 0 to 3      
        float maxdf=3f;
        K=K/maxdf;
        float max = 1.0f;
        boolean over= (K>max||K<0f)?true:false;
        if (over)K=max;
        float OriginalK=K;
        float b=K;
        float N=50f;
        float inc = max/N;
        float rinc = max/N;
        float ginc = max/(N+9f);                   
        float [] rs = new float [(int)N];
        rs[0]=max;
        float [] gs = new float [(int)N];
        gs[0]=0f;
        for (int i = 1; i < rs.length; i++)
        { 
            rs[i]=rs[i-1]-rinc;//System.out.println("r "+rs[i]);
            gs[i] = (gs[i-1]+ginc);
        }
        if(A==9||A==5||A==15){K=3f*K-1;
        if (K<0f)K=0f; if (K>1f)K=1f;}
        int j=0;
         for (float i =max; i>=0f&&j<rs.length; i=i-inc)
         {
            if(K>i)
            {
                r=rs[j];
                g=gs[j];              
                i=0f;
            }
            j++;
         }
        
        Color C= new Color (r,g,b);

        if(A==0)
        {     if (over)C=Color.black;
              else {
            if(K<1f/3f){C=SimpleGradient(Color.cyan, Color.green, 
                    3f*K);}
            else if (K>2f/3f){C=
                    SimpleGradient(Color.magenta.darker(), Color.yellow,
                    K=3f*K-2);}
            else {C=
                    SimpleGradient(Color.green.darker(), Color.magenta,
                    K*3f-1);}
        }}
        if(A==1)
        {       
             if (over)C=Color.black;
             else{
            if(K<1f/3f){C=
                    SimpleGradient(new Color(17, 92, 92), Color.green, 
                    3f*K);}
            else if (K>2f/3f){C=
                    SimpleGradient(new Color(223, 7, 183), Color.yellow,
                    K=3f*K-2);}
            else {C=
                    SimpleGradient(new Color(5,128, 185), 
                    Color.magenta.darker(),
                    K*3f-1);}
             }
        }
        if (A==2)        
        {   if (over)C=Color.black;
         else {   
            if(K<1f/3f){C=
                    SimpleGradient(new Color(27, 139, 35), Color.green, 
                    3f*K);}
            else if (K>2f/3f){C=
                    SimpleGradient(Color.orange, Color.blue,
                    K=3f*K-2);}
            else {C=SimpleGradient(Color.pink, Color.magenta.darker(),
                    K*3f-1);}
        }}
        if (A==3) 
        {     if (over)C=Color.white;
         else{  
            if(K<1f/3f){C=SimpleGradient(Color.cyan, Color.blue, 
                    3f*K);}
            else if (K>2f/3f){C=
                    SimpleGradient(Color.magenta, 
                    Color.red.darker().darker(), 
                    K=3f*K-2);}
            else {C=SimpleGradient(Color.yellow, Color.orange,
                    K*3f-1);}
         }  }
        if(A==4)
       {if (over)C=Color.red;
         else {C=SimpleGradient(Color.white, 
                 Color.black.brighter ().brighter ().brighter (), K);}}
        if (A==5){if (over)C=Color.red;
        else {
                    if (K>0f&&K<1f)  C=new Color (K, K, K);
                    else C=new Color(r,r,r);
        }}
        if (A==6){if (over)C=Color.red;
        else C= new Color (r, r, g+gs[gs.length-1]*K);}
        if (A==7) {if (over)C=Color.red;
         else C=SimpleGradient(Color.white, Color.black, K);}
         if (A==8) {if (over) C=Color.white;
         else{
             if (K<1f/12f)return 
                     SimpleGradient(Color.yellow,  Color.orange,
                      //new Color(253,160,40),
                     K*3f);
             else if (K<2f/12f)return 
                     SimpleGradient(new Color(255, 190, 4), 
                            new Color(134, 34, 9)
                    , 3f*K);       
            else if (K<3f/12f)return 
                     SimpleGradient(new Color(247, 251, 0), 
                    new Color(53,60,40), K*3f);     
              
            else if (K<4f/12f)return 
                     SimpleGradient(new Color(184, 182, 28),
                    new Color(55, 240, 22)
                     , 
                    K*3f);
            
            else if (K>=1f/3f&&K<=5f/12f)
                   return SimpleGradient(new Color(64, 225, 95),  
                    new Color(29, 61, 59), K*3f-1f);
            else if (K>1f/3f&&K<=6f/12f)
                   return SimpleGradient(new Color(14, 199, 50), 
                           new Color(5, 72, 1), K*3f-1f);
               else if (K>1f/3f&&K<=7f/12f)
                   return SimpleGradient(Color.cyan.brighter(), 
                           Color.blue, K*3f-1f);
            else if (K>1f/3f&&K<=8f/12f)
                return SimpleGradient(
                        new Color(97, 185, 239), 
                           new Color(8,71,180), 
//                        new Color(24, 159, 150),
//                      new Color(83, 70, 82), 
                           K*3f-1f);
            
            
                else if (K>2f/3f&&K<9f/12f)
                    return SimpleGradient(Color.pink, 
                      new Color(83, 70, 182), 3f*K-2f);
                else if (K>2f/3f&&K<10f/12f)
                    return SimpleGradient(Color.magenta, 
                            Color.pink.darker().darker().darker(), K*3f-2f);
            else if (K>2f/3f&&K<11f/12f)
                    return SimpleGradient(Color.magenta, 
                            Color.blue.darker().darker().darker(), K*3f-2f);
                else if (K>2f/3f)
             return SimpleGradient(//new Color(247, 19, 190), 
                   Color.magenta,
                     //new Color(24, 159, 150),
                      new Color(83, 70, 82).darker(), K*3f-2f);
//                  else if (K>2f/3f)
//                return  SimpleGradient(new Color(224, 15, 50), 
//                           new Color(43,125,210), 3f*K-2f);
            }}
        if (A==9) {if (over)C=Color.red;
           else {
            if (K<=0f)C= Color.yellow;
            else if (K>=1f)C= new Color (r,g,g).brighter ();}}
        if (A==10){ if (over)C=Color.red;
        else C= SimpleGradient(Color.black, Color.white, K);}
        if (A==11) {if (over)C=Color.gray;
        else C=Set11(maxdf, K, inc);}
        if(A==12) {     if(over)C=Color.GREEN;
              else{C=SimpleGradient(
                   Color.yellow, Color.yellow.darker(), K);}}
        if (A==13) {if (over)C=Color.black; 
        else C=new Color (b,b/2f,g);}
        if (A==14){ {K=3f*K-1;
        if (K<0f)K=0f; if (K>1f)K=1f;}
        if (over)C=Color.red;
          else{            
            if (K<=0f)C=new Color (b,r,r);
            else if (K>=1f) C=new Color (b,g,r);
            if(K>0f&&K<1f) C=SimpleGradient(Color.blue, Color.red, K);
          }}
        if (A==15){if (over)C=Color.white; 
            else if(K>0f&&K<1f)
                C=SimpleGradient(Color.red, Color.green, K);
            else C = new Color(g,g,g);
        }
        if(A==16){     if(over)C=Color.GREEN;
              else{
            if(K<1f/3f) 
            {C=SimpleGradient( Color.green, 
                     Color.green.darker (), 3f*K);}
            else if (K>2f/3f)
            {C=SimpleGradient(Color.gray, 
                     Color.black.brighter ().brighter (), 
                    K*3f-2);}
            else {C=SimpleGradient(Color.cyan, 
                    Color.magenta, K*3f-1);}
              }}
        if (A==17) { if (over)C=Color.red;
          else C = SimpleGradient(new Color(242, 157,247), 
                  new  Color(2, 253, 107), K);}
        if (A==18) {if (over)C=Color.red;
        else C=new Color (g,b,b).brighter().brighter();}
        if (A==19) if (over)C=Color.red;else C=new Color (g,b,g);
        if (A==20) {if (over) C=Color.blue;
         else{
            if (K<1f/6f)return SimpleGradient(Color.yellow, Color.green
                    , 3f*K);
            else if (K<2f/6f)return 
                    SimpleGradient(Color.yellow, 
                    Color.green.brighter(), 3f*K);
               else if (K<3f/6f)return
                    SimpleGradient(new Color(19, 206, 141), 
                       new Color(3,29,21), K*3f-1f);
                else if (K<4f/6f)return 
                    SimpleGradient(new Color(255, 190, 4), 
                        Color.red.darker().darker (), 3f*K-1f);
                else if (K<5f/6f)return
                    SimpleGradient(Color.magenta, 
                        Color.blue.darker().darker().darker(), K*3f-2f);
                 else 
             return SimpleGradient(Color.cyan.brighter(), 
                     Color.blue, 3f*K-2f);
            }}
            return C;                
}

/**Returns the value along a colour gradient according to the start S
 * and finish F colours, at the point K, which is a percentage (i.e., 
 * k <1.0f and >0f).*/
Color SimpleGradient(Color S, Color F, float K) 
{
    Color Colour=null;   
    float Sred=S.getRed()/255f;
    float Fred=F.getRed()/255f;
    float Sgreen = S.getGreen()/255f;
    float Fgreen = F.getGreen()/255;
    float Sblue = S.getBlue()/255f;
    float Fblue = F.getBlue()/255f;
    float dred = -(Sred-Fred);
    float dgreen = -(Sgreen-Fgreen);
    float dblue = -(Sblue - Fblue);                  
    float R = Sred+K*dred;
    float G = Sgreen+ K*dgreen;
    float B = Sblue + K*dblue;
    Colour = new Color(R, G, B);
    return Colour;
}
      
Color ColourSwitch (int index)
{
        Color C = Color.white;
             switch(index+1)
             {
            case 11: C
                    =Color.pink.darker();
            break; case 10: C
                     =new Color(195f/255f, 77f/255f, 27f/255f);                    
            break; case 9: C
                    =new Color(32f/255f, 128f/255f, 13f/255f);            
            break; case 8: C
                     =new Color(20f/255f, 253f/255f, 20f/255f);
            break; case 7: C
                    =Color.yellow;
            break; case 6: C
                     =new Color(132f/255f, 28f/255f, 187f/255f);
             break; case 5: C
                    =new Color(161f/255f, 1f/255f, 163f/255f);
            break; case 4: C
                    =new Color(235f/255f, 111f/255f, 232f/255f);
//            break; case 0: C
//                    =new Color(210f/255f, 208f/255f, 223f/255f);
            break;  case 1: C
                    =new Color(73f/255f, 224f/255f, 255f/255f);
            break; case 2: C
                      =new Color(0f/255f,172f/255f, 250f/255f);           
             break; case 3: C
                     =new Color(138f/255f, 94f/255f, 255f/255f);                    
            break; 
                
//                default: 
//                C=new Color(210f/255f, 208f/255f, 223f/255f);
            }
   return C;
}
   
/**Sets the colour for scale 11.
* @param maxdf the maximum value to use
* @param K float to colour code
* @param inc the increment between
* @return Color a colour*/
public Color Set11 (float maxdf, float K, float inc)
{        
            int index = 0;
            
            for (int i = UserColourdfs.length-1; i>0; i--)
            {                
                if (K>UserColourdfs[i]/maxdf)
                {
                    index=i;
                    i =0;
                }
            }
            
            Color C=ColourSwitch(index);
            
            
            float StartOfInterval = UserColourdfs[index]/maxdf;           
            float EndOfInterval = index==UserColourdfs.length-1?
                maxdf:UserColourdfs[index+1]/maxdf;
            //go until the next user Df is reached
            //if the marker is not the end of the array
            

            float interval =EndOfInterval-StartOfInterval;         
            float maxbrighters = 10f;        
            float increment = interval/maxbrighters;
            float red=C.getRed ()/255f;
            float green = C.getGreen ()/255f;
            float blue =   C.getBlue ()/255f;
            
            for (float i = StartOfInterval; i<K;//
            i=i+increment)
            {
                if (red>inc)red=red-inc;
                else if (red < 1.0f-inc)red=red+inc;
                
                if (green>inc)green=green-inc;
                else if (green < 1.0f-inc)green=green+inc;
                
                if (blue>inc)blue=blue-inc;
                else if (blue < 1.0f-inc)blue=blue+inc;
            }
            C=new Color(red, green, blue);
            return C;
}    
  
   
/**Plots graphs for generalized dimension.*/
static void drawsGeneralizedDimensions (FLVars V, 
        float [][]GD, String title)
{  
       
        FracStats S = new FracStats (GD[FLMain.DQindex], 
                V.Qs, V.Qs.length); 
        String Title =  title  + "(slope="+FLutil.fnum (S.Pslope)
          + FLMain.rsq+"="+FLutil.fnum (S.Prsq)+")";
        float min=V.Qs[0], max=V.Qs[0],
                ymin=GD[FLMain.DQindex] [0], 
                ymax=GD[FLMain.DQindex] [0];
        for (int j = 0; j < V.Qs.length; j++)
        {
            min = Math.min (V.Qs[j], min);
            max = Math.max (V.Qs[j], max);
            ymin = Math.min (GD[FLMain.DQindex] [j], ymin);
            ymax = Math.max (GD[FLMain.DQindex] [j], ymax);
        }
        min = min - 1.0f;
        max = max + 1.0f;
        ymin = ymin - .2f;
        ymax = ymax + .2f;
        if (ymin<0);else ymin=0;
        if (ymax>3);else ymax=3;
        float [] x= {min, max};
        float[] y ={ymax, ymax};
        
        if (V.GraphQs)
        {
            PlotWindow P = new PlotWindow ("D(Q)vsQ", "Q", "D(Q)", 
                    V.Qs, GD[FLMain.DQindex]);        
            P.setLineWidth (1);
            P.addPoints (V.Qs, GD[FLMain.dDQindex], P.CIRCLE);
            P.setColor (Color.blue);
            P.addLabel (.7, .6, "Slope=" + FLutil.fnum (S.Pslope)
                + " Y-int="+FLutil.fnum (S.Pyint));
            P.addLabel (0,0, Title);
            P.draw ();
        }
        
        if(V.GraphAQ)
        {
            PlotWindow A = new PlotWindow ( FLMain.alpha+" vs Q", 
                "Q",FLMain.alpha+"(Q)", V.Qs,GD[FLMain.ALPHAQindex]);
            A.setColor (Color.green);
            A.addLabel (0,0, Title);
            A.draw ();
        }
        
        if (V.GraphFQ)
        {
            PlotWindow F = new PlotWindow (FLMain.fat+"(" +
                 FLMain.alpha +  "(Q))vsQ", "Q", 
                 FLMain.fat+"(" + FLMain.alpha +"(Q))", V.Qs, 
                 GD[FLMain.FALPHAQindex]);
             F.setColor (Color.gray);
             F.addLabel (.3, .6, FLMain.fat+"(" + FLMain.alpha+")");
             F.addLabel(0,0, Title);
             F.draw ();
        }    
        S = new FracStats (GD[FLMain.dDQindex], V.Qs, V.Qs.length);
        Title = title+ "(slope="+FLutil.fnum (S.Pslope)
            + FLMain.rsq+"="+FLutil.fnum (S.Prsq)+")";
        ymin=GD[FLMain.dDQindex] [0];
        ymax=GD[FLMain.dDQindex] [0];
        for (int j = 0; j < V.Qs.length; j++)
        {
            ymin = Math.min (GD[FLMain.dDQindex] [j], ymin);
            ymax = Math.max (GD[FLMain.dDQindex] [j]  , ymax);
        }
        ymin = ymin - 2;
        ymax = ymax + 2;
       
        if(V.GraphT)
        {
            PlotWindow xP = new PlotWindow (FLMain.tau,
            "Q", "Ln" + FLMain.sum + "[P^Q]/Ln[1/" + FLMain.epsilon +
                    "]", new float[] {0, 0}, new float[] {0,0});
            xP.setLimits (min, max, ymin, ymax);
            xP.setLineWidth (1);
            xP.setColor (Color.black);
            xP.addPoints (V.Qs, GD[FLMain.dDQindex], xP.LINE);
            xP.addLabel (.5, .6, 
                    FLMain.tau+" (slope="+FLutil.fnum (S.Pslope)+")");
            xP.addLabel(0,0, Title);
            xP.setLineWidth (1);
            xP.setColor (Color.red);
            xP.addPoints (V.Qs, GD[FLMain.DQindex]
                    , xP.LINE);
            xP.addLabel (.5, .8, "D(Q)");
            xP.draw ();
        }        
}
        
/**Draws differently coloured squares on the passed Graphics2D, as 
* specified using the passed parameters.  The colours depend on the
* LocalAlpha, and ColourCode parameters. The way the image is 
* coloured depends on TYPE, which has to be an instance of an 
* FLGraphics static int or else nothing is done to the image.  
* Nothing is returned.  To see the image it must be shown.
* @param isgray boolean make it true to tell 
* the function this is grayscale
* @param ip ImageProcessor to draw on
* @param TYPE int for the type of colouring to do, based on static 
* ints in FLGraphics
* @param g2 Graphics2D on which to draw the colours
* @param rois array of ij.gui.Roi's to colour or outline
* @param dfs float array of fractal dimensions or other values 
* coding the colour for a corresponding area.
* @param ColourCode int for the scheme to choose in the 
* makeColourRule
* @param makescaleimage boolean true makes a copy of all the 
* scale images
* @param LocalAlpha float for transparency
* @param comp int for the composite type; default is 5
* @param foreground int for foreground pixel colour, usually black*/   
public static void ColourRois (ImageProcessor ip, 
    int TYPE,
    boolean isgray,
    Graphics2D g2,
    Roi [] rois, 
    float[] dfs,
    int ColourCode, 
    float LocalAlpha, 
    int comp,
    int foreground,
    boolean makescaleimage    
   )
{        
        int blue=0;       
        FLGraphics r= new FLGraphics ();       
        Color [] gcolours = MakeColours(dfs, ColourCode);
        for (int i = 0; i < dfs.length; i++)
        {
                if(dfs[i]!=GUI.FL_.DUD)
                {
                    g2.setColor (gcolours[i]);            
                    g2.setComposite
                        (AlphaComposite.getInstance (getcomp (comp),
                        newAlpha(LocalAlpha)));
                    //colour the roi using the appropriate rule
                    if (TYPE==FILL_IRREGULAR_ROI)
                    {
                        g2.fillPolygon (rois[i].getPolygon ());
                    }
                    else if (TYPE==FILL_RECTANGLE)
                    {          
                        g2.fill(rois[i].getPolygon ());
                    }
                    else if (TYPE==DRAW_IRREG_ROI_WRITE_STRNG)
                    {          
                        g2.drawPolygon (rois[i].getPolygon ());
                        g2.setFont( new Font("Tahoma", 1, 10));
                        g2.drawString (FLutil.fnum (dfs[i], 2), 
                        (int)rois[i].getBounds ().getCenterX (),
                        (int)rois[i].getBounds ().getCenterY ());
                    }
                    else if (TYPE==DRAW_IRREGULAR_ROI_NO_STRING)
                    {          
                        g2.drawPolygon (rois[i].getPolygon ());
                    }
                    else if (TYPE==DRAW_RECTANGLE_WRITE_STRING)
                    {          
                        g2.drawRect
                        ((int)rois[i].getBounds().getX (), 
                         (int)rois[i].getBounds().getY (), 
                         (int)rois[i].getBounds ().getWidth (),
                         (int)rois[i].getBounds ().getHeight ());   
                        g2.setFont( new Font("Tahoma", 1, 10));
                        g2.drawString (FLutil.fnum (dfs[i], 2),
                            (int)rois[i].getBounds ().getCenterX (),
                            (int)rois[i].getBounds ().getCenterY ());
                    }
                    else if (TYPE==REPLACE_FRGRND_IN_IRREG_ROI)
                        ReplaceForegroundPixelsInIrregularRoi(rois[i], g2,
                            ip, foreground);
                    else if (TYPE==REPLACE_FRGRND_PIX_RECT_ROI)
                        ReplaceForegroundPixelsInRectangularRoi(
                        rois[i], g2, ip, foreground, isgray, gcolours[i], dfs[i]);
                }
        }
        if(makescaleimage)
            makescale(                    
                    foreground, 
                    false,  
                    Color.white, 
                    ColourCode, 
                    false,
                    LocalAlpha, 
                    comp);                     
}

static float newAlpha(float LocalAlpha)
{
        float floatLocalAlpha=(float)LocalAlpha/255.0f;
        floatLocalAlpha=floatLocalAlpha>1.0f?1.0f:floatLocalAlpha;
        floatLocalAlpha=floatLocalAlpha<0.0f?0.0f:floatLocalAlpha;
        return floatLocalAlpha;
}

/**Colours only the pixels that are the passed foreground value in 
* the passed roi on the passed image.
* @param isgray boolean true if this is a grayscale image
* @param C Color to use
* @param Df float used to draw on ip if if isgray is true
* @param roi Roi to colour
* @param g2 Graphics 2D on which to colour the roi; this is usually 
* from a BufferedImage copy of the original image that is being 
* coloured over.
* @param ip ImageProcessor on which to colour
* @param foreground int for foreground of passed image */    
public static void ReplaceForegroundPixelsInRectangularRoi(
        Roi roi, 
        Graphics2D g2,
        ImageProcessor ip,
        int foreground,
        boolean isgray,
        Color C,
        float Df)
{        
        int startx = (int)(roi.getBoundingRect().getX()); 
        int starty = (int)(roi.getBoundingRect().getY()); 

        for (int xx = startx; 
            xx< startx+(int)(roi.getBoundingRect().getWidth()); 
            xx++)
        {

            for (int yy = starty; 
                yy< starty+(int)(roi.getBoundingRect().getHeight()); 
                yy++)
            {
                //int B=null;
                 ip.setRoi (xx, yy, 1, 1);
                if (isgray)  
                {
                   // float B=ip.getPixelValue(xx, yy);//colour intensity*Df
                     //ip.putPixelValue (xx, yy,  (B* Df));//*B);
                     //g2.setColor(new Color((int)(B*Df)));//C.getRGB ()));
                     g2.fillRect (xx, yy, 1, 1);
                     //g2.setColor(C);
                }
                else
                {
                    int [] histogram=ip.getHistogram ();
                    if (histogram[foreground]>0)
                    g2.fillRect (xx, yy, 1, 1);
                }
                 //g2.fillRect (xx, yy, 1, 1);

            }                  
        }
}
    
/**
* Colours only the pixels that are the foreground colour and in the 
 * passed roi on the passed image.
* @param  roi Roi to colour
* @param  g2 Graphics2D on which to colour the roi
* @param  ip ImageProcessor to colour
* @param  foreground int for the foreground colour*/     
public static void ReplaceForegroundPixelsInIrregularRoi(
    Roi roi, 
    Graphics2D g2,
    ImageProcessor ip,
    int foreground)
{        
        int startx = (int)(roi.getBoundingRect().getX()); 
        int starty = (int)(roi.getBoundingRect().getY()); 
        for (int xx = startx; 
            xx< startx+(int)(roi.getBoundingRect().getWidth()); xx++)
        {                   
            for (int yy = starty; 
            yy< starty+(int)(roi.getBoundingRect().getHeight());yy++)
            {
                ip.setRoi (xx, yy, 1, 1);
                //Roi smallroi = ip.getRoi ();
                int [] histogram=ip.getHistogram ();
                if (histogram[foreground]>0)
                {                           
                   if(roi.contains(xx, yy))
                    g2.fillRect (xx, yy, 1, 1);
                }                  
            }
        }
}
public static float MaxDf=3f;
public static float increment = .025f;
    
/**Converts the passed integer to a value for the composite type.
 * @param comptype an int for the composite type; 5 is default
 * for AlphaComposite.SRC_OVER
 *<pre>
 *switch ( comptype )
    {
        case 0:
        {alph = AlphaComposite.SRC; break;}
        case 1:
        {alph = AlphaComposite.DST_IN; break;}
        case 2:
        {alph = AlphaComposite.DST_OUT; break;}
        case 3:
        {alph = AlphaComposite.DST_OVER; break;}
        case 4:
        {alph = AlphaComposite.SRC_IN; break;}
        case 5:
        {alph = AlphaComposite.SRC_OVER; break;}
        case 6:
        {alph = AlphaComposite.SRC_OUT; break;}
        case 7:
        {alph = AlphaComposite.DST; break;}
        case 8:
        {alph = AlphaComposite.DST_ATOP; break;}
        case 9:
        {alph = AlphaComposite.SRC_ATOP; break;}
        case 10:
        {alph = AlphaComposite.SRC; break;}
        case 11:
        {alph = AlphaComposite.DST; break;}
        default:
        {alph = AlphaComposite.CLEAR; break;}
 *</pre>
 * @return int the composite type to use*/
public static int getcomp (int comptype)
{
        int alph = 0;
        switch ( comptype )
        {
            case 0:
            {alph = AlphaComposite.SRC; break;}
            case 1:
            {alph = AlphaComposite.DST_IN; break;}
            case 2:
            {alph = AlphaComposite.DST_OUT; break;}
            case 3:
            {alph = AlphaComposite.DST_OVER; break;}
            case 4:
            {alph = AlphaComposite.SRC_IN; break;}
            case 5:
            {alph = AlphaComposite.SRC_OVER; break;}
            case 6:
            {alph = AlphaComposite.SRC_OUT; break;}
            case 7:     
            {alph = AlphaComposite.DST; break;}
            case 8: 
            {alph = AlphaComposite.DST_ATOP; break;}
            case 9:
            {alph = AlphaComposite.SRC_ATOP; break;}
            case 10:
            {alph = AlphaComposite.SRC; break;}
            case 11:
            {alph = AlphaComposite.DST; break;}
            default:
            {alph = AlphaComposite.CLEAR; break;}
        }
        return alph;
}
    
/**Makes an image showing the colour spectrum used to colour code
* an image coloured using the same parameters as those passed 
* here.  Invokes a {@link #makeColourRule colour rule}.   
* @param makeAllScales boolean true makes an image for all of the
* 21 colour schemes; this is a utility for generating the scales but is
* not used in the program itself.  The icons for the colour tree are 
* made using it.
* @param LocalAlpha float for transparency   
* @param foreground int foreground,
* @param UseParticleAnalyzer boolean UseParticleAnalyzer,
* @param bcol Color bcol,
* @param ColourCode int ColourCode*/
public static void makescale (   
    int foreground,
    boolean UseParticleAnalyzer, 
    Color bcol, 
    int ColourCode, 
    boolean makeAllScales,
    float LocalAlpha,
    int comp)
{
        if (ColourCode==MAKEALL) makeAllScales=true;
        float columnsdone=1f;
        int numberofscales=1;    
        if (makeAllScales)numberofscales=NUMCOLOURCODES;        
        for (int i = 0; i < numberofscales; i++)        
        {
        if(makeAllScales)ColourCode=i;
        FLGraphics flg = new FLGraphics();
        columnsdone=1f;
         boolean OverOne=false;
         int h=500, cwidth = 80;
         int numcolumns = 3;
         int w = numcolumns*cwidth;
         BufferedImage bis
         = new BufferedImage (w, h,
         BufferedImage.TYPE_INT_ARGB);
         //Create a buffered image with translucency
         Graphics2D g2 = (Graphics2D)bis.getGraphics ();
         
         Color backgroundc = Color.black;
         if (foreground==0)backgroundc=Color.white;
         if(UseParticleAnalyzer) backgroundc=bcol;         
         
         g2.setColor (backgroundc);
         //fill in with the background colour
         g2.fillRect (0,0, w, h);
         boolean done1column=false;
         int x = 0;//coordinate for writing bar of colour
         float m=0f, n=0f;
         int bar = 20; boolean skip=true;
         int wordx =bar, wy = 0;int here = 0;
         //loop through each value of k
         float [] dfarray = new float[(int)(MaxDf/increment)];
         int q = 0;
        for (float v = 0.00f; v<=3.00f; v=v+.025f)dfarray[q++]=v;
         Color [] gcolours = MakeColours(dfarray, ColourCode);
         for (int z=0; z<dfarray.length; z++)
         {             
                        
             if(dfarray[z]>=columnsdone&&done1column==false)
             {
                 done1column=true;
              columnsdone++;
              x=(int)(cwidth*(columnsdone-1)); 
              wordx=bar+x; here=0;
             }
             else done1column=false;
             //System.out.println(gcolours[z]);
             g2.setColor (gcolours[z]);
             g2.setComposite
                    (AlphaComposite.getInstance (getcomp (comp),
                    newAlpha(LocalAlpha)));   
             g2.fillRect (x, here, bar, h/40);
             if (skip)
             {g2.drawString (FLutil.fnum (dfarray[z],2), wordx, here);
              skip=false;}
             else skip=true;
             here = (int)(here + h/40);
         }
         ImagePlus scaleimg =
                 new ImagePlus ("Scale "+ColourCode, bis);
         ImageProcessor newip = scaleimg.getProcessor ();//scaleimg.s
         ImageWindow iw = new ImageWindow (scaleimg);
         // newimg.setWindow(iw);
         iw.setVisible (true);//show();
}}    
    
/**Returns an array of colours corresponding to the passed array of 
* values.
* @param D float array for determining the corresponding colour
* @param ColourCode int between 0 and 20 specifying the colour 
* scheme to use
* @return Color [] colours where each colour[i] maps to D[i]*/
public static Color  [] MakeColours (float [] D, int ColourCode )
{                
         FLGraphics flg = new FLGraphics(); 
         //loop through each value of k
         Color [] gcolours =new Color [D.length];
         for (int i = 0; i< D.length; i++)
         {             
             gcolours[i]=flg.makeColourRule (D[i], ColourCode);
         }
         return gcolours;
}
      
/**
 * 
 * @param args 
 */
public static void main(String [] args)
{
    int code =(int) IJ.getNumber ("Code", 11);
    makescale(
            255, 
            false, 
            Color.white,
            code,//12, 
            false, //true, 
            235, 5//, //DefaultDfs
            );

}       
/**Takes an array of images and makes them into a stack by
*padding any that are not the same size to the size of the largest.
* @param iparray array of ImageProcessors
* to convert to a stack
* @return StackWindow containing all the 
* iparray's ImageProcessors padded
* to the size of the largest ImageProcessor*/
public static StackWindow ShowPaddedStack
        (ImageProcessor [] iparray, boolean blackbackground, 
        String title)
{
         int width=iparray[0].getWidth ();
         int height = iparray[0].getHeight ();
         for (int i = 1; i < iparray.length; i++)
         {
             width=Math.max (iparray[i].getWidth (), width);
             height =Math.max (iparray[i].getHeight (), height);
         }
         
         ImageStack IS=new ImageStack (width, height);
         for (int i = 0; i < iparray.length; i++)
         {
             IS.addSlice ("",
                     FLResizer.expandImage (iparray[i], 
                     width, height, 0,0,
                     blackbackground?Color.black:
                             Color.white)
                     );
         }
         StackWindow iw =
                 new StackWindow (new ImagePlus ("newHC"+
                 title, IS));
         iw.setVisible (true);
         return iw;
 }
    
/**Creates a BufferedImage and calls {@link #DrawPolyandCirc 
 * method} to draw the convex hull and bounding 
* circle on it.  If thisslice is 1, creates a new instance of 
 * {@link  FLVars#IpArray}, then adds a new slice for 1 and 
 * each time it is called 
* then if thisslice=lastslice, shows the stack in a StackWindow by 
* calling {@link #ShowPaddedStack}. Thus, thisslice must be set to
* one on the first call or it will throw an exception unless the array is
* initialized elsewhere (if it is called only once, then lastslice must 
* also be set to 1). If aSub is true of isRectangleSubScan is true it 
* also adds a slice to the {@link FLVars#HCImageProcessors array}
 * of arrays of imageprocessors.
* @param isRectangleSubScan boolean true if is rect sub scan
* @param ThisRectangleSubScan int for this scan
* @param xytranslate distance to translate
* @param DrawCircle true draws the bounding circle
* @param DrawHull true draws the convex hull
* @param thisslice int for the current slice's index (use 1 if not a 
* stack)
* @param lastslice int for the number of slices  (use 1 if not a stack)
* @param drawing ImagePlus to copy
* @param poly PolygonRoi for convex hull
* @param circ float [] coordinates of circle and its diameter*/
public static void drawshapes (
     boolean DrawCircle,
     boolean DrawHull,
     int thisslice, 
     int lastslice,
     PolygonRoi poly,
     float [] circ,
     ImagePlus drawing,                         
     boolean isRectangleSubScan,
     int ThisRectangleSubScan,
     int [] xytranslate, 
     String title,
     FLVars V)
{         
         int width=drawing.getWidth ();
         int height = drawing.getHeight ();
         BufferedImage bi = new BufferedImage
                 ( width, height,  BufferedImage.TYPE_INT_ARGB);
         if (xytranslate==null)xytranslate = new int [] {0,0};
         Graphics2D g = (Graphics2D)bi.getGraphics ();
         g.drawImage ( drawing.getImage (), 0, 0, null);
         DrawPolyandCirc (DrawCircle, DrawHull,
                 g, poly, circ, xytranslate);         
         //if this is a subscan, store the image in an array of arrays
         ImagePlus img=
                 new ImagePlus ("HC"+V.TitleAndSlice, bi);
         if(isRectangleSubScan)
             V.HCImageProcessors[ThisRectangleSubScan][thisslice-1]
                     =img.getProcessor ();
         else
         {             
             if (thisslice==1)V.IpArray=new ImageProcessor[lastslice];
             V.IpArray[thisslice-1]=img.getProcessor ();
             if (thisslice==lastslice) 
                 FLGraphics.ShowPaddedStack (V.IpArray, 
                         V.background==V.BLACK, title);             
         }
}
/**Draws an oval specified by the coordinates in c, and the polygon 
* that is specified as roi.  The drawing method Graphics2D.data.
* drawOval(x, y, width, height) draws an oval that fits in the 
* rectangle specified by the coordinates using x,y for the top left 
* corner.  The coordinates are also rounded to integers before being
* passed to this method by the usual calling function, so the original 
* circle is somewhat modified from the more precise coordinates 
* {@link CircStats#measureHullandCircle calculated} earlier.
* @param xytranslate value for translating drawing
* @param DrawCircle true draws the circle
* @param DrawHull true draws the hull
* @param g2 Graphics2D where the drawing will occur
* @param roi convex hull to draw
* @param c float [] of x, y for top left corner and width (diameter) of 
* rectangle in which to draw the circle*/
public static void DrawPolyandCirc (
             boolean DrawCircle,
             boolean DrawHull,            
             Graphics2D g2,
             Roi roi,
             float [] c,
             int [] xytranslate
             )
{
         BasicStroke roundStroke = new BasicStroke (0f, 
                 BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER);
         int i = 0;
         g2.setColor (Color.red);
         g2.setStroke (roundStroke);           
         if(DrawHull&&roi!=null)
         {
             g2.translate (xytranslate[0], xytranslate[1]);
             g2.drawPolygon (roi.getPolygon ());
             g2.translate (-xytranslate[0], -xytranslate[1]);
         }             
         if(DrawCircle&&c!=null)
         {
           g2.translate (xytranslate[0], xytranslate[1]);
           g2.translate (1, 1);
           g2.drawOval ((int)c[0], (int)c[1], (int)c[2], (int)c[2]);
           g2.translate (-1, -1);
           g2.translate (-xytranslate[0], -xytranslate[1]);
         }        
 }
     
     
}
