// Notice
//
// The contents of this file are subject to the
// same license as the source code for ImageJ.
// FracLac is in the public domain.
// If you use it to publish research, please cite the source.
// The initial developer of the original
// code is Audrey Karperien, Charles Sturt University.
package FLAnalyzer;
import FLUtilities.FLstatistics;
import FLUtilities.FLutil;
import ij.IJ;
import ij.ImagePlus;
import ij.ImageStack;
import ij.gui.*;
import ij.process.ImageProcessor;
import ij.process.ImageStatistics;
import ij.text.TextWindow;
import java.awt.*;
import java.util.Random;
import javax.swing.JOptionPane;
/**Contains main working methods for FracLac, a plugin for ImageJ.
 *@see GUI.FL_
* @version 2.4e FracLac_ May 2006
* @version 1.37g ImageJ
* @author Audrey Karperien (akarpe01@postoffice.csu.edu.au)
* @author Thomas R. Roy, University of Alberta, Canada
* (convex hull)
* @author ImageJ code and borrowed code from Wayne Rasband of
* the NIH
* @author H.F. Jelinek, Charles Sturt University, Australia
* (consultant and developer)
* @since jdk 1.5.0_04 */
public class FLMain
{  
    public static final char [] e = {(char)949};
    public static final String epsilon=new String(e);
    public static final char []a = {(char)945};
    public static final String alpha=new String(a);
    public static final char []l = {(char)955};
    public static final String lambda=new String(l);
    public static final char []L = {(char)923};
    public static final String Lambda=new String(L);
    public static final char []m = {(char)956};
    public static final String mu=new String(m);
    public static final char []s = {(char)963};
    public static final String sigma=new String(s);
    public static final char []summation ={ (char)8721};
    public static final String sum=new String(summation);
    public static final char []t = {(char)964};
    public static final String tau=new String(t);
    public static final char []squared = {(char)178};
    public static final String sq=new String(squared);
    public static final String rsq="r"+sq;
    public static String cvsq =  "("+sigma+"/"+mu+")"+sq;
    public static String cv = "("+sigma+"/"+mu+")";
    public static final char [] effat ={(char)402};
    public static final String fat=new String (effat);
    public static final String falpha =fat+"("+alpha+")";
    public static final String bpd = "Binned Probability Distribution",
    bs ="Box Size", meanprob="Mean Probability", nc = 
            "not calculated";
    Color [] LacColors =
    {Color.blue, Color.green, Color.magenta, Color.orange};
    int colorcounter = 0;
/**
 * String for the &Lambda; fractal dimension = slope[ln(&sigma;)
 * /ln(&epsilon;)]-(slope[ln(&lambda;)/ln(&epsilon;)])/2*/
     public String LD="(slope[ln"+sigma+"/ln"+epsilon+"])" +
                    "-((slope[ln("+lambda+")/ln("+epsilon+")])/2)";
     public static int dDQindex=1, DQindex=0, TAUindex=2,
             ALPHAQindex=3,  FALPHAQindex=4;    
    boolean changedSeries=false;
    float bestCV=0f;
    int bestLOC=0;
    float [][] bestGD=null;  
/**
* An instance of a class for storing user variables for 
* running the scans.*/
    public FLVars V = new FLVars ();
/**
* Instance that holds all of the working data for each 
* scan.*/
    public FLDataProcessor D=null;                      
//This is all for showing the grids in a graphic
    ImageStack GridStack = null;  
    Image GRIDIMAGEimg= null;
    ImagePlus GRIDIMAGEipl = null;
    ImageProcessor GRIDIMAGEipr =  null;
/**Holds the original raw data from each scan for each 
* box at each box size (&epsilon;).  Data are the 
* number of pixels for binary images 
* or else intensity for gray scale images.  Intensity is 
* max-min+1 for grayscale scans using the 
* {@link FLVars#DIFFERENTIALBC differential box counting}
*  method, or else (max-min)*boxsize<sup>2</sup> for the 
* {@link FLVars#VARIATION2DBC 2 d variation}.   
* The data are crunched in the
* {@link FLAnalyzer.FLDataProcessor data} class.*/
    public float [][][]Pix = null;
/**Holds data temporarily for each box count*/
    float []PPBr = null; 
    float [][] Empties=null;  
/**Array of image filename and roi location information.*/
    public String [] Titles = null;  
/**Current rectangle enclosing the image, that is
* reset in {@link #FindMargins margins} method.*/
    public Rectangle RectEnclosingForeground;    
    boolean debug=false; 
/**Array of rois for scanning images.
* @see GUI.FL_#DrawSingleRectangleDrawings
* @see GUI.FL_#OpenAndScanFilesWithoutShowingThem
* @see GUI.FL_#SetUpOneRectangularRoi
* @see GUI.FL_#ScanOneRectangleRoi
* @see GUI.FL_#DrawManyRectangles
* @see GUI.FL_#SetUpEachRectanglesRoi
* @see GUI.FL_#DrawParticles
* @see GUI.FL_#SetUpParticles
* @see GUI.FL_#ScanRandomRectangleSubscans
* @see GUI.FL_#SetUpRectangleSubscans
* @see GUI.FL_#DoEntireImage
* @see GUI.FL_#DoIrregularRoi
* @see GUI.FL_#DoARectangularRoi*/
    public Roi [] Rois = null;    
    static String tab = "\t", newline = "\n";
/**The array of box sizes used to {@link #StartBoxCounts scan}
* an image,  which is set with each new image or image ROI, in the
* {@link FLAnalyzer.FLMain#MakeNewBoxSizes new size method}.
 * The series of sizes is calculated according to the user {@link 
 * FLVars#Series option}.
* see #Epsilons*/
    public int[][] BOXSIZES;   
/**Array of &epsilon; ({@link #BOXSIZES box size}
* /{link #V.GreaterOfHtAndWd image dimension}
* used in scans, which is set with each new image or image ROI, in 
 * the {@link FLAnalyzer.FLMain#MakeNewBoxSizes new size 
 * method}. */
    public float[] [] Epsilons;  
/**Array of random origins for  
* {@link FLVars#RandomMassSample random mass scans}.*/
    public int [][] RMxy = null;   
    int empties;    
    float progress = 0;     
/**Array for x and y coordinates for rois using particle analyzer.*/
    public float [] wandx;
/**Array for x and y coordinates for rois using particle analyzer.*/
    public float [] wandy; 
    public String [] MFheadings={
    "Q: I(Q,"+epsilon+")="+sum+"[Pi^Q]; "+sum+"Pi=1; " +
            "Pi=i at "+epsilon+"="+mu+"=Pi^Q/"+sum+"[Pi^Q]",            
    "D(Q)="+tau+"/(1-Q)", 
    tau + "=lim[ln(I(Q," +epsilon+ "))/ln(1/" +epsilon+ ")]",
            "mean "+tau,    
    alpha+"="+sum+"["+mu+"i*Pi]/ln"+epsilon,
    falpha+"="+sum+"["+mu+"*ln"+mu+"]/ln "+epsilon};   
 //                END VARIABLE DECLARATIONS                 //
    
 //===============================// 
 //                                 METHODS                                         //
 //===============================//
/**Constructor that accepts no parameters*/
 public FLMain ()   {     }      
     
/**Draws the passed polygon (convex hull) and bounding circle
* on the passed graphics, using the values to translate.* 
* @param DrawCircle boolean true draws the bounding circle
* @param DrawHull boolean true draws the convex hull   
* @param roi PolygonRoi to draw
* @param c float [][] of convex hull points
* @param xytranslate int[][] for distances to translate 
* @param g Graphics2D to draw on */
 public void drawshapesArrays (
         boolean DrawCircle,
         boolean DrawHull,           
         PolygonRoi []roi,
         float [][]c,
         int[][] xytranslate,
         Graphics2D g )
 {         
     BasicStroke roundStroke = new BasicStroke (0f, BasicStroke.
             CAP_BUTT,BasicStroke.JOIN_MITER);         
     g.setColor (Color.red);
     g.setStroke (roundStroke);         
     for (int i = 0; i < roi.length; i++)
     {             
         if(DrawHull&&roi[i]!=null)
         {
             g.translate (xytranslate[0][i], xytranslate[1][i]);
             g.drawPolygon (roi[i].getPolygon ());
             g.translate (-1*xytranslate[0][i],- 1*xytranslate[1][i]);
         }             
         if(DrawCircle&&c[i]!=null)
         {
             g.translate (xytranslate[0][i], xytranslate[1][i]);
             g.translate (1, 1);
             g.drawOval ((int)c[i][0], (int)c[i][1], (int)c[i][2], (int)c[i][2]);
             g.translate (-1*xytranslate[0][i], -1*xytranslate[1][i]);
             g.translate (-1, -1);
         }             
     }         
} 
/**Returns the highest index for which the value is greater than 0
* @param x int [] to assess
* @return int that is the index representing the highest index 
* with a value above 0 or returns -9999999 if no value is >0*/
public static int MaxIndex(int[] x)
{
    boolean allzero=true;
    int maxIndex =-999;
    for (int i = x.length-1; i>=0; i--)
        if (x[i]>0){maxIndex=i; i=-1;allzero=false;}        
    if(allzero) {IJ.error("No value"); return -999;}
    else return maxIndex;
}

/**Returns the lowest index for which the value is 
* greater than 0 or else -1 if no value is greater than 0
* @param x int[] to assess
* @return int that is the smallest index or else -6 if there is no
* index with a value >0*/
public static int MinPositiveIndex(int[] x)
{
    boolean nopixels=true;
    int index = -6;
    for (int i = 0; i<x.length; i++)
    {
        if (x[i]<0){IJ.error ("Negative Value");}
        if (x[i]>0) {index = i; i = x.length; nopixels=false;}            
    }
    if (nopixels){IJ.error ("No Value");return -6;}
    return index;
}

/**Directs global and sub scans. Calls a function to {@link 
* #ControlScansForEachLocation control scans} for each {@link 
* FLVars#GridPositions grid position}. Returns false, sets the 
* {@link FLVars#abort abort switch}, and shows a message if there
* is no image passed. Returns the value from the call to the {@link 
* #ControlScansForEachLocation controller} otherwise. Stores 
* images for {@link FLVars#originaldrawing drawing}.
 * Calls {@link #SetImage margin set up method}. 
* @param XOfAreaBeingChecked x coordinate of scan
* @param YOfAreaBeingChecked y coordinate of scan
* @param FullWidthOfAreaBeingChecked widht of scan
* @param FullHeightOfAreaBeingChecked height of scan
* @param thisslice int for the current slice
* @param lastslice int for number of slices to scan
* @param isRectangleSubScan boolean true if this is a rect sub scan
* @param ThisRectangleSubScan int for the current subscan
* @param title String for the image title
* @param img ImagePlus to process
* @return boolean */
public boolean SendImage (
     String title,
     ImagePlus img,
     int XOfAreaBeingChecked,
     int YOfAreaBeingChecked,
     int FullWidthOfAreaBeingChecked,
     int FullHeightOfAreaBeingChecked,
     int thisslice,
     int lastslice,                        
     boolean isRectangleSubScan,
     //is false only if from ScanEntireImage then SetUpScans
     int ThisRectangleSubScan)
{       
     IJ.showStatus ("Preparing "+V.TitleAndSlice);
     progress=0;
     ImageProcessor ip = img.getProcessor ();    
     if (ip!=null)
     {
         V.originaldrawing=
                 new ImagePlus ("oDrawing"+V.TitleAndSlice, ip);
         V.originaldrawing.setProcessor("oDrawing"+V.TitleAndSlice,ip);
         ip = SetImage (title, img); 
         if (ip==null||V.abort)
         {
             V.abort=true;
             IJ.showMessage ("error in processing"); 
             return false;
         }
         else if (!V.isAbort ())
         {    
              
             //                    Analyze the entire roi or image                 /       
                boolean scanned= ControlScansForEachLocation (                             
                         ip,
                         XOfAreaBeingChecked,
                         YOfAreaBeingChecked,
                         FullWidthOfAreaBeingChecked,
                         FullHeightOfAreaBeingChecked,
                         thisslice, lastslice,
                         RectEnclosingForeground.x,//will be scan origins
                         RectEnclosingForeground.y,//will be scan origins  
                         isRectangleSubScan,
                         ThisRectangleSubScan);   
                 return scanned;
             }        
     }
     else {IJ.showMessage ("No Image"); V.setAbort (true);}
     return false;//end if there is no image
}  
     
/**Resets total pixels value and tests for binary.  Shows a 
* message if the {@link FLVars#getImageType image type}
* is supposed to be binary but the image is not, and switches
* the {@link FLVars#abort} warning to true.
* @param stats ImageStatistics to assess
* @see FLVars#TotalAllPixels*/
     public void testtype(ImageStatistics stats)
     {        
         V.TotalAllPixels = stats.histogram[255]+stats.histogram[0];
         if (V.isGray())V.TotalAllPixels=stats.pixelCount;         
         if (V.TotalAllPixels!=stats.pixelCount &&
                 (V.getImageType()==V.BINARY))
         {
             IJ.showMessage ("Binary image required");
             V.abort=true;  
         }
     }
   
/**Makes new boxsizes and finds margins and prepares the image 
 * to be analyzed or returns null if no foreground pixels are found on
 * a binary image scan or if any of the calls change the {@link 
 * FLVars#abort abort switch}.   Calls {@link #testtype tester} and
 * records {@link FLVars#TotalAllPixels total pixels}, {@link 
 * FLVars#TotalForegroundPixels foreground pixels},and PixelRatio
 * . <p>Kills any lefover roi and calls a method to find {@link 
 * #FindMargins binary margins} or {@link #FindGrayMargins 
 * grayscale image margins}. Calls for new {@link 
 * FLAnalyzer.FLMain#MakeNewBoxSizes box sizes} using the value
 * in {@link FLVars#LesserOfHtAndWd} or 2 for skinny images.  
 * Sets the border and makes a new ImageProcessor by calling 
 * {@link FLAnalyzer.FLResizer#expandImage expander} then 
 * again resets margin values and kills the roi and sets drawing 
 * variable to the current image.</p>
 * @return ImageProcessor ready to work on
*  @param  Title String for name of image
*  @param  img ImagePlus for preparation scan of subareas rather
 * than the entire roi or image */
public ImageProcessor SetImage (String Title, ImagePlus img)
{
     ImageProcessor thisip=img.getProcessor ();   
     ImageStatistics stats = img.getStatistics ();
     //----------------------------------------------------------------------------------- /
     //              TEST for BINARINESS and RECORD SOME              /
     // STATISTICS ABOUT THE PIXELS IN THE WHOLE IMAGE  /
     //------------------------------------------------------------------------------------/
     testtype(stats);         
     if (!V.abort)
     {
         V.TotalForegroundPixels=stats.histogram[V.getForeground ()];
         V.PixelRatio =V.TotalForegroundPixels/V.TotalAllPixels;
         img.killRoi ();
         if (V.isGray()) V.Margins=FindGrayMargins(thisip,
                 thisip.getWidth (), thisip.getHeight ());
         else V.Margins=FindMargins (thisip);
         if(V.Margins==null)
         {
             V.abort=true; 
             return null;
         }
         else V.abort=false;
         int margright=V.Margins[1];
         int margleft=V.Margins[0];
         int margtop=V.Margins[2];
         int margbottom=V.Margins[3];
        
         if (!V.isAbort ())
         {//---------------------------------------------------------------------------------/
            //                                 GET NEW BOXSIZES                             /
             //------------------------------------------------------------------------------/
             if(V.LesserOfHtAndWd<2) MakeNewBoxSizes (2);
             else  MakeNewBoxSizes (V.LesserOfHtAndWd);                             
             if (V.isAbort ()) return null;
             
             //                Make a border and expand the image                /
             //                                                                                                 /
             float mx = V.maxpercent/100.00f;                 
             V.border = (int)Math.max (mx*(float)(margright-margleft),        
                     mx*(float)(margbottom-margtop));        
             if (V.isGray ())
                 V.border=BOXSIZES[0][BOXSIZES[0].length-1];               
             int wOld = thisip.getWidth ();
             int hOld =thisip.getHeight ();
             int wNew = wOld + V.border;
             int hNew = hOld + V.border;
             V.xOffset = (wNew - wOld)/2;// offset for centering
             V.yOffset = (hNew - hOld)/2; // offset for centering
             ImageProcessor newIP = 
                     FLResizer.expandImage (thisip, wNew, hNew, V.xOffset,
                     V.yOffset, V.isGray() ? V.filler : 
                     V.getBackground()==V.BLACK ? Color.black :
                     Color.white);
             thisip = newIP;                 
            
             //-----------------------------------------------------------------------------/
             //                           Record where the margins are                /
             //-----------------------------------------------------------------------------/
              V.Margins=null;
              if(V.isGray())
                 V.Margins = FindGrayMargins(thisip, wOld, hOld);
             else  V.Margins =FindMargins (thisip);
             if (V.Margins==null)
             {
                 V.abort=true;
                 return null;
             }
         }
 }
 return thisip;
}
     

/**Resets the arrays for data gathering.
@see #Pix
@see #Empties
@see #Titles*/
void InitializeArrays()
{
    Pix =  new float [V.GridPositions][BOXSIZES[0].length][];
    Empties =  new float [V.GridPositions][BOXSIZES[0].length];
    Titles = new String [V.GridPositions];
}  
     
void DoHCFor1Grid(int RoiX, int RoiY, int RoiWidth, int RoiHeight)
{
    V.ShowData=false;
    V.WriteRawData=false;    
    String s= V.TitleAndSlice+" ("+RoiX+","+RoiY+":"+
                    RoiWidth+"x"+RoiHeight+") ";
    V.BCdata=V.BCdata+s+tab;
        for (int i = 0; i<V.boxcountheadings.length-1; i++)
            V.BCdata=V.BCdata+nc+tab;
        V.BCdata=V.BCdata+HullandCircleDataWriter ()+newline;  
}

/**Controls scans of the passed ImageProcessor and analysis of the
* scan data.  Uses values previously stored for {@link #BOXSIZES 
* box sizes} (&epsilon;s) and {@link 
* FLAnalyzer.FLVars#GridPositions scan locations}, and calculates
* {@link FLAnalyzer.FracStats#FindDfStats the fractal dimension}
* and other statistics on the data gathered. Returns true if 
* successful, or else false if a null value is returned by the {@link 
* #StartBoxCounts counter}.  <LI>Initializes {@link 
* #InitializeArrays arrays}, and makes {@link #GetXY random}
* x,y pairs for scan origins after the first scan which is done at the 
* passed (x,y) (also makes {@link #MakeRandomMassLocations 
* origins for random scans}. <LI>Calls the {@link #StartBoxCounts
 * counter}. <LI>Stores the image name and scan origins in 
 * {@link #Titles the titles array}.<LI>After the scan, calls the 
 * {@link FLAnalyzer.FLDataProcessor data processor} to analyze 
 * the data and calculate statistics. <LI>Stores information for 
 * colour coding in the {@link FLVars#colourarray colour array} 
 * and for printing the processed data in 
 * {@link #WriteDataStringsForAllLocations  strings}. 
* @see #InitializeArrays
* @param RoiX x coordinate of roi
* @param RoiY y coordinate of roi
* @param RoiWidth width of area being scanned
* @param RoiHeight height of area being scanned
* @param thisslice int for current slice
* @param lastslice int for total slices to process
* @param isRectangleSubScan boolean true if rect subscan
* @param ThisRectangleSubScan int for this scan
* @param ipk ImageProcessor ipk is analyzed
* @param x integer for x coordinate to start at
* @param y integer for y coordinate to start at
* @return boolean true if there are no errors */
public boolean ControlScansForEachLocation (                            
ImageProcessor ipk,
int RoiX,
int RoiY,
int RoiWidth,
int RoiHeight,
int thisslice,
int lastslice,
int x,
int y,                        
boolean isRectangleSubScan,
int ThisRectangleSubScan)
{
    GetCircAndHullStats(ipk, thisslice, lastslice, 
                ThisRectangleSubScan,  isRectangleSubScan); 
    if(V.GridPositions==0)
    {  
        DoHCFor1Grid(RoiX, RoiY, RoiWidth, RoiHeight);           
        return true;
    }
    V.BoxCountHeadings
            =FLVars.MakeHeadings(V.boxcountheadings);
   if(V.isDocircul ())
       V.BoxCountHeadings = V.BoxCountHeadings+tab+
            FLVars.MakeHeadings(V.HullAndCircleDataHeadings);
    //          initialize variables and arrays for scans                  
    InitializeArrays();  
    int OriginalX = x, OriginalY=y;
    //                                                                                                 
    //  Make random x,y for each location for multiple origins
    //                                                                                                 
    int [][]XY = GetXY(x, y, ipk);
    //                                                                                               
    //              Do a complete scan for each origin                    
    //                                                                                               
    if(!V.abort)
    {
        for  (int LOC = 0; LOC<V.GridPositions; LOC++)
        { 
            if(LOC>0)
            {//don't use randoms if this is a block ?or 1st locationFIXME  
                if(V.BLOCKTEXTUREANALYSIS)
                {
                    x = OriginalX+XY[LOC][0];
                    y = OriginalY+XY[LOC][1];
                }
                else
                {
                    x = OriginalX-XY[LOC][0];
                    y = OriginalY-XY[LOC][1];
                }
            }
            //   Initialize random mass scan origins on first slice            
            if(thisslice==1&&V.RandomMassSample)
            {
                RMxy = MakeRandomMassLocations(ipk);
                if(V.subsamples<1)
                {
                   IJ.error ("Error. Pixel ratio is set too high for this image.");  
                   V.abort=true; 
                   return false;
                }
            }                         
            //                                                                                         
            //    get box count data  and store it in global arrays
            //               once for each grid position                      
            //                                                                                         
            boolean worked = StartBoxCounts (ipk, x, y, LOC );
            if(!worked)
            {
                V.setAbort(true);
                return false;
            }
            //                                                                                                 /
            //       store image info and scan origins in a string             
            //                                                                                                
            String ScanDimensions=(V.StartX-V.xOffset)
                +","+(V.StartY-V.yOffset)+"--"+(V.EndX-V.xOffset)
                +","+(V.EndY-V.yOffset)+ " ";
            Titles[LOC]= V.TitleAndSlice+" ("+RoiX+","+RoiY+":"+
                    RoiWidth+"x"+RoiHeight+") " +ScanDimensions;       
            //            store origins  which may have changed during scan  /
            V.Globalxy[LOC][0]=V.getActualx ();
            V.Globalxy[LOC][1]=V.getActualy ();
        }//ENd for each grid do a box count
     }//end if not aborted                 
     int [] roimargins=
        {V.getActualy(),V.actualbot,V.actualrt,V.actualx};  
    //                                                                                                  /
    //           Interpret and store the data from                             /
    //           each box for each size at each origin                        / 
    //                                                                                                  / 
    IJ.showStatus("Analyzing data.");
    D = new FLDataProcessor();
    D.ProcessData (this, V);
    IJ.showStatus("Done analyzing data.");    
    //                                                                                                  /
    //         store colours for drawing colour coded images        /
    //                                                                                                  /
    if (    V.ScanSeparateRandomSubAreas ||
            V.isUseParticleAnalyzer () || 
            V.UseRectAnalyzer ||
            V.UseRoiManager)
        { 
            if ((V.TotalForegroundPixels>0)&&!V.isGray())
            {
                V.colourarray[V.particlenumber]=D.data.countDF[0];
                V.particlenumber++;
            }
            if (V.isGray())
            {
                  V.colourarray[V.particlenumber]=D.data.graymassDF[0]; 
                  V.particlenumber++;
            }
        }            
    //                                                                                                  /
    //       WRITE DATA STRINGS FOR RESULTS FILES           /
    //                                                                                                  /
    IJ.showStatus("Writing strings for all locations.");
    WriteDataStringsForAllLocations(thisslice, Titles);
    return true;
}//end of function
     
/**Returns an array of randomly selected  x,y coordinates at 
* which to start scans.
* @param x int for x start
* @param y int for y start
* @return x,y pairs for random starts based on x, y and {@link 
* FLVars#Margins}*/  
int [][] GetXY(int x, int y, ImageProcessor ip)
{
        int [][]xy = FLutil.new2dArray (V.GridPositions, 2, 0);
        if(changedSeries) return xy;
        int s=BOXSIZES[0][BOXSIZES[0].length-1];
        int rt=V.Margins[1];
        int bot = V.Margins[3];
        int a=x,b=y, c = 0;
        int []L=getMaxBlockBlocks ( a,b, bot, rt, s, s, s);
        int wd=L[0]*s, ht=L[1]*s;  
        Random rnd = new Random();             
        int g=V.GridPositions;         
        for (int i = 0; i < g; i++)
        {//make a set of random x,y for each grid position            
            if(V.BLOCKTEXTUREANALYSIS)
            {//for block analyses, keep the largest box in the image block 
                int D=rt-wd-a;               
                int E=bot-ht-b;  
                xy[i][0]=D>0?rnd.nextInt(D):0;
                xy[i][1]=E>0?rnd.nextInt(E):0;  
            }           
            else xy[i]=FLutil.RandomXY (
                    Math.min(s-2, (V.border/2)-2), 
                    Math.min(s-2, (V.border/2)-2));             
            if(!V.isGray())
            {//skip locations with no foreground pixels
                ip.setRoi (xy[i][0], xy[i][1], wd, ht);
                int [] h=ip.getHistogram ();
                float f =(float)h[V.foreground];
                float gb=(float)h[V.background];
                if(f<=0)i--;
                //if((f/gb)<0.001f)i-- ; 
            }
            c++;
            if (c>((ip.getWidth ()-s)*(ip.getHeight ()-s)))return xy;
            //arbitrary FIXME
        }
        return xy;
}
     
/**Stores strings of data according to the scan type, for later 
* printing from the {@link GUI.FL_#WriteScanData data writer}.
* @see #GetFrequenciesAndMasses
* @see #MultifractalDataProcessor
* @see #StoreSLACData
* @see #BoxCountDataWriter
* @param thisslice    int for the current slice
* @param s String of image titles at each location*/
public void WriteDataStringsForAllLocations(int thisslice, String []s)
{
    int es = BOXSIZES[0].length;
    int grids = V.GridPositions;
    IJ.showStatus("Writing results.");
    // -------------------------------------------------------------------------------/
    //            MAKE STRINGS OF DATA PER  BOX SIZE             /
    //                      AND SLOPES FOR DATA FILE                        /
    //------------------------------------------------------------------------------- /
    if(V.ShowData) 
    {
        if(!V.isGray()) 
        {
            if(V.SLAC) V.DataString=V.DataString+StoreSLACData(s);
            else V.DataString= V.DataString+StoreData(s);
        }        
        else V.GDataString=
                V.GDataString+StoreGrayscaleData(s, thisslice);
    }
    // --------------------------------------------------------------------------------/
    //      MAKE  STRINGS OF RAW DATA IF USER WANTS       /
    //-------------------------------------------------------------------------------  /
    if(V.printFrequencies) GetFrequenciesAndMasses();  
    // --------------------------------------------------------------------------------/
    //            MAKE  STRINGS OF MULTIFRACTAL STATS          /
    //-------------------------------------------------------------------------------  /
    if(V.isMF())
    {
        V.MFdata=V.MFdata+MultifractalDataProcessor(thisslice);         
        if(V.GraphLac)
        {
            if(V.isGray())graphgrayL();
            else graphL();
        }             
    }
    // --------------------------------------------------------------------------------/
    //           OR BINARY SLIDING LACUNARITY STATS               /
    //-------------------------------------------------------------------------------  /
    else if (V.SLAC&&!V.isGray())
        V.SLACdata=V.SLACdata+ GetSLACData();
    // --------------------------------------------------------------------------------/
    //           OR GRAYSCALE SLIDING LACUNARITY STATS     /
    //------------------------------------------------------------------------------- /
    else if(V.SLAC&&V.isGray())
        V.GSLACdata=V.GSLACdata+ GetGSLACData ();
    // -------------------------------------------------------------------------------/
    //           OR REGULAR BINARY BOX COUNT STATS            /
    //------------------------------------------------------------------------------- /
    else 
    {    
        if(!V.isGray()) 
        {
            V.BCdata=V.BCdata+BoxCountDataWriter()+
            (
                    V.isDocircul () ? 
                (tab+HullandCircleDataWriter()+newline) :
            newline); 
        }
        // -------------------------------------------------------------------------------/
        //           OR REGULAR GRAYSCALE  BOX COUNT STATS   /
        //------------------------------------------------------------------------------- / 
        else V.GBCdata=V.GBCdata+
          StoreGrayScaleBoxCountSummaryOverAllLocations(thisslice);
         //                    PLOT LACUNARITY DATA             
        if(V.GraphLac)
        {
            if(V.isGray())graphgrayL();
            else graphL();
        }                  
    }
      
    
    // -------------------------------------------------------------------------------/
    //          IF SELECTED, GRAPH REGRESSION LINES   
    //------------------------------------------------------------------------------- /  
    if (V.isGraphRegression ())
    {
        for (int g = 0; g < grids; g++)
        {                  
            FLPlotter.plotLogAndRegression( 
                (V.isGray ()?
                    D.data.graySumOfBoxMassesAtEpsilon[g]  :
                    D.data.CountAtEpsilon[g]),  
                    Epsilons[g],  es, 
                (V.isGray()?
                    D.data.graymassDF[g]  :     D.data.countDF[g]), 
                (V.isGray()?
                    D.data.graymassDFyint[g]   :    D.data.countDFyint[g]), 
                (V.isGray()?
                    D.data.graymassDFrsq[g]   :     D.data.countDFrsq[g]), 
                Titles[g]+
                    (V.isGray()?   "gray "  :     " "), 
                    epsilon, "Count",  true,  true); 
            
            //                        graph grayscale regression lines                 
            if(!V.isGray())  FLPlotter.plotLogAndRegression(
                D.data.SmoothedCount[g], 
                D.data.SmoothedCountSizes[g], 
                D.data.SmoothedCountSizes[g].length,
                D.data.Smootheddf[g], 
                D.data.Smoothedyint[g], 
                D.data.Smoothedrsq[g], 
                Titles[g]+"Smoothed",
                epsilon, 
                "Smoothed Count",
                true, true);
        }
    }         
}

/**Calls data writers for 
* {@link #WriteUnbinnedFrequenciesAndMasses unbinned}and
* {@link #WriteBinnedFrequenciesAndMasses binned}masses.*/     
void GetFrequenciesAndMasses()
{             
     WriteUnbinnedFrequenciesAndMasses ();
     WriteBinnedFrequenciesAndMasses();
}

/**Makes headings for the data file depending on what 
* type of scan was done, and returns  a string of values stored in
* several variables.  The string starts with consecutive 
* {@link #SummaryLineAtThisLocation lines} summarizing each
* scan's information, then presents a line of headings for 
* slopes with {@link #MakeColumnsOfDataAndSlopes columns}
* beneath them. The columns are the data from each scan.  For each 
* data type, Y, (e.g., mean or standard deviation) there is one 
* column for each grid location.  Thus, if there were 2 grid locations,
* there will be 2 columns of Y, the mean, two of the standard
* deviation, etc., with the slope of the ln Y against the ln &epsilon;
* above each column.  Headings are made for the data columns. 
* The {@link FLVars#DataFileHeadings file headings} are 
*modified to match the length of the longest line in the data string.
* @return String of data from all scans at this slice
* @param s string [] of titles */
 String StoreData(String[]s)
 {    
        MakeDataHeadings();    
        String r="";       
        //then a summary string for each location 
        for (int i = 0; i < V.GridPositions; i++)
        r=r+SummaryLineAtThisLocation(i, s[i])+newline;          
        //            followed by lines of slopes and columns of data-         //     
        r=r+MakeColumnsOfDataAndSlopes ();        
        return r;
}

/**Stores data for a sliding box lacunarity scan.  Orchestrates the 
* data file preparation by making headings, a summary line of
* information, and columns of data and their slopes through calls to
* other methods.  Is essentially the sister to 
* {@link #StoreData the box count scan data file maker}
* @see #SummarySLACLineAtThisLocation
* @see #MakeColumnsOfDataAndSlopes
* @see #MakeSLACDataFileHeadings
* @see #StoreData*/                         
String StoreSLACData(String[]s)
{  
     MakeSLACDataFileHeadings();        
     //  Make data file headings then a string for each location        /
     String r=SummarySLACLineAtThisLocation(s[0])+newline;          
     //                           Make columns of data;                        /
     //                 one for each heading  and epsilon                          /
     r=r+MakeColumnsOfSLACDataAndSlopes ();        
     return r;
}

/**Makes headings for the data file for a regular box count scan. 
* First, the main summary {@link #datafileheadingsarray 
* headings} are arranged in a string, then enough tabs follow to
* make enough columns to accommodate the reported {@link
* #DataColumnHeadArray data} at each position for 
* {@link FLVars#GridPositions the number of scan positions}.  
* @see FLVars#MakeHeadings 
* @see #datafileheadingsarray 
* @see FLVars#DataFileHeadings*/ 
void MakeDataHeadings()
 {
     V.DataFileHeadings =V.MakeHeadings (datafileheadingsarray); 
     int l = datafileheadingsarray.length;        
     int e = (2+(DataColumnHeadArray.length*V.GridPositions));
     if(e<l)e=l;
     //        -Make the number of spaces for headings for each column 
     //               --   to match the number of grid locations                    -/
     for (int i = l; i <e ; i++)
         V.DataFileHeadings=V.DataFileHeadings+tab+" ";
 }
 
/**Makes headings for the data file for a sliding box lacunarity scan.
* First, the main summary {@link #SLACdatafileheadingsarray 
* headings} are arranged in a string, then enough tabs follow to 
* make enough columns to accommodate the reported {@link 
* #DataColumnHeadArray data}<i>Note that there 
 * is only one {@link FLVars#GridPositions grid origin}
 * for a SLAC scan, so only one column is written per data type.  
* @see FLVars#MakeHeadings
* @see #datafileheadingsarray
* @see FLVars#DataFileHeadings*/ 
private void MakeSLACDataFileHeadings()
{
     V.DataFileHeadings =
             V.MakeHeadings (SLACdatafileheadingsarray); 
     int l = SLACdatafileheadingsarray.length;        
     int e = (2+(DataColumnHeadArray.length));
     if(e<l)e=l;
     //        -Make the number of spaces for headings for each column  
     for (int i = l; i <e ; i++)
         V.DataFileHeadings=V.DataFileHeadings+tab+" ";
 }

/**Used to make the beginning of the headings for {@link 
* #StoreGrayscaleData storing grayscale data}. Uses a preset 
* string for {@link FLVars#StringForIntensityCalculation 
* intensity} and {@link FLVars#StringForDFFormula fractal 
* dimension}formulae.
* @return String of headings for grayscale scans*/
public String [] GHeadings ()
{
     String I=V.StringForIntensityCalculation (V.grayscantype);
     String F=V.StringForDFFormula (V.grayscantype);
     return new String [] {
             "Grayscale Data (File & Roi) ",
             "Pixels",
             "Mass FRACTAL DIMENSION (D)",
             "Formula for D",
             "Formula for Intensity",    
             rsq+" for regression line",
             "Std Error (SE)",    
            "Y-Intercept",
            "Lacunarity ("+Lambda+")=" +mu+cvsq+" FOR I "+I,
             cvsq+"for "+Lambda,   
             Lambda+"=" +mu+" "+bpd+" "+lambda,                                
             cvsq+"for "+mu+" BPD "+Lambda,   
            "Y-Int Prefactor",                       
            "Slice",
            "Number of Box Sizes", 
            "Maximum box size",
            "Minimum box size"
             }; 
}

/**Headings for the data file for a box count scan.
@see #MakeDataHeadings
@see #StoreData*/          
String [] datafileheadingsarray =
{
        "File+Slice (ROI location(x,y): width x height) GRID ",                            
        "Fractal Dimension (D)=regression " +slopeVsE("F"),
        "D(L)=" +LD,  
        "CORRELATION COEFFICIENT (" +rsq+") for D",
        "STANDARD ERROR (SE) for D",
        "Y-intercept for D",                    
        "Smoothed D",
        rsq,
        "SE",                    
        Lambda+"="+meanOverBoxes("F"+lambda),
        "Variation in "+Lambda+"="+cv,
        Lambda+"="+meanOverBoxes("E"+lambda),                   
        cv,
        Lambda+"="+meanOverBoxes("FBPD"+lambda),
        cv,
        Lambda+"="+meanOverBoxes("EBPD"+lambda),
        cv,                 
        cv +" for count of boxes containing foreground pixels (F) "  ,
        cv +"for count including empty boxes (E)" ,
        cv+"/"+cv+" for F/E",
        "Box Sizes for Smoothed D"
  };
            
/**Headings for a sliding box lacunarity scan.
@see #StoreSLACData
@see #MakeSLACDataFileHeadings*/        
String [] SLACdatafileheadingsarray =
{
        "File+Slice (ROI location(x,y): width x height) GRID ",                         
        "Fractal Dimension Dm=regression " +slopeVsE("mass"),     
        Lambda+"D=" +LD,
        "CORRELATION COEFFICIENT (" +rsq+") for Dm",
        "STANDARD ERROR (SE) for Dm",
        "Y-intercept for Dm",  
        Lambda+"="+meanOverBoxes("F"+lambda),
        "Variation in "+Lambda+"="+cv,
        Lambda+"="+meanOverBoxes("E"+lambda),                   
        cv,
        Lambda+"="+meanOverBoxes("FBPD"+lambda),
        cv,
        Lambda+"="+meanOverBoxes("EBPD"+lambda),
        cv,                    
        cv +" for count of boxes containing foreground pixels (F) "  ,
        cv +"for count including empty boxes (E)" ,
        cv+"/"+cv+" for F/E"
};
            
    
/**Makes headings and strings for grayscale scan results window. 
* Makes a string of {@link FLVars#GDataHeadings} headings for 
* the results window for this scan, and makes a {@link 
* #StringOfGSBoxCountDataForThisGrid summary string} for 
* each location with appended columns of {@link 
* #DataAndSlopesforGrayscaleScans data and slopes}against 
* &epsilon;.
* @see GUI.FL_#WriteScanData
* @param s String []
* @param thisslice int for the current slice
* @return the string of data*/
public String StoreGrayscaleData(String[]s, int thisslice)
{
     String g = "";
     //        Make the number of spaces for headings for each column 
     //           to match the number of grid locations                    
     String [] FirstPartOfGDataHeadings = GHeadings();
     V.GDataHeadings =  
             V.MakeHeadings (FirstPartOfGDataHeadings);         
     int end=//length of all of the columns and their leader
             2+ColumnHeadingsForGrayscaleData.length*
                         V.GridPositions;
     int start = FirstPartOfGDataHeadings.length;
     for (int i = start; i < end; i++)
             V.GDataHeadings=V.GDataHeadings+tab+" ";
     //        Make a summary string for each location                  
    for (int i = 0; i < V.GridPositions; i++)
     g=g+StringOfGSBoxCountDataForThisGrid (s[i], i, thisslice);          
     //  Make columns of data for each heading and grid location
     g =g+DataAndSlopesforGrayscaleScans();                      
     return g;
}    
     
/**Puts up a text window showing the masses and their frequencies
* stored in {@link FLData#UnbinnedFrequencies} and 
* {@link FLData#UnbinnedMasses}.*/
void WriteUnbinnedFrequenciesAndMasses()
{ 
    V.FrequencyData="";
    String I = V.isGray()?
        V.StringForIntensityCalculation (V.grayscantype):
        "Mass (Pixels at "+epsilon+")";  
    String mass = V.isGray()?"I at ":"mass at ";
    String Headings = bs+tab+epsilon +tab+meanprob;
    for (int i = 0; i<V.MaxFrequencies;i++)
        Headings=Headings+tab+"F or M";
    String FH = "" +newline+newline +"FREQUENCIES (F)"
            +newline+bs+tab+epsilon +tab+meanprob;
    for (int i = 0; i<V.MaxFrequencies;i++)
        FH=FH+tab+"Frequency";
    FH=FH+newline;
    int L = FLutil.LongestArray(D.data.UnbinnedFrequencies);
    L=Math.max(V.MaxFrequencies, L);
    String MH = "" +newline+ newline +"MASS (M or I) "+I
            +newline+bs+tab+epsilon + tab+" ";
    for (int i = 0; i<L;i++) MH=MH+tab+mass+epsilon;
    MH=MH+newline;
    for (int LOC=0; LOC<V.GridPositions; LOC++)  
    { 
            String f = "",  m = "";
            for (int e=0; e<BOXSIZES[LOC].length; e++)
            {
                f = f+
                    FLutil.fnum (BOXSIZES[LOC][e])+tab+
                    FLutil.fnum (Epsilons[LOC][e])+tab+
                    FLutil.fnum
                    (D.data.MassLacMeanAtEpsilon
                    [LOC][e])
                    +tab+
                    (D.data.UnbinnedFrequencies[LOC][e])+newline;
                m = m  +
                    FLutil.fnum (BOXSIZES[LOC][e])+tab+
                    FLutil.fnum (Epsilons[LOC][e])+tab+
                    "" +
                    tab+
                    D.data.UnbinnedMasses[LOC][e]+newline;
            }
            V.FrequencyData= V.FrequencyData+
                    "Grid Location "+LOC+newline;
            V.FrequencyData=V.FrequencyData+FH+f+newline;               
            V.FrequencyData = V.FrequencyData+MH +m+newline;                 
    }                
    TextWindow t=new TextWindow ("Frequencies & Masses "+
            V.TitleAndSlice, Headings, V.FrequencyData, 500, 500);                 
}
      
/**Puts up a text window showing the masses and their frequencies
* stored in {@link FLAnalyzer.FLData#BinnedFrequencies} and 
* {@link FLAnalyzer.FLData#BinnedMasses}.*/ 
void WriteBinnedFrequenciesAndMasses()
{ 
    V.BinnedFrequencyData=""; 
    String I = V.isGray()? 
        V.StringForIntensityCalculation (V.grayscantype):
        "Mass (Pixels at "+epsilon+")";  
    String Headings = bs+tab+epsilon +tab+meanprob;
    for (int i = 0; i<V.MaxFrequencies;i++)
        Headings=Headings+tab+"Probability or Bin";
    String FH = "" +newline+ newline    +"Binned Probabilities"
            +newline+bs+tab +epsilon +tab+meanprob;
    for (int i = 0; i<V.MaxFrequencies;i++)
        FH=FH+tab+"Probability";
    FH=FH+newline;
    int L = FLutil.LongestArray(D.data.BinnedFrequencies);
    L=Math.max(V.MaxFrequencies, L);
    String MH = "" +newline+ newline +"Bin for Pix="+I
            +newline+bs+tab+epsilon + tab+" ";
    for (int i = 0; i<L;i++) MH=MH+tab+"Midpoint of Bin "
            +epsilon;
    MH=MH+newline;
    for (int LOC=0; LOC<V.GridPositions; LOC++)  
    {
        String f = "",  m = "";
        for (int e=0; e<BOXSIZES[LOC].length; e++)
        {
             f = f+ FLutil.fnum (BOXSIZES[LOC][e])+tab+
                     FLutil.fnum (Epsilons[LOC][e])+tab+
                     FLutil.fnum (D.data.MeanOfBPDAtEpsilon [LOC][e])
                     +tab+(D.data.BinnedFrequencies[LOC][e])+newline;     
             m = m  + FLutil.fnum (BOXSIZES[LOC][e])+tab+
                     FLutil.fnum (Epsilons[LOC][e])+tab+ "" +tab+
                     D.data.BinnedMasses[LOC][e]+newline;
        }
        V.BinnedFrequencyData=
                V.BinnedFrequencyData+"Grid Location "+LOC+newline;
        V.BinnedFrequencyData= V.BinnedFrequencyData+
                FH+f+newline;               
        V.BinnedFrequencyData = V.BinnedFrequencyData+MH
                +m+newline;                 
    }              
    TextWindow b=new TextWindow("Binned Probabilities & Masses "
            +V.TitleAndSlice, Headings, 
            V.BinnedFrequencyData, 500, 500);                 
}

/**Makes headings for multifractal data file and finds multifractal
* statistics from data stored in {@link #Pix pixels array}.  Makes
* {@link FLUtilities.FLutil#SmoothedArray 
* smoothed} and {@link FLUtilities.FLutil#minMassCover 
* minimum cover} arrays from the pixel data and calls {@link 
* #doMF  multifractal} data processing method on the new arrays
* as well as on the original data.  {@link #PlotFAndAlpha 
* Multifractal spectra} are plotted elsewhere.
* @see #MFheadings
* @see FLVars#MFHeadings
* @see FLVars#MakeHeadings
* @see GUI.FL_#WriteScanData
* @see #doMF
* @see FLutil#minMassCover
* @see FLutil#SmoothedArray
* @see FLVars#dataoption*/
public String MultifractalDataProcessor(int thisslice)
{
    FLUtilities.BoxCountHolder smooth;    
    String d="";
    V.MFHeadings=FLVars.MakeHeadings (MFheadings);
    int grids = V.GridPositions;   
  //IF THE USER WANTS A SMOOTHED OR MIN COVER TEST
    if(V.dataoption==V.MINCOVER
            ||V.dataoption==V.SMOOTHEDMINCOVER)
    {
        //    ALWAYS WRITE THE MINIMUM COVER STATS
        d=d+ doMF(D.data.minmasses, D.data.minBoxes, 
                -1, V.dataoption, thisslice);
        //    OPTIONALLY FIND A SMOOTHED MINIMUM COVER
        if (V.dataoption==V.SMOOTHEDMINCOVER)
        {
            {smooth = FLutil.SmoothedArray (D.data.minmasses, 
                     D.data.minBoxes);    
                d=d+doMF(smooth.masses, smooth.boxes,
                        -1,V.dataoption, thisslice);
            }
        }
    }
//   IF THERE IS NO MINIMUM COVER DO EACH GRID'S STATS
if(V.dataoption!=V.MINCOVER)
{
    for (int g=0; g < grids; g++)              
    {
        //IF THE USER WANTS SMOOTHED DATA, USE IT
        if(V.dataoption==V.SMOOTHED)
            {
                smooth =FLutil.SmoothedArray(Pix[g], BOXSIZES[g]);                     
                d=d+doMF(smooth.masses, smooth.boxes,
                        g, V.dataoption, thisslice);
            }
        else d=d+doMF(Pix[g],BOXSIZES[g], g, V.dataoption, thisslice);                        
    }
}
    return d;
}

CircStats C=new CircStats ();
/**Makes a {@link FLAnalyzer.CircStats circle and hull processor}
 *and records various morphometrics.
* @param ipk ImageProcessor for finding the hull and circle
* @param thisslice int for the current slice
* @param lastslice int for the last slice to process
* @param ThisRectangleSubScan int for drawing 
* @param isRectangleSubScan boolean for drawing*/
public void GetCircAndHullStats( 
    ImageProcessor ipk,
    int thisslice, 
    int lastslice,
    int ThisRectangleSubScan,
    boolean isRectangleSubScan)
{
    initializeCircStats ();
    C=new CircStats ();
    if (V.isDocircul ())
    {
            if((V.TotalForegroundPixels>0)&&!V.isGray())
            {
                 V=C.getCircStats (                     
                    ipk,
                    thisslice,
                    lastslice,
                    V.isVerticalLine,
                    V.isHorizontalLine, 
                    isRectangleSubScan,
                    ThisRectangleSubScan, 
                    V, Rois);
                }
    }
}

/**Writes a tabbed string of data summarized over all locations from
 * a regular box count.  The string is appended to {@link 
* FLVars#BCdata the data string} for this scan by the calling 
* function. Uses data previously stored in the {@link 
* FLAnalyzer.FLDataProcessor data processor}. There is no 
 tab and no newline at the end.*/
String BoxCountDataWriter()
{  
    IJ.showStatus("Writing box count results.");
    float minbox = 0, maxbox = 0, 
            numboxes = 0, grids=V.GridPositions;
    for (int i = 0; i < V.GridPositions; i++)
    {
        numboxes = numboxes+BOXSIZES[i].length;
        minbox = minbox+BOXSIZES[i][0];
        maxbox = maxbox+BOXSIZES[i][BOXSIZES[i].length-1];
    }
    String min=FLutil.fnum (minbox/grids);
    String max = FLutil.fnum (maxbox/grids);
    String num = FLutil.fnum (numboxes/grids);
    
    String s=        
        Titles[0]  + tab                 
        +FLutil.fnum (V.TotalForegroundPixels) +tab
        +FLutil.fnum (V.TotalAllPixels) +tab
        +FLutil.fnum (D.data.avgs.df)+tab       
        +FLutil.fnum (FLutil.MeanOfArray (D.data.LDF))+tab
        +FLutil.fnum (D.data.avgs.FractalStatsRsq)+tab
        +FLutil.fnum (D.data.avgs.stderr)+tab
           
        +FLutil.fnum (D.data.CountDFStats.coefVar)    +tab
           
        +FLutil.fnum (D.data.avgs.smootheddf)+tab
        +FLutil.fnum (D.data.avgs.Smoothedrsq)+tab
        +FLutil.fnum (D.data.avgs.Smoothedstderr)+tab
           
        +FLutil.fnum (D.data.mins.df)+tab
        +FLutil.fnum (D.data.mins.FractalStatsRsq)+tab
        +FLutil.fnum (D.data.mins.stderr)+tab
           
        +FLutil.fnum (D.data.mins.smootheddf)+tab
        +FLutil.fnum (D.data.mins.Smoothedrsq)+tab
        +FLutil.fnum (D.data.mins.Smoothedstderr)+tab
           
        +FLutil.fnum (D.data.countYinterceptStats.mean)     +tab
        +FLutil.fnum (D.data.CountPrefactorLacOverAllGrids)+"/"+
                FLutil.fnum (D.data.MassPrefactorLacOverAllGrids)+tab
        +FLutil.fnum (D.data.countYinterceptStats.coefVar) +tab
        +FLutil.fnum (D.data.MassLacAtGridstats.mean) +tab 
        +FLutil.fnum (D.data.MassLacAtGridstats.cvsq)+tab 
        +FLutil.fnum (D.data.EMassLacAtGridstats.mean) +tab 
        +FLutil.fnum (D.data.EMassLacAtGridstats.cvsq)+tab
        +FLutil.fnum (D.data.BPDLacAtGridStats.mean)+tab 
        +FLutil.fnum (D.data.BPDLacAtGridStats.cvsq) +tab
        +FLutil.fnum (D.data.EBPDLacAtGridStats.mean)+tab 
        +FLutil.fnum (D.data.EBPDLacAtGridStats.cvsq) +tab
        +FLutil.fnum (D.data.CountCVStats.mean)+tab
        +FLutil.fnum (D.data.CountCVStats.mean
                  /D.data.EmptiesCVStats.mean) +tab
       + num + tab //+FLutil.fnum (BOXSIZES[0].length) +tab
       + FLutil.fnum (D.data.avgs.smoothedNumBoxes) +tab
        +FLutil.fnum (V.GridPositions) +tab
       + max+tab //+BOXSIZES[0][BOXSIZES[0].length-1]+tab
        +min;//+BOXSIZES[0][0];        
           ;      
     return s;
}

/**Returns a string of data from the convex hull and minimum 
 *bounding circle.  The string has no tab nor new line at the end. */
 String HullandCircleDataWriter()
{        
   String s=                            
        FLutil.fnum (V.TotalForegroundPixels) +tab
        +FLutil.fnum (V.TotalAllPixels) +tab
        +FLutil.fnum (V.TotalForegroundPixels/V.AREAofHull)+tab
        +C.SpanRatio+tab                 
        +C.Masscentres    +tab
        +C.MaxSpanOfPixels+tab
        +C.Area+tab
        +C.Perimeter+tab
        +C.Circularity +tab
        +C.MargWidth  +tab
        +C.MargHeight   +tab
        +C.MaxRadiusStr  +tab
        +C.MaxOverMinRadiiStr  +tab
        +C.CVRadiiStr  +tab
        +C.MeanOfHullRadii  +tab
        +C.Circlecentres    +tab
        +C.CircleDiameter +tab
        +C.MaxRadiusCircleStr +tab
        +C.MaxOverMinRadiiCircleStr +tab
        +C.CVRadiiCircleStr+tab
        +C.MeanOfHullCircleRadii+tab                 
        +V.methodused ;      
     return s;
}   
 
/**Arranges the previously calculated statistics and {@link 
* FLAnalyzer.FLDataProcessor data} from all grayscale scans over
* all {@link FLVars#GridPositions locations} in a string for 
* printing.
* @return String of data averaged over all grids*/
public String StoreGrayScaleBoxCountSummaryOverAllLocations(
    int thisslice)
{      
    float minbox = 0, maxbox = 0, 
            numboxes = 0, grids=V.GridPositions;
    for (int i = 0; i < V.GridPositions; i++)
    {
        numboxes = numboxes+BOXSIZES[i].length;
        minbox = minbox+BOXSIZES[i][0];
        maxbox = maxbox+BOXSIZES[i][BOXSIZES[i].length-1];
    }
    String min=FLutil.fnum (minbox/grids);
    String max = FLutil.fnum (maxbox/grids);
    String num=FLutil.fnum (numboxes/grids);
    String LongString =V.TitleAndSlice+tab                 
            +FLutil.fnum(V.TotalAllPixels) +tab          
            +FLutil.fnum(D.data.grayMassDFStats.mean)+tab
            +FLutil.fnum(D.data.grayMassDFrsqStats.mean)+tab                   
            +FLutil.fnum(D.data.grayMassStdErrStats.mean)+tab  
            +FLutil.fnum(D.data.grayMassDFStats.coefVar)+tab
            +FLutil.fnum(D.data.grayMassYintStats.mean)+tab 
            +FLutil.fnum (D.data.grayMassYintStats.coefVar)+tab 
            +FLutil.fnum (D.data.grayMassLacAtGridstats.mean) +  tab                    
            +FLutil.fnum(D.data.grayMassLacAtGridstats.coefVar)+tab
            + FLutil.fnum(D.data.BPDLacAtGridStats.mean)+tab                    
            + FLutil.fnum(D.data.BPDLacAtGridStats.coefVar)+tab    
            +V.StringForIntensityCalculation (V.grayscantype)+tab
            +V.StringForDFFormula (V.grayscantype)+tab    
            + FLutil.fnum (D.data.grayPrefactorLacOverAllGrids) +tab
            + FLutil.fnum (V.GridPositions) + tab
            + num + tab
            + max + tab    
            + min + tab + newline;    
    return LongString;
}                                                    
                        
/**Arranges the previously calculated statistics and data from one
* {@link FLAnalyzer.FLDataProcessor#FindGrayDf grayscale }
*  scan at all box sizes for one grid position in a string for printing.
* Includes {@link FLVars#StringForIntensityCalculation  intensity}
* and {@link FLVars#StringForDFFormula DF} forumulae.
* @param title String for the title to print with the data
* @param thisslice int for the current slice
* @param i int for the index of this grid position
* @return String of grayscale scan box counting data*/
public String StringOfGSBoxCountDataForThisGrid(
          String title, int i, int thisslice)
{
    IJ.showStatus("Writing grayscale box count results."); 
    String LongString =title+tab +                
            FLutil.fnum(V.TotalAllPixels) +tab          
            +FLutil.fnum (D.data.graymassDF[i])+tab  
            +V.StringForDFFormula (V.grayscantype)+tab  
            +V.StringForIntensityCalculation (V.grayscantype)+tab 
            +FLutil.fnum (D.data.graymassDFrsq[i])+tab    
            +FLutil.fnum (D.data.graymassDFstderr[i])+tab  
            +FLutil.fnum (D.data.graymassDFyint[i])+tab  
            +FLutil.fnum (D.data.grayMassLacAtGrid[i]) +  tab     
            +FLutil.fnum (D.data.grayVariationInMassLacAtGrid[i]) +tab
            +FLutil.fnum (D.data.BPDLacAtGrid[i])+tab 
            +FLutil.fnum (D.data.VariationInBPDLacAtGrid[i])+tab 
            +FLutil.fnum (D.data.graymassDFyintprefactor[i]) +tab           
            +FLutil.fnum (thisslice) + tab
            +FLutil.fnum (BOXSIZES[i].length) + tab    
            +BOXSIZES[i][BOXSIZES[i].length-1]+tab
            +BOXSIZES[i][0]+tab + newline;    
    return LongString;
}                                                                                                    
       
/**Returns a string of and graphs lacunarity data gathered during a
* sliding box lacunarity scan.  Uses data stored in {@link 
* FLAnalyzer.FLData#MassLacAtEpsilon F&lambda;},
* {@link FLAnalyzer.FLData#BPDLacAtEpsilon FBPD&lambda;},
* {@link FLAnalyzer.FLData#EMassLacAtEpsilon E&lambda;}, and
* {@link FLAnalyzer.FLData#EBPDLacAtEpsilon EBPD&Lambda;} 
* arrays.  Calls {@link FLUtilities.FLutil#Plus1 a function to add 1}
* to each value in these arrays prior to processing, then 
* {@link #MakeStrings makes columns} from the resulting arrays,
* listing a value from each following each {@link #BOXSIZES 
* boxsize} and {@link #Epsilons &epsilon;}.  Makes a {@link 
* #TabSlopeVsE string of slopes vs &epsilon;} below each column 
* of data.  If the {@link FLVars#GraphLac option} is selected, 
* generates {@link #logPlots graphs}.  The first column of the 
* string lists scan and file information and {@link 
* FLAnalyzer.FLData#massDF the mass Df}.  The data are not the 
* same as in a nonoverlapping box count, because that count is, in 
* effect, over a fixed grid, meaning individual boxes of a certain size
* do not overlap.  In contrast, for sliding lacunarity, the data are 
* from overlapping samples.  At each &epsilon;, a box is moved over
* the grid by some fixed increment that does not have to be the 
* width of the box.</LI>For binary images, heterogeneity is based 
* on the Coefficient of Variation CV=&sigma;/&mu; for foreground 
* pixels per box and for pixels per box in the image space. It is 
* calculated using the raw data and a binned probability distribution
* with the number of bins set using {@link 
* FLVars#MaxFrequencies}.<UL>Thus,<OL><LI>for boxes that 
* had at least 1 foreground pixel:<UL><LI>F&lambda;=1+
* (&sigma;/&mu;)<sup>2</sup></LI><LI>FBPD&lambda;=1+
* (&sigma;/&mu;)<sup>2</sup></LI></UL></LI><LI>and for 
* all boxes in the image space:<UL><LI>E&lambda;=1+(&sigma;
* /&mu;)<sup>2</sup></LI><LI>EBPD&lambda;=1+(&sigma;/
* &mu;)<sup>2</sup></LI></UL></LI></OL></Ul><LI>For
* grayscale images, heterogeneity is based on the variation in gray 
* values.</LI><LI>This function delivers graphs of the different
* &Lambda;s or Ln &Lambda; against Ln&epsilon;.</LI></ul><p>
* This method also delivers the mass  fractal dimension, calculated 
* from the ln-ln plot of mean foreground pixels per box vs &epsilon;
* (box size/image width).
* @see "Plotnick, Gardner, O'Neill, Landscape Ecology,8:3:201-211"
* @see "Smith, Lange, Marks"
* @see #logPlots    
* @param thisroi int for this roi
* @param thisslice int for current slice
* @return String of columns of data*/
public String GetSLACData ()
{
        //make arrays and variables 
        //ASSUME ONE GRID POSITION  FIXME
        String F=
                "D=slope [ln "+FLMain.mu+"/ln "+FLMain.epsilon+"]";
        String I = mu+"=Mean Pixels Per Box";
        String S = "";
        String
        s1="F"+lambda +"=1+"+cvsq+" "+Titles[0], 
        s2="E"+lambda +"=1+"+cvsq+" "+Titles[0], 
        s3="F "+bpd+" "+lambda+"=1+"+cvsq+Titles[0], 
        s4="EBPD"+lambda+"=1+"+cvsq+" "+Titles[0]; 

        //ADD 1 TO EACH VALUE IN EACH LACUNARITY ARRAY 
        float [] L1=FLutil.Plus1 (D.data.MassLacAtEpsilon[0]);
        float [] L3=FLutil.Plus1 (D.data.BPDLacAtEpsilon[0]);
        float [] L2=FLutil.Plus1 (D.data.EMassLacAtEpsilon[0]);
        float [] L4=FLutil.Plus1 (D.data.EBPDLacAtEpsilon[0]);        
        float [][] f=
        {
                 FLutil.IntToFloatArray (BOXSIZES[0]),
                 Epsilons[0],
                 D.data.MassLacMeanAtEpsilon[0],             
                 D.data.MassLacStdDevAtEpsilon[0],
                 D.data.CountAtEpsilon[0],
                 L1   ,
                 D.data.EMassMeansAtEpsilon[0],
                 D.data.EMassStdDevAtEpsilon[0] ,
                 D.data.EmptiesAtEpsilon[0],
                 L2,                     
                 D.data.MeanOfBPDAtEpsilon[0],
                 D.data.StdDevOfBPDAtEpsilon[0],
                 L3,                     
                 D.data.EMeanOfBPDAtEpsilon[0],
                 D.data.EStdDevOfBPDAtEpsilon[0],
                 L4
        };      

     String [] Info = 
     {
        Titles[0],
        "Fractal Dimension (D)="+FLutil.fnum(-D.data.massDF[0]), 
        F,
        I,                     
        rsq+"for D="+FLutil.fnum(D.data.massDFrsq[0]),
        "inverse of prefactor="+FLutil.fnum(D.data.mprefactor[0]),
        "y-intercept=" +FLutil.fnum(D.data.massDFyint[0]), 
        "X slide="+V.SLIDEX,
        "Y slide="+V.SLIDEY,
        "Bins ="+V.MaxFrequencies
     };
     
     int end = BOXSIZES[0].length>Info.length
             ?         BOXSIZES[0].length
             :          Info.length;               
     //-----------------------------------------------------------------------------------/
     //           write the information in the first column, and              /
     //                       the data in the following columns                       /
     //                                                                                                        /
     for (int i = 0; i < end; i++)
     {         
         S=S+newline+ (i<Info.length ? Info[i]+tab : ""+tab) +
              (i<BOXSIZES[0].length 
         ?     MakeStrings(f, i)
          :      " " );           
     }
     //---------------------------------------------------------------------------------/
     //             Make a string introducing the slopes and                 /
     //                      showing the slopes of each column                   /
     //                               against epsilon                                            /
     //---------------------------------------------------------------------------------/
     S=S+newline+"Slopes (from power regressions)"+tab+
             "lim [ln Y vs ln "+epsilon+"]"+tab+newline+" "+tab+
              TabSlopeVsE0(f); 
     
     //                    GRAPH LACUNARITY VALUES                                 //
     if(V.GraphLac) logPlots (
             new float [][]{L1, L2, L3, L4}, 
             new String []{s1, s2, s3, s4}, 0);              
     //                    RETURN THE DATA STRING                                      /
    return S;
 }
    
/**Graphs lacunarity values against &epsilon;.Uses stored values in 
* {@link FLData#MassLacAtEpsilon},  
* {@link FLData#BPDLacAtEpsilon},
* {@link FLData#EMassLacAtEpsilon}, and
* {@link FLData#EBPDLacAtEpsilon}. 
* @see #WriteDataStringsForAllLocations
* @see #logPlots*/    
public void graphL ()
{           
    int grids=V.GridPositions;    
    String F ="D=slope [ln "+FLMain.mu+"/ln "+FLMain.epsilon+"]";
    String I = mu+"=Mean Pixels per Box";
    String S = "";

    float [][] L1 = new float[grids][];
    float [][] L2 = new float [grids][];
    float [][] L3 = new float [grids][];
    float [][] L4 = new float [grids][];
    //ADD 1 TO EACH VALUE IN EACH LACUNARITY ARRAY
    for (int k = 0; k < grids; k++)
    {
         L1[k]=FLutil.Plus1 (D.data.MassLacAtEpsilon[k]);
         L3[k]=FLutil.Plus1 (D.data.BPDLacAtEpsilon[k]);
         L2[k]=FLutil.Plus1 (D.data.EMassLacAtEpsilon[k]);
         L4[k]=FLutil.Plus1 (D.data.EBPDLacAtEpsilon[k]);
    }

     for (int LOC=0; LOC<grids; LOC++)
     {
        String  
                s1="F"+lambda +"=1+"+cvsq+" "+Titles[LOC], 
                s2="E"+lambda +"=1+"+cvsq+" "+Titles[LOC], 
                s3="F "+bpd+" "+lambda+"=1+"+cvsq+Titles[LOC], 
                s4="EBPD"+lambda+"=1+"+cvsq+" "+Titles[LOC]; 
        logPlots ( new float [][]{L1[LOC], L2[LOC], L3[LOC], L4[LOC]}, 
               new String []{s1, s2, s3, s4}, LOC);              
     }
}
    
/**Graphs lacunarity against epsilon for binned probability 
* distribution and unbinned {@link FLAnalyzer.FLDataProcessor
* #FindGrayDf grayscale} data {@link FLutil#Plus1 plus one}.
* @see #logPlots
* @see FLAnalyzer.FLData#grayMassLacAtEpsilon lacunarity
* @see FLAnalyzer.FLData#BPDLacAtEpsilon bpd lacunarity*/
public void graphgrayL ()
{
//        //make arrays and variables             
    int grids=V.GridPositions;  
    String F ="D=slope [ln "+FLMain.mu+"/ln "+FLMain.epsilon+"]";
    String I = mu+"=Mean Pixels per Box";
    String S = "";
    float [][] L1 = new float[grids][];
    float [][] L2 = new float [grids][];
    //once for each value in each array in L, write the value plus 1 
    for (int k = 0; k < grids; k++)
    {
         L1[k]=FLutil.Plus1(D.data.grayMassLacAtEpsilon[k]); 
         L2[k]=FLutil.Plus1(D.data.BPDLacAtEpsilon[k]);                  
    }               
    for (int LOC=0; LOC<grids; LOC++)
    {//once for each grid position, graph lacunarity vs epsilon
        String
            s1="F"+lambda +"=1+"+cvsq+" "+Titles[LOC], 
            s2="F "+bpd+" "+lambda+"=1+"+cvsq+Titles[LOC];          
        logPlots ( new float [][]{L1[LOC], L2[LOC]},
                new String []{s1, s2}, LOC);              
    }   

}

/**Returns a string of file and scan information, as well as columns 
* of data to write to the {@link FLVars#GSLACdata data string} 
* for grayscale sliding box scans, and may generate graphics. 
* Calls {@link FLUtilities.FLutil#Plus1 a function to add 1} to the 
 * data arrays  ({@link FLAnalyzer.FLData#grayMassLacAtEpsilon}
 * and {@link FLAnalyzer.FLData#BPDLacAtEpsilon }) used in 
 * grayscale sliding box lacunarity scans.  The
 * {@link #MakeStrings columns}
* also list file and scan information as well as {@link #Epsilons 
* &epsilon;}, {@link #BOXSIZES box size}, {@link 
* FLAnalyzer.FLData#grayCountAtEpsilon}, {@link 
* FLAnalyzer.FLData#graySumOfBoxMassesAtEpsilon}, {@link 
* FLAnalyzer.FLData#grayMassLacMeanAtEpsilon}, {@link 
* FLAnalyzer.FLData#grayMassLacStdDevAtEpsilon}, {@link 
* FLAnalyzer.FLData#MeanOfBPDAtEpsilon}, and {@link 
* FLAnalyzer.FLData#StdDevOfBPDAtEpsilon}. Below each column
* is the {@link #TabSlopeVsE slope} vs &epsilon;. Generates 
* {@link #logPlots graphics } if the {@link FLVars#GraphLac 
* option} to view these has been selected.
* @return String of sliding lacunarity data for grayscale images
* @see #GetSLACData (works the same as this function)*/
public String GetGSLACData ()
{
    float [][] BS=FLutil.IntToFloatArray (BOXSIZES);
    String F=V.StringForDFFormula (V.grayscantype);
    String I =V.StringForIntensityCalculation (V.grayscantype);        
    int es = BS[0].length;   
    int LOC=0;//there is only one origin for sliding lacunarity scans
    String S = "";   
    String [] Info =  
    {
            (Titles[LOC]), 
            ("Fractal Dimension (D) ="
                    +FLutil.fnum (D.data.graymassDF[LOC])), 
            (F),
            (I),                     
            (rsq+"for D ="+FLutil.fnum(D.data.graymassDFrsq[LOC])),
            ("inverse of prefactor="
                    +FLutil.fnum(D.data.graymassDFyintprefactor[LOC])),
            ("y-intercept="+FLutil.fnum(D.data.graymassDFyint[LOC])), 
            ("X slide="+V.SLIDEX),
            ("Y slide="+V.SLIDEY),
            "Bins ="+V.MaxFrequencies
    };            
     float [][] Lacs = new float [1][];
     float [][] bLacs  = new float [1][];
     Lacs[LOC]=FLutil.Plus1 (D.data.grayMassLacAtEpsilon[LOC]);
     bLacs[LOC]=FLutil.Plus1(D.data.BPDLacAtEpsilon[LOC]);
     float [][][] n=new float [][][]{
                        BS,
                        Epsilons,
                        D.data.grayCountAtEpsilon,
                        D.data.graySumOfBoxMassesAtEpsilon,
                        D.data.grayMassLacMeanAtEpsilon,
                        D.data.grayMassLacStdDevAtEpsilon, 
                        Lacs,
                        D.data.MeanOfBPDAtEpsilon,
                        D.data.StdDevOfBPDAtEpsilon,
                        bLacs};
     
     //     once for each box size or image info line, whichever is more   
     //          write the information in the first column, and                      
     //                 the data in the following columns                                    
     int end = es>Info.length?es: Info.length;
     for (int i = 0; i < end; i++)
     {
         S=S+ (i<Info.length?Info[i]+tab :" "+tab) +
             (i<es    ?  (MakeStrings (n, i, V.GridPositions))
                           :  " "
              );
         S=S+newline;
     }
     //           Add a string for the slopes below each column                    
     S=S+newline+
             "Slopes: "+"slope [ln Y/ln "+epsilon+"]" +
             TabSlopeVsE(n);    
     S=S+newline;
     //                    Draw Lacunarity Graphs                                              /
     if(V.GraphLac) logPlots
              ( new float [][]{Lacs[0], bLacs[0]},
                new String []{"I"+lambda+"=1+"+cvsq+":"+Titles[LOC],
                "IBPD "+lambda+"=1+"+cvsq+":"+Titles[LOC]},
                 LOC);
    return S;
}

/**Returns a string of formatted and tabbed values for each array at
 * A[][e]. If the passed array is {{m00,m01,m02}, {m10,m11,m12}}
 * and e is 1, the string is "m01+tab+m11+tab". Adds a blank space
 * if the index is out of bounds for an array.
 * @param e int for the index of the values to string together
*  @param A float [][] array to make a string from 
 * @return string for each A[i to length], the value in A[i][g][e]+tab
*  for each A[i][g to length]
 * @see FLUtilities.FLutil#fnum formatter*/
String MakeStrings (float [][] A, int e)
{
     String w="";    
     for (int i = 0; i < A.length; i++)
     {         
                if (e>=A[i].length)w=w+" "+tab;
                else  w=w+FLutil.fnum(A[i][e])+tab;                 
     }
     return w; 
}

/**Returns a string of tabbed values for each array at A[][][e]. If the 
* passed array is <pre>{{ {m00,m01,m02}, {m10,m11,m12}},
*   { {s00,  s01,  s02},   {s10, s11,  s12}}}</pre> and e is 0, the 
* string is "m00+tab+m10+tab+s00+tab+s10".  Adds a blank
 * space if the index is out of bounds for an array.
* @param A float [][][] 
* @return string for each A[i to length], the value in A[i][g][e]+tab
* for each A[i][g to length]*/
String MakeStrings (float [][][] A, int e,  int grids)
{
     String w="";    
     for (int i = 0; i < A.length; i++)
     {
         for (int g = 0; g < grids; g++)      
         {         
            if (e<A[i][g].length) w=w+FLutil.fnum(A[i][g][e])+tab;                 
            else w = w+" "+tab;
         }
     }
     return w; 
} 

/**Returns the slopes of the passed arrays, separated by tabs, 
 * against the value stored in {@link #Epsilons} at [0].
* @param a float [datatypes][] of values at &epsilon; 
*  from which to find the {@link 
 * FracStats#SlopeOfPowerRegression log vs log slopes}.
* @return String of slopes with tabs between them */
String TabSlopeVsE0(float a[][])
{    
     FracStats f = new FracStats ();
     String P="";
     for (int i =0; i < a.length; i++)
     {
        P=P+FLutil.fnum( f.SlopeOfPowerRegression (a[i], 
                    Epsilons[0], Epsilons[0].length))+tab;        
     }
     return P;
}

/**Returns the slopes of the passed arrays, separated by tabs.
* @param a float [datatypes][grid][epsilon] of values at &epsilon; for 
* each grid, from which to find the 
* {@link FracStats#SlopeOfPowerRegression log vs log slopes}.
* @return String of slopes with tabs between them
* @param g int for which grid location*/
String TabSlopeVsE(float a [][][])
{           
     FracStats f = new FracStats ();
     String P="";
     for (int i =0; i < a.length; i++)
     {
        for (int g = 0; g < a[i].length; g++)
        {          
              P=P+FLutil.fnum( f.SlopeOfPowerRegression( a[i][g], 
                      Epsilons[g], Epsilons[g].length))+tab;                               
          
         }
     }
     return P;
}

/**Returns a string arranged as rows of statistics of the passed 
* arrays, each row headed by a title, and its contents separated by 
* tabs, in columns corresponding to the passed Y array's data 
* arrays.
* @param x float [] for determining power regression
* @param y float [g][e][value] of values at e for each g, from which to
* find the {@link FracStats#PlainLinearRegression slopes}
*@return String of slopes, correlation coefficient, and standard 
* error for the {@link FracStats#PlainLinearRegression regression
* line} vs Q, and the {@link FLstatistics#CoefVar coefficient of 
* variation (&sigma;/&mu;)} for the data*/
String TabbedStats(float y [][], float x[])
{           
String[] h={"Slope: Y vs Q",rsq+"vs Q","Standard Error (vs Q)",cv}; 
FracStats f = new FracStats ();
int [] I={f.SLOPEindex, f.RSQindex, f.STDERRindex};
String P=newline;
float [][] r = new float [y.length][];
float [] cv = new float [y.length];
for (int i =0; i < y.length; i++)
{
      r[i] = f.PlainLinearRegression (y[i], x, x.length);  
      cv[i]=FLUtilities.FLstatistics.CoefVar (y[i], y[i].length);
} 

 //make a string of tabbed regression stats for each array
 for (int i = 0; i < h.length; i++)
 { 
     P=P+newline;
     P=P+h[i]+tab;
//print the heading in every first column of each row
     //then print a row of the same stat for each column, 
     //and a row of CVs for each column in the last row 
        for (int j=0; j < y.length; j++)
        {  if (i==h.length-1)P=P+FLutil.fnum(cv[j])+tab;
                else P=P+FLutil.fnum(r[j][I[i]])+tab;
        }             
 }
 //Once for each statistic, write a row
 //once for each array, make a column
 //each row also has a heading and cv, slope, rsq, stderr
 return P;
}   

/**Draws plots of the log L[i][] vs log {@link #BOXSIZES &epsilon;}.
* The scale used on graphs is previously determined by {@link 
* FLVars#SameScale}.  
* @param L float [][] of lacunarity values to plot, one graph for 
* each array in the 2d array
* @param s String [] of titles for the graphs; the titles should
*  correspond to the order of the array of values in the 2d array so 
* that s[i] goes with L[i][]
 @param g int for grid*/ 
void logPlots( float [][] L,  String [] s, int g ) 
{
     float [][] logL=new float [L.length][];
     float [] Elog = new float [BOXSIZES[g].length];            
     for (int i = 0; i < BOXSIZES[g].length; i++)
     {
         Elog[i]=(float)Math.log (Epsilons[g][i]);                
     }
     boolean [] bad = new boolean [L.length];             
     for (int j = 0; j < L.length; j++)
         for (int i =0; i<BOXSIZES[g].length; i++)
         {  
             if (i==0)bad[i]=false;
             if (L[j][i]==0)   bad[i]=true;
         }
     for (int j = 0; j < L.length; j++)
     {  
        if (bad[j])
            logL[j]=FLutil.LogFloatArray(L[j], BOXSIZES[g].length);
        else logL[j]=FLutil.copyFloat (L[j], BOXSIZES[g].length);                          
     }
    float yMin=FLutil.minArray (logL[0]);
    float yMax=FLutil.maxArray (logL[0]);
    float xMin=FLutil.minArray (Elog);
    float xMax=FLutil.maxArray (Elog);
    if(V.SameScale)
     {
        for (int i = 0; i < L.length; i++)
         {
             yMin=Math.min(yMin, FLutil.minArray (L[i]));
             yMax=Math.max(yMax, FLutil.maxArray (L[i]));
         }                                     
     } 
     for (int k = 0; k < L.length; k++)
     {                                    
         String X="Ln "+epsilon;
         String Y="Ln "+lambda;
         if(bad[k]) Y=lambda;                          
         if(!V.SameScale)
         {
             yMin=FLutil.minArray (logL[k]);
             yMax=FLutil.maxArray (logL[k]);
         }
         PlotWindow P = new PlotWindow (s[k], X,  Y,  Elog,  logL[k]);
         P.setLimits (xMin, xMax, yMin, yMax);
         P.setLineWidth (1);
         P.setColor (LacColors[colorcounter].darker ());            
         P.addPoints (Elog, logL[k], P.BOX);
         P.setColor (LacColors[colorcounter].darker ());
         P.draw ();
     }             
 }



/** Returns four arrays all as long as the number of Qs, containing
* data for calculating the multifractal spectra according to the 
* method of Chhabra and Jensen. The data are for the &epsilon; at 
* which they were gathereD.data.  The returned array contains 
* sums to the exponentQ:<ul><LI>let i = the i<sup>th</sup> 
* box of j boxes and P<sub>i</sub> = the pixels in i/total pixels 
* counted so the &#8721;P<sub>i</sub>=1</LI>
* <h4>NB at Q=1: &#8721;P<sub>i,&epsilon;</sub></sub>
* <sup>Q</sup>=&#8721;lnP<sub>i,&epsilon;</sub>P<sub>i,
* &epsilon;</sub></LI><LI>Otherwise &#8721;P<sub>i,
* &epsilon;</sub><sup>Q</sup>is just that</LI></h4>
*<LI>&mu;<sub>i,Q,&epsilon;</sub>=P<sub>i,&epsilon;
* </sub><sup>Q</sup>/&#8721;P<sub>i-j,&epsilon;</sub>
* <sup>Q</sup></LI><LI>&alpha;(Q)=&#8721;(&mu;<sub>i,Q
* ,&epsilon;</sub>*lnP<sub>i,&epsilon;</sub>) (later to be /ln
* &epsilon;)</LI><LI>$#402;(&alpha;<sub>Q</sub>)=&#8721;
* (&mu;<sub>i,Q,&epsilon;</sub>*ln&&mu;<sub>i,Q,&epsilon;
* </sub>) (later to be /ln&epsilon;) </LI><LI> Mass=&#8721;
* Mass<sup>Q</sup> /&#8721;Mass<sup>0</sup>(&#8721;
* Mass<sup>0</sup>=count)</LI></ul>The slopes are calculated
* in the next {@link #FinishQslopes function} that uses these data.
* @return an array having at [0] the &#8721;P <sub>i</sub>
* <sup>Q</sup>, at [1] &#8721;mass &tau; terms, at [2] 
* &#8721;&alpha; terms and at [3] &#8721;F(&alpha;) terms for
* each Q in the Qs array.
* @param P float [] of probability values
* @param QS float array of exponents
* @param M float array of masses*/
public float [][] SummedPAtEpsilon (float [] P, 
     float[] QS, float [] M)
{
 float [] SummedProbsToQ = FLutil.newArray (V.Qs.length, 0.0f);
 float [] DS = FLutil.newArray (V.Qs.length, 0.0f);
 float [] Mass = FLutil.newArray (V.Qs.length, 0.0f);
 float [] A = FLutil.newArray (V.Qs.length, 0.0f);
 float [] F = FLutil.newArray (V.Qs.length, 0.0f);
 float SummedMass = FLutil.SumArray (M,M.length);
 // IJ.showMessage("no ploblem");
 //if (M.length!=P.length)IJ.showMessage("ploblem");
 for (int q = 0; q <QS.length; q++)
 {
     //For each box
     //            Sum all the probabilities in the array
     //            to the exponent Q,// unless Q=1,
     for (int i = 0; i < P.length; i++)
     {
         if (QS[q]==1f)DS[q]=DS[q]+(float)Math.log (P[i])*P[i];
         else DS[q]= DS[q]+ (float)Math.pow (P[i], QS[q]);
         SummedProbsToQ[q]= SummedProbsToQ[q]+
                 (float)Math.pow (P[i], QS[q]);
     }
     for (int i = 0; i < M.length; i++)
     {
     Mass[q]=Mass[q]+(float)Math.pow((M[i]/SummedMass),
             QS[q]-1.0f);
     }
     for (int i = 0; i < P.length; i++)
     {
         //  Sum the alphas and f(alphas)
         //using the method of Chhabra and Jensen:
         //mu(i) = [p(i)^Q]/SUM[P(i to j)^Q]
         //alpha = SUM[mu*ln{P(i)}]/ln{epsilon}
         //f(alpha) = SUM[mu*ln{mu}]/ln{epsilon}
         float mu = (float)Math.pow (P[i], QS[q])/ SummedProbsToQ[q];
         A[q]=A[q]+ (mu*(float)Math.log (P[i]));
         F[q]=F[q]+ (mu*(float)Math.log (mu));
     }
     //          store the mean also
     Mass[q]=Mass[q]/(float)P.length;
 }
 float [][] ST = new float [4][V.Qs.length];
 ST[0]=DS;//this is now the Generalized Dimensions array
 ST[1]=Mass;
//this is the taus by an alternative mean method array
 ST[2]=A;//this is the alpha values array
 ST[3]=F;//this is the f at alpha values,
 //the multifractal spectrum array
 return ST;
}       
     
/**This function finishes preparing data that was gathered during 
* box counting for the final steps of multifractal analysis.  It 
* calculates the log-log slopes of regression lines for the passed 
* arrays against the stored values in &#949; {@link #Epsilons 
* array}. These are for the generalized dimension (D<sub>Q
* </sub>),&#945;, &#964;, and &#402;(&#945;) values for 
* multifractal spectra, and require the passed arrays to be raised to
* a set of exponents and {@link #SummedPAtEpsilon summed}.
* Invokes an instance of {@link FracStats FracStats} to calculate 
* the slopes of the passed arrays against &#949; which are 
* returned and should be {@link 
 * FLAnalyzer.FLPlotter#PlotFAndAlpha graphed} to reveal the 
 * multifractal spectra:<ul>
* <LI>&#964;<sub>Q</sub> vs &#949;<sup>-1</sup>
* = -{@link FracStats#df slope}</LI>
* <LI>&#945;<sub>Q</sub> vs Ln&#949;<sup>-1</sup>
* = -{@link FracStats#Pslope slope}</LI>
* <LI>F(&#945;<sub>Q</sub>) vs Ln &#949; <sup>-1</sup>
* = -{@link FracStats#Pslope slope}</LI>
* <LI>&#948;D<sub>Q</sub>= F(&#945;<sub>Q</sub>)-(Q*
* &#945;<sub>Q</sub>);</LI>
* <LI>D<sub>Q</sub>=&#948;D<sub>Q</sub>/(1-Q);</LI>
* </ul>
* @return float [] of &#948;D<sub>Q</sub>, D<sub>Q</sub>, 
* &#964;, &#945;, &#402;(&#945;)
* @see <ul><LI>A. Chhabra and R.V. Jensen, Direct Determination
* of the &#402;(&#945;) singularity spectrum, Phys. Rev. Lett.
* 62: 1327, 1989.</LI><LI>A. N. D. Posadas, D. Gimenez, M. 
* Bittelli, C. M. P. Vaz, and M. Flury, Multifractal Characterization
* of Soil Particle-Size Distributions, Soil Sci. Soc. Am. J. 65:1361
* 1367 2001</LI></ul>
* @param LOC int for the grid location
* @param E float array for &#949; (box sizes)
* @param N int for the number of box sizes to use
* @param QArray floats for the values of the arbitrary exponent Q
* @param SUMMEDPtoQatEPSILONS float [][] of &#8721; 
* probabilities<sup>Q</sup>per &#949;
* @param MEANTAU float [] of mean &#964; values
* @param ALPHA float [] of &#945;
* @param FatALPHA float [] of &#402;(&#945;) for generalized 
* dimensions*/
public float[][] FinishQslopes (
     float [][] SUMMEDPtoQatEPSILONS,
     float [][] MEANTAU,
     float [][] ALPHA,
     float [][] FatALPHA,
     int LOC,             
     float []E,
     int N,
     float [] QArray)
{        
float [][] Sumptq = FLutil.new2dArray (QArray.length, N, 0.0f);
float [][] SummedTau = FLutil.new2dArray (QArray.length, N, 0.0f);
float [][] SumAlpha=FLutil.new2dArray(QArray.length, N, 0.0f);
float [][] SumF = FLutil.new2dArray (QArray.length, N, 0.0f);         
float [] InvEpsilons = FLutil.InverseFloatArray (E, N);         
float [] LogInvEpsilons=FLutil.LogInverseFloatArray(E, N);

for (int e = 0; e < N; e++)
{
 for (int q = 0; q< QArray.length; q++)
 {
     Sumptq[q][e] =SUMMEDPtoQatEPSILONS[e][q];
     SummedTau[q][e] =MEANTAU[e][q];
     SumAlpha[q][e]= ALPHA[e][q];
     SumF[q][e]= FatALPHA[e][q];
 }             
}         
float []thisxDqs = FLutil.newArray (QArray.length, 0.0f);
float []thisDqs = FLutil.newArray (QArray.length, 0.0f);
float []thisTaus = FLutil.newArray (QArray.length, 0.0f);
float []thisAlphas = FLutil.newArray (QArray.length, 0.0f);
float []thisFs = FLutil.newArray (QArray.length, 0.0f);                  
FracStats QStats;
for (int q = 0; q < QArray.length; q++)
{
 if (QArray[q]==1)
 {
     QStats=new FracStats(Sumptq[q], LogInvEpsilons, N);
     thisDqs[q]=-QStats.Pslope;                 
     thisxDqs[q]=0.0f;
     thisTaus[q]=0.0f;                 
     QStats=new FracStats(SumAlpha[q],LogInvEpsilons, N);
     thisAlphas[q]=-QStats.Pslope;                 
     QStats = new FracStats ( SumF[q],  LogInvEpsilons, N);
     thisFs[q]=-QStats.Pslope;                
  }
 else
 {
     QStats = new FracStats ( SummedTau[q], E, N);
     thisTaus[q]=QStats.df;
     QStats =new FracStats ( SumAlpha[q], LogInvEpsilons, N);
     thisAlphas[q]=-QStats.Pslope;                 
     QStats =new FracStats ( SumF[q],  LogInvEpsilons, N);
     thisFs[q]=-QStats.Pslope;                 
     thisxDqs[q]=thisFs[q]-(V.Qs[q]*thisAlphas[q]);
     thisDqs[q]= thisxDqs[q]/(float)(1.0f-V.Qs[q]);                 
 }
}         
float [][] D ={thisDqs, thisxDqs,  thisTaus, thisAlphas, thisFs};
return D;
}


/**Returns one line of summary data per grid location.
* @return String the data in a string for display    
* @param ssddd String to start off line of data with image name and 
* roi coordinates if appropriate   
* @param i int for this location*/
public String SummaryLineAtThisLocation (int i,
 String ssddd )
{          
String data =ssddd   +tab           

+FLutil.fnum (D.data.countDF[i])+tab
// "Box Count Df" + tab +
+FLutil.fnum (D.data.LDF[i])+tab
+FLutil.fnum (D.data.countDFrsq[i])+ tab
// "r squared" + tab+
+FLutil.fnum (D.data.countDFstderr[i])+tab
//"Standard error" + tab+
+FLutil.fnum ( D.data.countDFyint[i])+tab
//"Y Intercept"+ tab+                     
+FLutil.fnum( D.data.Smootheddf[i]) +tab
//"Horizontal Slope Removed"+ tab+
+FLutil.fnum ( D.data.Smoothedrsq[i])+tab
//"HSR r squared" + tab+
+FLutil.fnum ( D.data.SmoothedStdderr[i])+tab

+FLutil.fnum (D.data.MassLacAtGrid[i])+tab
+FLutil.fnum (D.data.VariationInMassLacAtGrid[i])+tab

+FLutil.fnum (D.data.EMassLacAtGrid[i])+tab   
+FLutil.fnum (D.data.VariationInEMassLacAtGrid[i])+tab   

+FLutil.fnum (D.data.BPDLacAtGrid[i])+tab
+FLutil.fnum (D.data.VariationInBPDLacAtGrid[i])+tab       

+FLutil.fnum (D.data.EBPDLacAtGrid[i])+tab  
+FLutil.fnum(D.data.EVariationInBPDLacAtGrid[i])+tab

+FLutil.fnum (D.data.CountCVsqAtGrid[i]) + tab
+FLutil.fnum (D.data.EmptiesCVSqAtGrid[i])    + tab  
+FLutil.fnum
(D.data.CountCVsqAtGrid[i]/D.data.EmptiesCVSqAtGrid[i]) +tab
        +FLutil.fnum(D.data.SmoothedCountSizes[i].length);             

return data;
}

/**Returns one line of summary data for a sliding box lacunarity
 *scan.  The values reported coordinate with the headings in 
 *{@link #MakeSLACDataFileHeadings shown in the data file}. 
*   @return String the data in a string for display
*   @param ssddd String to start off line of data with image name  
*   roi coordinates if appropriate
*   @see #StoreSLACData calling function*/
public String SummarySLACLineAtThisLocation (String ssddd )
{          
 int i =0;//SLAC scans use one grid origin
String data =ssddd +tab        
+FLutil.fnum (-1*D.data.massDF[i])+tab
+"("+FLutil.fnum (D.data.LDF[i])+")"+tab
+FLutil.fnum (D.data.massDFrsq[i])+ tab
+FLutil.fnum (D.data.massDFstderr[i])+tab
+FLutil.fnum ( D.data.massDFyint[i])+tab
+FLutil.fnum (D.data.MassLacAtGrid[i])+tab
+FLutil.fnum (D.data.VariationInMassLacAtGrid[i])+tab
+FLutil.fnum (D.data.EMassLacAtGrid[i])+tab   
+FLutil.fnum (D.data.VariationInEMassLacAtGrid[i])+tab   
+FLutil.fnum (D.data.BPDLacAtGrid[i])+tab
+FLutil.fnum (D.data.VariationInBPDLacAtGrid[i])+tab       
+FLutil.fnum (D.data.EBPDLacAtGrid[i])+tab  
+FLutil.fnum(D.data.EVariationInBPDLacAtGrid[i])+tab     
+FLutil.fnum (D.data.CountCVsqAtGrid[i]) + tab
+FLutil.fnum (D.data.EmptiesCVSqAtGrid[i])    + tab 
+FLutil.fnum(D.data.CountCVsqAtGrid[i]/
        D.data.EmptiesCVSqAtGrid[i]) ;            

return data;
}

void initializeCircStats ()
{
    V.circularity=0;
    V.perimeter=0;
    V.major=0;
    V.minor=0;
    V.AREA=0;
    V.CircleDiameter=0;
}

/**Plots the data for and returns a string of the coefficient of 
* variation, correlation coefficient, and slope for the multifractal 
* scan chosen as {@link #bestLOC optimum}.
* @param GD float [][] an array of floats */
public String ShowOptimizedMF (float [][] GD)
{                
FracStats F = new FracStats ();
float []stats=F.PlainLinearRegression(GD[1],V.Qs,V.Qs.length);         
FLGraphics.drawsGeneralizedDimensions 
     ( V, GD, Titles[bestLOC]);                 
FLPlotter.PlotFAndAlpha
             (GD[3],GD[4], bestLOC, "Optimized " +
             Titles[bestLOC], V.Qs, V.isGraphFA ());             

return new String (
     newline+"Optimal Grid Position for "+Titles[bestLOC]
     +tab+cv+tab+"slope"+tab+rsq+
     newline+
     bestLOC+tab+FLutil.fnum (bestCV)
         +tab+FLutil.fnum (stats[F.DFSLOPEindex])
         +tab+FLutil.fnum (stats[F.RSQindex]));
}

/**Draws a plot of the passed &#402;<sub>(&alpha;(Q))</sub>
* vs &alpha;<sub>(Q)</sub> with &alpha; on the horizontal axis,
* for a multifractal spectrum.  Values for Q>0 and <0  are drawn in 
* different colours. Multifractal analysis exaggerates differences in 
* scaling by raising the distribution of pixels per box to a set of 
* exponents, Q, and recalculating fractal measures.  The 
* multifractal spectra graphically display this mathematical 
* distortion.</li><li>A typical &#402;(&alpha;) vs &alpha;
* multifractal spectrum is convex, humped upward (at least for 
* Q>0) and diverging.</li><li>A monofractal spectrum, 
* in contrast, converges.</li><li>The D<sub>Q=0</sub>
* corresponds to the box counting dimension.</li><li>The plot of D
* <sub>Q</sub> vs Q is generally decreasing for multifractals but 
* not monofractals.</li><li>Box counting sampling may not 
* represent the probability distribution, so the graphs may deviate 
* somewhat, especially for Q<0 where the distortion of very small
* probabilities is prominent.  The corrections employed here
* improve sampling through random mass sampling, shifting 
* {@link FLVars#GridPositions the grid}, {@link FLUtilities.FLutil
* #SmoothedArray smoothing the data} and 
* {@link FLUtilities.FLutil#minMassCover minimizing the 
* covering}. 
* @see #MultifractalDataProcessor
* @param alphaz array of floats for &alpha;
* @param Fatalphaz float [] for &#402;(<sub>&alpha;</sub>)
* @param type string specifying the type of plot (e.g., slope 
* corrected or minimum cover)
* @param imgtitle String for the image's title
* @param Qs float array of values for the arbitrary 
*  exponent Q that was used to make the &alpha;s
*  and &#402;(&alpha;s)*/
public void PlotFAndAlpha (
     float []alphaz,
     float[] Fatalphaz,
     String type,             
     String imgtitle,
     float [] Qs)
{
    float amax = alphaz[0];
    float amin = amax;
    float fmax = Fatalphaz[0];
    float fmin = fmax;
    float dqat0=0;
    int negs = 0;
    int pos = 0;
    for (int c = 0; c < Qs.length; c++)
    {
     if (Qs[c]<=0)negs++;
     if(Qs[c]>=0)pos++;
     if (Qs[c]==0)dqat0=Fatalphaz[c];
    }
    float [] pA = FLutil.newArray (pos, 0.0f);
    float [] nF = FLutil.newArray (negs, 0.0f);
    float [] nA = FLutil.newArray (negs, 0.0f);
    float [] pF = FLutil.newArray (pos, 0.0f);
    int n = 0, a = 0;
    for (int u=0; u <Qs.length; u++)
    {
         if(Qs[u]>=0)
         {
             pA[a]=alphaz[u];
             pF[a]=Fatalphaz[u];
             a++;
         }

         if(Qs[u]<=0)
         {
             nA[n]=alphaz[u];
             nF[n]=Fatalphaz[u];
             n++;
         }
    }

    for (int q = 0; q < Qs.length; q++)
    {             
         amin = Math.min (alphaz[q], amin);
         amax = Math.max (alphaz[q], amax);
         fmax = Math.max (Fatalphaz[q], fmax);
         fmin = Math.min (Fatalphaz[q], fmin);
    }         
    float line = fmax;         
    String title = imgtitle;
    amin=0;
    amax=3;
    fmin = 0;
    fmax = 2;
    float []x =            {amin, amax};
    float []y=            {fmax, fmax};

    String label = type+ "; Q("+FLutil.fnum (Qs[0])+" to "
         + FLutil.fnum (Qs[Qs.length-1])+")";
    PlotWindow pmf = new PlotWindow ( imgtitle, alpha, falpha, x, y);
    pmf.setLimits (amin, amax, fmin, fmax);
    pmf.addLabel (.2, .1, "max="+FLutil.fnum (line));
    pmf.addLabel (.2, .2, "q=0: "+FLutil.fnum (dqat0));
    pmf.draw ();
    pmf.setLineWidth (1);
    pmf.setColor (Color.green);
    pmf.addPoints (pA, pF, PlotWindow.CROSS);         
    pmf.addLabel (.2, .4, "Q>=0: GREEN X");
    pmf.setLineWidth (1);
    pmf.setColor (Color.red);
    pmf.addPoints (nA, nF,  PlotWindow.CIRCLE);
    pmf.addLabel (.7, .4, "Q<=0: RED O");
    pmf.addLabel (.6, .8, label );
    pmf.draw ();
}


/**Plots the data in X,Y and shows the slope on the graph.
* @see FLAnalyzer.FracStats#Pslope
* @param setmin boolean true makes the minimum 0 and the 
* maximum 3
* @param X float[] of points to plot
* @param Y float[] of point to plot
* @param type string that is printed on the graph     
* @param imgtitle String for the image's title to print on the graph
* @param xaxis String for the x axis
* @param yaxis String for the y axis*/
public void plotXY (
     boolean setmin,
     float []X,
     float[] Y,
     String type,           
     String imgtitle,
     String xaxis,
     String yaxis )
{
    float xmax = X[0];
    float xmin = X[0];
    float ymax = Y[0];
    float ymin = Y[0];

    for (int q = 0; q < X.length; q++)
    {
         xmin = Math.min (X[q], xmin);
         xmax = Math.max (X[q], xmax);
         ymax = Math.max (Y[q], ymax);
         ymin = Math.min (Y[q], ymin);
    }
    if(setmin) {ymin=0; ymax=3;}
    FracStats F=new FracStats (Y, X, X.length);
    float slope=F.Pslope;
    float []x = {xmin, xmax};         
    float []y={ymax, ymax};
    String title =    imgtitle;
    PlotWindow pmf = new PlotWindow (title, xaxis,  yaxis, x, y);
    pmf.setLimits (xmin, xmax, ymin, ymax);
    pmf.addLabel (.2, .1, "slope "+FLutil.fnum (slope));
    pmf.setLineWidth (1);
    pmf.setColor (Color.black);
    pmf.addPoints (X, Y, PlotWindow.TRIANGLE);
    pmf.addLabel (.4, .2, type);
    pmf.draw ();         
}


/**Returns a string of headings for printing binary scan data. Makes 
* a tab separated string from  the passed array of strings, where 
* each member of that list is numbered and repeated once for each
* {@link FLVars#GridPositions grid}.
* @return String of headings for printing data
* @param H String array to make into a tabbed string
 @param g for the number of times to repeat a heading*/
public String MakeMultipleHeadings (String [] H, int g)
{
    String locs = "";
    String head = "";        
    for (int p=0; p < H.length; p++)
    {
         head = H[p];
         for (int y =1; y<=g; y++)
             locs = locs + head  + y + tab;                                     
    }
    return locs;
}       

/**Returns the string "slope [ln(s)/ln(e)]" where e is *&epsilon;
*@param s String for slope string
*@return String saying slope vs epsilon*/
String slopeVsE(String s)
{
 return new String ("slope [ln("+s+")/ln("+epsilon+")]");
}

/**Returns the string "(Sum [J])/Number of Box Sizes"
where Sum is &#8721;
@param J String to use in equation string
@return formatted string for mean */
String  meanOverBoxes (String J)
{return new String 
     ("("+FLMain.sum+"["+ J+"])/Number of Box Sizes");
}


//========     writes regular scan data files   ========//


/**Returns a string and makes headings of data for box counts.
* The string has {@link #MakeMultipleHeadings headings} then 
* {@link #MakeStrings columns} of {@link #BOXSIZES box sizes}
* and {@link #Epsilons &epsilon;s} with the corresponding data 
* and {@link #TabSlopeVsE slopes} for  various variables at 
*  each box size and grid position.  It is supposed to be appended 
* to {@link FLVars#DataString the data string} to be shown 
* in the data file. Uses data stored in {@link 
* FLAnalyzer.FLData#CountAtEpsilon}, 
* {@link FLAnalyzer.FLData#EmptiesAtEpsilon},
* {@link FLAnalyzer.FLData#MassLacMeanAtEpsilon F lacunarity},
* {@link FLAnalyzer.FLData#MassLacStdDevAtEpsilon},
* {@link FLAnalyzer.FLData#MassLacAtEpsilon},   
* {@link FLAnalyzer.FLData#EMassMeansAtEpsilon},
* {@link FLAnalyzer.FLData#EMassStdDevAtEpsilon},
* {@link FLAnalyzer.FLData#EMassLacAtEpsilon E lacunarity}, 
* {@link FLAnalyzer.FLData#MeanOfBPDAtEpsilon},  
* {@link FLAnalyzer.FLData#StdDevOfBPDAtEpsilon},
* {@link FLAnalyzer.FLData#BPDLacAtEpsilon BPD lacunarity},
* {@link FLAnalyzer.FLData#EMeanOfBPDAtEpsilon},
* {@link FLAnalyzer.FLData#EStdDevOfBPDAtEpsilon},
* {@link FLAnalyzer.FLData#EBPDLacAtEpsilon EBPD lacunarity}) 
* For lacunarity values, {@link FLutil#Plus1 adds 1} prior to 
* printing.
* @return String of multiple lines of data for display */
 public String MakeColumnsOfDataAndSlopes ()
 { 
    int es = FLutil.LongestArray (BOXSIZES);
    int grids = V.GridPositions;
    float [][][] n =
          {
                FLutil.IntToFloatArray (BOXSIZES), 
                Epsilons,  
                D.data.CountAtEpsilon,
                D.data.EmptiesAtEpsilon,
                D.data.MassLacMeanAtEpsilon,
                D.data.MassLacStdDevAtEpsilon,
                FLutil.Plus1 (D.data.MassLacAtEpsilon),   
                D.data.EMassMeansAtEpsilon,
                D.data.EMassStdDevAtEpsilon,
                FLutil.Plus1 (D.data.EMassLacAtEpsilon), 
                D.data.MeanOfBPDAtEpsilon,  
                D.data.StdDevOfBPDAtEpsilon,
                FLutil.Plus1 (D.data.BPDLacAtEpsilon),
                D.data.EMeanOfBPDAtEpsilon,
                D.data.EStdDevOfBPDAtEpsilon,
                FLutil.Plus1 (D.data.EBPDLacAtEpsilon)};

    String [] slopeh=new String []
    { 
        "Box Size",
         epsilon,
        "D(counts)="+slopeVsE("F")+": ",                       
        slopeVsE("E")+":",
        "D(mass)="+slopeVsE( "F"+FLMain.mu)+": ",
        slopeVsE("F"+FLMain.sigma) + ": ",
            slopeVsE("F"+lambda)+": ",
        slopeVsE("E"+FLMain.mu)+": ",
        slopeVsE("E"+FLMain.sigma)+": ",
        slopeVsE ("E"+lambda)+": ",
        "D="+slopeVsE("FBPD"+FLMain.mu)+": ",
        slopeVsE("FBPD"+FLMain.sigma)+": ",
        slopeVsE("FBPD"+lambda)+": ",
        slopeVsE("EBPD"+FLMain.mu)+": ",
        slopeVsE("EBPD"+FLMain.sigma)+": ",
        slopeVsE("EBPD"+lambda)+": "
    };

    String S=newline
    +"Slopes for Data"+newline    
    +MakeMultipleHeadings (slopeh, grids)+newline   
    +TabSlopeVsE(n)+                 
    newline+"DATA"+newline+
    MakeMultipleHeadings(DataColumnHeadArray, grids)
    +newline;     

    //for each box size, write a line in each data column
    for (int m=0; m<es; m++)
    {
         S = S+MakeStrings(n, m, grids)+newline;
    }                     
    S=S+ (newline);                     
    return S;
 }
 
String [] DataColumnHeadArray = {
"Box Size",
epsilon+"=Box Size/Image Dimension",
"F=boxes containing foreground pixels: Location ",
"E=all boxes, including empties, on image: Location ",
"F"+mu+"=mean mass: ",
"F"+sigma+ "=standard deviation of mass: ",
"F"+lambda+"=1+"+FLVars.cvsq+ ": ",
"E"+mu+"=mean mass including empties : ",
"E"+sigma+ " :",
"E"+lambda+"=1+"+FLVars.cvsq+ " : ",
"FBPD"+mu+"" +
        "=mean mass from binned " +
        "probability distribution (BPD) foreground pixels: ",
"FBPD"+sigma+ " : ",
"FBPD"+lambda+"=1+"+FLVars.cvsq+ " : ",
"EBPD"+mu+"=mean mass from BPD including empties : ",
"EBPD"+sigma+ " : ",
"EBPD"+lambda+"=1+"+FLVars.cvsq+ " : "
 };
/**Returns a string and makes headings of data for box counts.
* The string has {@link #MakeMultipleHeadings headings} then 
* {@link #MakeStrings columns} of {@link #BOXSIZES box sizes}
* and {@link #Epsilons &epsilon;s} with the corresponding data 
* and {@link #TabSlopeVsE slopes} for  various variables at 
*  each box size and grid position.  It is supposed to be appended 
* to {@link FLVars#DataString the data string} to be shown 
* in the data file. Uses data stored in {@link 
 * FLAnalyzer.FLData#CountAtEpsilon}, 
 * {@link FLAnalyzer.FLData#EmptiesAtEpsilon},
* {@link FLAnalyzer.FLData#MassLacMeanAtEpsilon F lacunarity},
* {@link FLAnalyzer.FLData#MassLacStdDevAtEpsilon},
* {@link FLAnalyzer.FLData#MassLacAtEpsilon},   
* {@link FLAnalyzer.FLData#EMassMeansAtEpsilon},
* {@link FLAnalyzer.FLData#EMassStdDevAtEpsilon},
* {@link FLAnalyzer.FLData#EMassLacAtEpsilon E lacunarity}, 
* {@link FLAnalyzer.FLData#MeanOfBPDAtEpsilon},  
* {@link FLAnalyzer.FLData#StdDevOfBPDAtEpsilon},
* {@link FLAnalyzer.FLData#BPDLacAtEpsilon BPD lacunarity},
* {@link FLAnalyzer.FLData#EMeanOfBPDAtEpsilon},
* {@link FLAnalyzer.FLData#EStdDevOfBPDAtEpsilon},
* {@link FLAnalyzer.FLData#EBPDLacAtEpsilon EBPD lacunarity}) 
* For lacunarity values, {@link FLutil#Plus1 adds 1} prior to 
* printing.
* @return String of multiple lines of data for display */
 public String MakeColumnsOfSLACDataAndSlopes ()
 { 
    float [][] n =
              {
                    FLutil.IntToFloatArray (BOXSIZES[0]), 
                    Epsilons[0],  
                    D.data.CountAtEpsilon[0],
                    D.data.EmptiesAtEpsilon[0],
                    D.data.MassLacMeanAtEpsilon[0],
                    D.data.MassLacStdDevAtEpsilon[0],
                    FLutil.Plus1 (D.data.MassLacAtEpsilon[0]),   
                    D.data.EMassMeansAtEpsilon[0],
                    D.data.EMassStdDevAtEpsilon[0],
                    FLutil.Plus1 (D.data.EMassLacAtEpsilon[0]), 
                    D.data.MeanOfBPDAtEpsilon[0],  
                    D.data.StdDevOfBPDAtEpsilon[0],
                    FLutil.Plus1 (D.data.BPDLacAtEpsilon[0]),
                    D.data.EMeanOfBPDAtEpsilon[0],
                    D.data.EStdDevOfBPDAtEpsilon[0],
                    FLutil.Plus1 (D.data.EBPDLacAtEpsilon[0])};
     String [] slopeh=new String []
     { 
            "Box Size",
             epsilon,
            "D(counts)="+slopeVsE("F")+": ",                       
            slopeVsE("E")+":",
            "D(mass)="+slopeVsE( "F"+FLMain.mu)+": ",
            slopeVsE("F"+FLMain.sigma) + ": ",
            slopeVsE("F"+lambda)+": ",
            slopeVsE("E"+FLMain.mu)+": ",
            slopeVsE("E"+FLMain.sigma)+": ",
            slopeVsE ("E"+lambda)+": ",
            "D="+slopeVsE("FBPD"+FLMain.mu)+": ",
            slopeVsE("FBPD"+FLMain.sigma)+": ",
            slopeVsE("FBPD"+lambda)+": ",
            slopeVsE("EBPD"+FLMain.mu)+": ",
            slopeVsE("EBPD"+FLMain.sigma)+": ",
            slopeVsE("EBPD"+lambda)+": "
    };
 
     String S=newline
     +"Slopes for Data"+newline    
     +MakeMultipleHeadings (slopeh, 1)+newline
     +TabSlopeVsE0(n)+                 
     newline+"DATA"+newline+
     MakeMultipleHeadings(DataColumnHeadArray, 1)
     +newline;     
     
     //for each box size, write a line in each data column
     for (int m=0; m<Epsilons[0].length; m++)
     {
         S = S+MakeStrings(n, m)+newline;
     }                     
     S=S+ (newline);                     
     return S;
 }
 
//========             grayscale data writers         ========//
     

/**Prepares a string of data from grayscale scans.  Uses data 
* {@link FLDataProcessor#FindGrayDf calculated}
* and stored in data arrays:<ul>
* {@link FLData#graySumOfBoxMassesAtEpsilon},
* {@link FLData#grayCountAtEpsilon},
* {@link FLData#grayMassLacMeanAtEpsilon}, 
* {@link FLData#grayMassLacStdDevAtEpsilon},
* {@link FLData#grayMassLacAtEpsilon}
* {@link FLutil#Plus1 +1},
* {@link FLData#MeanOfBPDAtEpsilon},
* {@link FLData#StdDevOfBPDAtEpsilon},
* {@link FLData#BPDLacAtEpsilon}
* {@link FLutil#Plus1 +1},
* {@link FLData#cvsqBPDLacAtEpsilon}
* {@link FLutil#Plus1 +1}</ul>
* @return String of slopes with columns of data*/
public String DataAndSlopesforGrayscaleScans()
{        
    int grids = V.GridPositions;
    int es = FLutil.LongestArray (BOXSIZES);
    String [] Sh = { 
         "Box Size",
         epsilon,
         slopeVsE(sum+" I"),
         slopeVsE(sum+" Boxes"),
         slopeVsE(mu+" I"),                 
         slopeVsE(sigma+" I"),                         
         slopeVsE(lambda),     
         slopeVsE(mu+"I BPD"),
         slopeVsE(sigma+"I BPD"),
         slopeVsE("BPD "+lambda),  
         slopeVsE("1+"+FLVars.cvsq+" BPD")                
        }; 

    float [][][] n =new float [][][] {
        FLutil.IntToFloatArray (BOXSIZES),
        Epsilons,
        D.data.graySumOfBoxMassesAtEpsilon,
        D.data.grayCountAtEpsilon,
        D.data.grayMassLacMeanAtEpsilon, 
        D.data.grayMassLacStdDevAtEpsilon,
        FLutil.Plus1 (D.data.grayMassLacAtEpsilon),
        D.data.MeanOfBPDAtEpsilon,
        D.data.StdDevOfBPDAtEpsilon,
        FLutil.Plus1 (D.data.BPDLacAtEpsilon),
        FLutil.Plus1 (D.data.cvsqBPDLacAtEpsilon)
    };
    
    String S=
    newline+
    "Slopes (Power Regression)"
    +newline
    +V.StringForIntensityCalculation(V.grayscantype)
    +newline+   
    MakeMultipleHeadings(Sh, grids) 
    +newline
    + TabSlopeVsE(n)    + newline
    +"DATA"
    +newline+   
     MakeMultipleHeadings(ColumnHeadingsForGrayscaleData, grids)
    +newline;                     

 for (int m=0; m<es; m++)
 {
     S=S+     
     MakeStrings(n , m, grids)
     +newline;
 }
 return S;
}


//==========end of grayscale data writers==========//  
String [] ColumnHeadingsForGrayscaleData = {
                    bs,
                    epsilon,
                    sum+" I ", 
                    "Boxes",
                    mu+" I ", 
                    sigma+" I  ",                          
                    lambda+"=1+"+FLVars.cvsq+" I ", 
                    mu+" for "+bpd+" (BPD) I ",  
                    sigma+"BPD for I ",  
                    "BPD "+lambda,
                    lambda+"=1+"+FLVars.cvsq+" BPD for I "                  
    };

/**Calculates a series of box sizes that divide evenly into the  largest 
* size and are no larger than the smallest, with a buffer of 5 around 
* the smallest size.  Is called by {@link #MakeNewBoxSizes main 
* box making method} if the series is a relative series.
* @param max maximum box size
*  @param min  minimum box size
*  @return int [] box sizes*/
public int [] MakeBlockBoxSizes(int max, int min)
{
     int a [] = new int[max];
     a[0]=max;
     int j = 1;
     int MIN = min<=5?1:min-5;
     if(V.isGray()&&min<2)min=2;
     for (int i = 2; i<=max;  i++)
     {
         int test = max%i;
         if(test==0&&((max/i)>=MIN)){a[j]=max/i;j++;}                         
     } 
     //keep everything ~< min by keeping a boxsize if 
     //it is within 10 of the min
     int [] b = new int [j];
     int m = j-1;
     for (int k = 0; k < b.length; k++)
     {
         b[k]=a[m];
         m--;                         
     }
     return b;
}
/**Makes a series of {@link #BOXSIZES box sizes}, 
* and returns the number of values in it. Different {@link 
 * FLVars#Series types of series} are made. 
* The minimum size, maximum size, and 
* increment between boxes are calculated from user inputs  and the
* size of the current ROI or image (explained below). 
* For box counting analyses, the upper limit
* is calculated as a percentage of the image's size (evidence 
* suggests a practical value is around 45%). The lower limit should 
* reflect the resolution in the image (i.e., not be smaller than the 
* limit of resolution in pixels) but if not known must be estimated 
* (e.g., as the {@link FLAnalyzer.FLVars#UserMinBoxSize  default}
* ).  The final series of integers is subject to an unaccessible {@link
*  FLAnalyzer.FLVars#absolutemaximumnumberofboxes maximum
* } and {@link FLAnalyzer.FLVars#leastpossibleboxes minimum}
* number of boxes. <h2>Linear Series Calculations:</h2><ul>
* <LI>limit = passed value for Across
 * {@link FLAnalyzer.FLVars#maxpercent}/100</LI><LI>
 * or if this is a subsample, uses{@link 
* FLAnalyzer.FLVars#RandomMassMaxSubpercent} /100</LI>
* <LI>Maximum and number of boxes are set to this value
* temporarily.</LI><LI>If the user has chosen to set the number 
* of boxes, number is reset using {@link 
 * FLAnalyzer.FLVars#MaxNumBoxSizes the user's value}.</LI>
 * <LI>The number of  boxes is checked against the {@link 
 * FLAnalyzer.FLVars#leastpossibleboxes least} and {@link 
 * FLAnalyzer.FLVars#absolutemaximumnumberofboxes most}
 * permitted and reset if 
* required.</LI><LI>Then the minimum size is set at 1 or the 
* user's choice for {@link FLAnalyzer.FLVars#UserMinBoxSize
* minimum} size.</LI><LI>The increment between boxes is set as
* the (max-min)/number of boxes, corrected to never  be less than 1
* .</LI><LI>The box sizes are calculated starting from the 
* minimum until the maximum, adding the increment each time.
* </LI><LI>The last box is  reset to the maximum box size
* previously determined.</LI><LI>The &epsilon; array is 
* calculated as box size/L, where L = the {@link 
 * FLAnalyzer.FLVars#GreaterOfHtAndWd dimension} 
 * of the foreground pixels of the image.</LI></ul><p>Reinitializes 
* arrays</p>For a {@link FLVars#RELATIVESERIES relative 
* series}, the {@link #CheckRelative relative} box sizes method 
* is called. For a {@link FLVars#POWERSERIES}, calculations are 
* based on  {@link FLVars#exponent the exponent} and {@link 
* FLVars#base} in the {@link #getPowerSeries power series 
* method}. A {@link FLVars#SCALEDSERIES scaled series} is 
* made using the {@link #getScaledSeries scaled calculatoar}. 
* @return int for number of boxes in the series             
* @param Across float for distance across image*/
public int MakeNewBoxSizes (float Across)
{
 //====   set the upper limit on box size    ======//
 int P=  V.UseGreater?(int)V.GreaterOfHtAndWd: (int) Across;                     
 V.limit = (int)(P*(V.maxpercent/100.0f));                     
 if(V.getUserMaxBoxSize ()>0) V.limit=V.getUserMaxBoxSize ();                         
 if (V.isVerticalLine||V.isHorizontalLine)
      if(V.getUserMaxBoxSize()==0)
          V.limit=(int)(V.GreaterOfHtAndWd *(V.maxpercent/100.0f));
 //===     for random mass samples,     =======// 
 // ==     set the limit to a percentage of the roi   = //
 if (V.RandomMassSample) 
     V.limit=(int)(P*(V.RandomMassMaxSubpercent/100.0f));                                              
 if (V.limit<1)V.limit=1;
 //==   Set Maximum SIZE and NUMBER of boxes ==//
 int MaxBoxSize=V.limit;                     
 int numboxes = MaxBoxSize;                     
 if(V.isUserWantsToSpecifyNumberOfBoxes())                     
            numboxes = V.getMaxNumBoxSizes ();                                                                              
 if(numboxes<V.leastpossibleboxes)numboxes=V.leastpossibleboxes;                     
 else if (numboxes>V.absolutemaximumnumberofboxes)
            numboxes=V.absolutemaximumnumberofboxes;
 //===      Set the MINIMUM box size   ========//
 if(V.getUserMinBoxSize ()<1)V.setUserMinBoxSize (1);
 if (V.isGray()) V.setUserMinBoxSize (2);
 if (MaxBoxSize<=V.getUserMinBoxSize ())
     MaxBoxSize= V.getUserMinBoxSize()+1;                                          
 //====   adjust the max if necessary  ========//
 //=====     set the INCREMENT    ======== =//
 int increment = (int)((float)(MaxBoxSize-V.getUserMinBoxSize())
         /(float)numboxes);
 if (increment<1)increment=1;     
 //===       MAKE NEW BOX SIZES    =========//
 int [] BS = new int[numboxes];
 int index = 1;       
 //===    MAKE a power series of  BOX SIZES  ====//
 if(V.Series==V.POWERSERIES)                     
 {
     BS=getPowerSeries (numboxes, V.exponent, V.base, 
             MaxBoxSize, V.isGray());
     index=BS.length;
 }
 //===  or  a SCALED SERIES of  BOX SIZES ====//
 else if (V.Series==V.SCALEDSERIES)
 {  
     float M=(V.ScaledMaxBoxSize==0)?
         MaxBoxSize : V.ScaledMaxBoxSize;
     int min = V.getUserMinBoxSize();
     if (V.isGray()&&min<2)min=2;
     BS=getScaledSeries(V.BoxSizeScaleDenominator/
             V.BoxSizeScaleNumerator, M,min, V.isGray()) ;  
     index=BS.length;
 }
 else
 {//=====  or  a DEFAULT LINEAR series ======//
     BS[0]=V.getUserMinBoxSize();   
     if(V.isGray()&&BS[0]<2)BS[0]=2;
     for (index = 1;((BS[index-1]<V.limit)&&(index<numboxes ));
        index++)
     {
         BS[index] = BS[index-1] + increment;
     }
     if(index>1)BS[index-1]=V.limit;
     else V.limit=BS[index-1];
 }
 //=====  correct for too few boxes in a series    ===//
 if((!V.isMF ())&&!V.SLAC)
 {if (index<V.leastpossibleboxes 
          &&!V.isUserWantsToSpecifyNumberOfBoxes())
  {
      int [] newarray=new int  [V.leastpossibleboxes];
      int mn = V.isGray ()?2:1;
      for (int k = 0; k < V.leastpossibleboxes; k++)
      {
          newarray[k]=k<index?BS[k]>
                  mn?BS[k]:mn:newarray[k-1]+increment;
      }
      BS=new int[V.leastpossibleboxes];                          
      for (int j = 0; j<V.leastpossibleboxes; j++) BS[j]=newarray[j];
      V.limit=BS[BS.length-1];
      index=V.leastpossibleboxes;                          
  }
 }
 //=====  correct  for user settings  ======//
 if(V.isUserWantsToSpecifyNumberOfBoxes()
         &&index<V.getMaxNumBoxSizes ())
 {
     if(V.Series==V.DEFAULTBOXSIZES)
     {
         int [] Temp= new int[V.getMaxNumBoxSizes ()];
         for (int i = 0;  i < V.getMaxNumBoxSizes ();i++)
         {
             if (i<index) Temp[i]=BS[i];
             else Temp[i]=Temp[i-1]+increment;
         }
         BS=new int [Temp.length];
         for (int i = 0; i < BS.length; i++) BS[i]=Temp[i];
         index=Temp.length;
        //make as many boxes as requested
     }
 } 
 //===  or MAKE a RELATIVE series of  BOX SIZES ==//
 if(V.BLOCKTEXTUREANALYSIS||V.isRelative ())
 {          
     BS=CheckRelative();
     index = BS.length;                         
 }
 //              Accomodate horizontal or vertical lines              /
 else if(changedSeries)
 {          
    boolean change=true;    
    if(!V.isRelative ()) if(!IJ.showMessageWithCancel ("Change Series",
            "Use optimal series for this image?")) change=false;
     if (change)
     {
        BS=CheckRelative();
        index = BS.length;                         
      }
 }
 //             Make sure gray scale scans do not use a box size 
 //       smaller than 2, or all boxes will have the same delta of 0
 if(V.isGray()&&BS[0]==1) 
 {
     BS=loseOne(BS);
     index=BS.length;
 }
 //=====  make the EPSILON array   ======//
 Epsilons = new float[V.GridPositions][index];
 BOXSIZES = new int[V.GridPositions][index];
 float L = (float)V.GreaterOfHtAndWd;                     
 if(V.RandomMassSample) V.Subboxsize
         =(int)((V.percentsubboxsize/100.00f)*L);  
 for (int g=0 ; g < V.GridPositions; g++)
 for (int m = 0; m<index; m++)
 {
     Epsilons[g][m] =(float)BS[m]/L;
     BOXSIZES[g][m]=BS[m];
 }
 return index;
}

/*Returns the array without its first entry.
 * @param BS int []
 * @return int [] of the same values except the first */
int [] loseOne(int [] BS)
{
    int [] b = new int [BS.length-1];  
    for (int i = 0; i<b.length; i++)b[i]=BS[i+1];
    return b;
}

/**Returns a series of box sizes made using the {@link 
* #MakeScaledSeries scaler}. The {@link 
 * FLVars#BoxSizeScaleDenominator denominator}/
 * {@link FLVars#BoxSizeScaleNumerator numerator} are switched
 * because we  need scale to the ^-1; sizes are calculated using 
 * {@link #MakeScaledSeries the scaled series method}. 
* @return int [] of box sizes
* @param Scale float for scale
* @param Max float for maximum size
* @param Min float for minimum size */
int [] getScaledSeries(float Scale, float Max, float Min, boolean isgray)
{                        
     int [] scaledsizes=MakeScaledSeries (Max, Scale, Min, isgray);                         
     int index=scaledsizes.length;
     int [] BS=new int [index];
     for (int b=0; b < index; b++) BS[b]=scaledsizes[b];
     return BS;
}


/**Returns a series of box sizes calculated using the {@link 
* #MakeBlockBoxSizes relative series} making function.  Checks 
* the size of the array for legitimacy.  Starts with a new array 
* of the {@link FLVars#leastpossibleboxes minimum permitted}
* length of box sizes.  If the {@link FLVars#UserMinBoxSize user}
* minimum is &<; 2 for grayscale images, the minimum size 
* is set to 2.  The maximum starts at the 
 * {@link FLVars#GreaterOfHtAndWd greater of height or width}
 * of the roi or image, or {@link FLVars#limit user max}.  If the 
* max is less than the min, the series is made using min*2 and min,
* otherwise, it generally is made using max and min passed to the 
* {@link #MakeBlockBoxSizes box maker}.  For horizontal 
* or vertical {@link #changedSeries lines}, special considerations
* are made so that the max is the user's maximum if the user wants
* a maximum, or else the image dimension.  In most cases, however,
* the series is made by a call to the box maker using the 
* min and max as above.  If the length of the series is less than
* the {@link FLVars#leastpossibleboxes permitted minimum}, 
* the max and min are adjusted up and down until a series of 
* an acceptable length is returned or the limits are reached, in 
* which case the array is returned as is.
* @return int [] array of box sizes*/
int [] CheckRelative()
     {
         int [] bs = new int [V.leastpossibleboxes];
         int max = V.limit;
         int min=V.getUserMinBoxSize();
         if(V.isGray()&&min<2)min=2;
         int end = (int)V.LesserOfHtAndWd;
         if(V.UseGreater) end=(int)V.GreaterOfHtAndWd;
         if(end<=max)end=max;
//make sure the maximum is more than the minimum
//decrease the minimum if possible, or else increase 
//the maximum if required
         if (max<=min)return MakeBlockBoxSizes(min*2, min );         
          if (V.getUserMaxBoxSize ()>0)
          {
             int M=V.getUserMaxBoxSize ();
             if (M>min)max=M;
             else max=min*2;
             return MakeBlockBoxSizes(max, min );
         }
         if(changedSeries)
         {
             max = (V.isHorizontalLine)?V.Margins[1]-V.Margins[0] :
                 V.Margins[3]-V.Margins[2];
             int m = V.getUserMaxBoxSize(), n =0;
             if (m>0)  max=m;
             return MakeBlockBoxSizes(max, min);
         }
//if the image is very skinny, it is best analyzed
// very small boxes, or else the sampling is
//distorted 
         boolean DoPlus=true, DoMinus=true;                     
         int up = max;
         int down =max;                           
         for (int i = 0; DoPlus||DoMinus; i++)
             {
//try to find a suitable array of boxsizes
//within a reasonable distance of 
//the specified range
                        bs = MakeBlockBoxSizes (up, min);
                        if( bs.length>=V.leastpossibleboxes)return bs;
                        bs=MakeBlockBoxSizes(up, min+1);
                         if( bs.length>=V.leastpossibleboxes)return bs;
                        bs=MakeBlockBoxSizes(down, min);
                         if( bs.length>=V.leastpossibleboxes)return bs;
                        bs=MakeBlockBoxSizes(down, min+1);
                         if( bs.length>=V.leastpossibleboxes)return bs;
                         else
                         {
                             if (down<=min+1)DoMinus=false;                                     
                             if(up>end)DoPlus=false;
                             if(DoPlus)up++;
                             if(DoMinus)down--;                                     
                         }                                 
                 }
         return bs;
 }

/**Returns a series of integer boxsizes starting with a {@link 
* FLVars#base base} and increasing by that base raised to an 
* {@link FLVars#exponent exponent} that is incremented once for
* each <i>numboxes</i>. The box sizes are converted to integers 
* so the series is a pseudo power series, (i.e., b<sup>e</sup>,
* b<sup>2e</sup>, b<sup>3e</sup>....).  If the series is less than
*{@link FLVars#leastpossibleboxes the minimum} number of 
* boxes long, the length is extended past the {@link FLVars#limit 
* maximum}.
* @param numboxes int for the length of the series
* @return an integer array of box sizes starting at a {@link 
* FLVars#base stored base} and increasing in integers according 
* to {@link FLVars#exponent an exponent}*/
public static int [] getPowerSeries(int numboxes,
      float E, float base, float max, boolean isGray)
{
          int index = 0;
          float exp=E;
          int [] BS= new int [numboxes];
          if(isGray&&base<2)base=2;
          BS[0]=(int)base;
          for (index=1; index<numboxes&&index>0; index++)
          {
                 int f=(int)(Math.round (Math.pow (base, exp)));
                 if (f>max)break;
                 if(f>BS[index-1]) BS[index]=f;
                 else index--;
                 exp=exp+E;
           }
          if(index<2)
          {
              IJ.showMessage ("Could not calculate power series");
              BS=new int []{(int)base, (int)base+1};
              index=2;
          }
      int [] b = new int [index];
      for (int i = 0; i < index; i++)b[i]=BS[i];
      return b;
}

/**Returns a series of ints scaled from the largest to smallest 
* according to the passed value of scale (e.g., 50, 25, 13, 6, 3, 2).  
* @param LargestSize float for the largest size to use
* @param SmallestBox for the smallest box size
* @param Scale for the scale between elements
* @return int [] scaled series of ints*/
public static int [] MakeScaledSeries (float LargestSize,
     float Scale, float SmallestBox, boolean isgray)
{  
         if(isgray&&SmallestBox<2)SmallestBox=2;
         int PossibleBoxes =
                 1 + (int)(Math.log (LargestSize)/Math.log (Scale));                     
         int N=0;
         float size=LargestSize;
         for (int i  = 0; i<PossibleBoxes; i++)
         {
             float newsize=(size/Scale);
             int check=Math.round (newsize);
             if (check<Math.round (size)
             &&check>=(int)SmallestBox)
             {size=newsize; N++;}
         }
         int count=1;
         float LastSize=LargestSize;
         int [] IntSizes = new int [N+1];
         IntSizes[0]=(int)LargestSize;
         for (int i = 1; i < PossibleBoxes; i++)
         {
             float newsize=(LastSize/Scale);
             int check=Math.round (newsize);
             if (check<Math.round (LastSize)
             &&check>=(int)SmallestBox)
             {IntSizes[count]=check;LastSize=newsize;count++;}
         }
         int [] SmallestToLargest = new int [count];
         int c =0;
         for (int i = count-1; i >=0; i--)
         {
             SmallestToLargest[c]=IntSizes[i];
             c++;
         }               
         return SmallestToLargest;
     }

/**Runs the class to show the default sizes.
* @param args String [] of arguments*/
public static void main (String [] args)
{
//                     float scale=2f;
//                     int [] S=MakeScaledSeries (25f, scale,  1);
//                     for (int i = 0; i < S.length; i++)
//                     {
//                            System.out.println (S[i]);
//                     }
//                     float exp = 2.3f, base = 2;
//                     int limit = 12, leastpossibleboxes=5, num=5, min=6;
//                    int [] s=getPowerSeries (num,
//                            exp, base, limit,  leastpossibleboxes);
//                     System.out.println ("power");    
//                     for (int i = 0; i < s.length; i++)System.out.println (s[i]);                     
//                     //test the functions to calculate box sizes
//                     getMaxBlockBlocks(xstart, ystart, bot, rt, size);
//                      int M[] =MaxBlockBlocks(0, 0, 30, 35, 4, 10);
//                             System.out.println(M[0] + " " +M[1]);


}


/**Finds rectangular margins of the foreground pixels of a passed 
* ImageProcessor (i.e., from a binary image) and sets 
* measurements for height, width, and span. The image must be 
* binary because this method uses the value set in
 * {@link FLVars#foreground} to search for pixels of that colour
 * only. Resets the value in {@link 
 * FLVars#GreaterOfHtAndWd dimension} 
 * variable. Sets the
* {@link FLAnalyzer.FLVars#CVRadii left} and 
 * {@link FLAnalyzer.FLVars#MeanRadii right} horizontal
 * {@link FLAnalyzer.FLVars#MaxRadius top} and
 * {@link FLAnalyzer.FLVars#MaxOverMinRadii bottom} diameters
 * (Costa & Cesar, Shape  Analysis and Classification)
* @param ip ImageProcessor from binary image to work on
* @return boolean always returns true when done*/
public int [] FindMargins (ImageProcessor ip)
{
    int [] histogram = new int[256];
    float totalwidth = ip.getWidth ();
    float totalheight = ip.getHeight ();
    int [] margins =new int [4];
    V.CVRadii = 0;
    V.MeanRadii = 0;
    V.MaxRadius = 0;
    V.MaxOverMinRadii = 0;
    //Find left edge
    int margleft = -1;
    int margbottom=-1;
    int margright=-1;
    int margtop=-1;
    int margHeight=0;
    int margWidth=0;

    changedSeries=false;
    do//scan the image at its full height, starting from
     //the top left until a pixel is found, then call this
     //the left margin
    {	
        margleft++;
        if (margleft>=totalwidth)
        {
            IJ.error ("No non-background pixels found.");
            V.setAbort (true);
            return null;
        }
        if(!V.isAbort ())
        {               
            ip.setRoi ((int)margleft, 0, 1, (int)totalheight);
            histogram = ip.getHistogram ();                                                        
        }
    } while (histogram[V.getForeground ()]==0);                     

    if (!V.isAbort ())
    {
        int w = (int)(totalwidth-margleft);
        if (w<1){IJ.showMessage ("too skinny");w=1;}
        //Find top edge
        margtop = -1;
        do
        {
            margtop++;
            ip.setRoi ((int)margleft, (int)margtop,  w, 1);
            histogram = ip.getHistogram ();
        } while (histogram[V.getForeground ()]==0);

        //Find right edge
        int h = (int)(totalheight-margtop);
        if (h<1) {IJ.showMessage ("too skinny");h=1;};
        margright=(int)(totalwidth+1);
        do
        {	
            margright--;
            ip.setRoi ((int)margright-1, (int)margtop, 1,  h);
            histogram = ip.getHistogram ();        
        } while (histogram[V.getForeground ()]==0);

        margWidth = (margright - margleft);
        if(margWidth<2)
        {
            margright=margleft+1;
            V.isVerticalLine=true;
            changedSeries =true;
        }    
        else V.isVerticalLine=false;
        margWidth = (margright - margleft);

        //Find bottom edge
        margbottom = (int)(totalheight+1);   
        do
        {
        margbottom--;
        ip.setRoi ((int)margleft, (int)margbottom-1, (int)margWidth, 1);
        histogram = ip.getHistogram ();
        }while (histogram[V.getForeground ()]==0);
        margHeight = (margbottom - margtop);
        if(margHeight<2)
        {
            V.isHorizontalLine=true;
            margbottom=margtop+1;                
            changedSeries =true;
        }
        else V.isHorizontalLine=false;
        margHeight= (margbottom - margtop);
    }
    RectEnclosingForeground=new Rectangle ((int)margleft, 
            (int)margtop, (int)margWidth, (int)margHeight);
    V.LesserOfHtAndWd=Math.min (RectEnclosingForeground.width,
             RectEnclosingForeground.height);
    V.GreaterOfHtAndWd=(float)Math.max(
            RectEnclosingForeground.width, 
            RectEnclosingForeground.height);
    if(V.GreaterOfHtAndWd==0)IJ.showMessage ("Error");//FIXME
    float centrex = margleft + (margWidth/2.0f);
    float centrey = margtop + (margHeight/2.0f);            
    V.GreaterHorizontalOrVerticalDimension=V.GreaterOfHtAndWd;
    //            //Find longest vertical radius
    //            int highestx = margleft-1;
    //            boolean didfirst=false;
    //            float rtopmarginx=margleft, lefttopmarginx=margleft;
    //            float rightxOnbottommargin=margleft,
    //                    leftmostbottompixelx=margleft;
    //            float bottommostleftpixely=margtop,
    //                    topmostleftpixely=margtop;
    //            float bottommostrightpixely=margtop,
    //                    topmostrightmarginpixely=margtop;
    //            do
    //            {	highestx++;
    //                ip.setRoi((int)highestx, (int)margtop, 1,1);
    //                histogram = ip.getHistogram();
    //                if (histogram[V.getForeground()] > 0) {
    //                    float d = (float)Math.sqrt(((highestx-centrex)
    //                    *(highestx-centrex))
    //            + ( (margbottom - centrey )*(margbottom - centrey ) )   );
    //                    V.vdiam = (float)Math.max(V.vdiam, d);
    //                    if(didfirst==false)lefttopmarginx=(float)highestx;
    //                    didfirst=true;
    //                    rtopmarginx=(float)highestx;
    //                    //IJ.write("found a g pixel");
    //                }
    //            }while (highestx <= margright);
    //
    //            V.vdiam  = 2.0f*V.vdiam;
    //            highestx = margleft -1;
    //            didfirst=false;
    //            do {
    //                highestx++;
    //                ip.setRoi((int)highestx, (int)margbottom-1, 1,1);
    //                histogram = ip.getHistogram();
    //                if (histogram[V.getForeground()] >0) {
    //                    float d = (float)
    //                    Math.sqrt(((highestx-centrex)*(highestx-centrex))
    //                    +((margbottom-centrey)*(margbottom-centrey)));
    //                    V.bdiam = (float)Math.max(V.bdiam, d);
    //               if(didfirst==false)leftmostbottompixelx=(float)highestx;
    //                    didfirst=true;
    //                    rightxOnbottommargin=(float)highestx;
    //
    //                }
    //            } while (highestx<=margright);
    //            V.bdiam = 2.0f*V.bdiam;
    //            didfirst=false;
    //            //Find longest horizontal radius
    //            int highesty = margtop-1;
    //            do
    //            {	highesty++;
    //                ip.setRoi((int)margleft, (int)highesty, 1,1);
    //                histogram = ip.getHistogram();
    //                if (histogram[V.getForeground()] > 0) {
    //                    float d = (float)
    //                    Math.sqrt(((margleft-centrex)*(margleft-centrex))
    //                    +((highesty-centrey)*(highesty-centrey)));
    //                    V.hdiam=(float)Math.max(V.hdiam,d);
    //                    //IJ.write("found a c pixel");
    //                    if(didfirst==false)topmostleftpixely=(float)highesty;
    //                    didfirst=true;
    //                    bottommostleftpixely=(float)highesty;
    //                }
    //            }while (highesty<=margbottom);
    //            V.hdiam  = 2.0f*V.hdiam;
    //            didfirst=false;
    //            highesty=margtop-1;
    //            do
    //            {	highesty++;
    //                ip.setRoi((int)margright-1, (int)highesty, 1,1);
    //                histogram=ip.getHistogram();
    //                if (histogram[V.getForeground()] >0) {
    //                    float d=(float)
    //                    Math.sqrt(((margright-centrex)*(margright-centrex))
    //                    +((highesty-centrey)*(highesty-centrey)));
    //                    V.tdiam = (float)Math.max(V.tdiam,d);
//           if(didfirst==false)topmostrightmarginpixely=(float)highesty;
    //                    didfirst=true;
    //                    bottommostrightpixely=(float)highesty;
    //                    //IJ.write("found a b pixel");
    //                    //find the greater of
    //                }
    //            } while (highesty <= margbottom);
    //            V.tdiam  = 2.0f*V.tdiam;
        margins[0]=margleft;
        margins[1]=margright;
        margins[2]=margtop;
        margins[3]=margbottom;
        return margins;
}

/**Finds margins for grayscale images.
* @param ip ImageProcessor to work on
* @param oldWidth int for width
* @param oldHeight int for height
* @return int [] of margins where margins[0]=margleft; margins[1]
 * =margright; margins[2]=margtop;  margins[3]=margbottom;*/
public int [] FindGrayMargins (ImageProcessor ip, int 
      oldWidth, int oldHeight)
 {
     float totalwidth = ip.getWidth ();
     float totalheight = ip.getHeight ();
     int [] margins =new int [4];
     V.CVRadii = 0;
     V.MeanRadii = 0;
     V.MaxRadius = 0;
     V.MaxOverMinRadii = 0;
     //Find left edge
     int margleft =(int)V.xOffset;
     int margbottom=(int)V.yOffset+oldHeight;
     int margright=(int)oldWidth+V.xOffset;
     int margtop=V.yOffset;
     int  margWidth = (margright - margleft);
     if(margWidth<2)
     {
         margright=margleft+1;
         V.isVerticalLine=true;             
         changedSeries =true;                              
     }
     else V.isVerticalLine=false;
     margWidth = (margright - margleft);
     //Find bottom edge
     int  margHeight = (margbottom - margtop);   
     if(margHeight<2)
     {
         V.isHorizontalLine=true;
         margbottom=margtop+1;
         changedSeries =true;
     }
     else V.isHorizontalLine=false;
     margHeight= (margbottom - margtop);
     RectEnclosingForeground = new Rectangle ((int)margleft,
             (int)margtop,(int)margWidth, (int)margHeight);
     V.LesserOfHtAndWd=Math.min(RectEnclosingForeground.width,
             RectEnclosingForeground.height);
     V.GreaterOfHtAndWd=(float)Math.max
             (RectEnclosingForeground.width, 
             RectEnclosingForeground.height);
     if(V.GreaterOfHtAndWd==0)IJ.showMessage ("Error");//FIXME
     float centrex = margleft + (margWidth/2.0f);
     float centrey = margtop + (margHeight/2.0f);
     V.GreaterHorizontalOrVerticalDimension=V.GreaterOfHtAndWd;    
     margins[0]=margleft;
     margins[1]=margright;
     margins[2]=margtop;
     margins[3]=margbottom;
     return margins;
 }

/**Fills an array with the count at each box size for the passed 
* parameters. Calls the {@link #moveGridalloverCountPixels pixel 
* counting method} on this area once for each grid calibre in the 
* {@link #BOXSIZES box size array}.   Returns false if one of the 
* counts from {@link #moveGridalloverCountPixels the counter} 
* returns a null value, otherwise puts each returned array into 
* {@link #Pix the pixel counter}, where the length of each array at 
* [g] [b] is the box count and the array itself is the mass/intensity, 
* and stores {@link #Empties the empties}.
* @param ipc ImageProcessor to work on
* @param x int for starting horizontal coordinate
* @param y int for starting vertical coordinate
* @param LOC int for which grid location this is
* @return boolean true when done*/
public boolean StartBoxCounts (                         
 ImageProcessor ipc,
 int x,
 int y,
 int LOC )
{ 
    for (int b=0; b<BOXSIZES[LOC].length; b++)
    {
        //Goes through for each box size, and counts pixels using 
        //that box size; ppb is made as an array containing one value 
        //for each box's number of pixels--that is, how many pixels were
        //in each box that had boxes at this particular size.
        //The count at this box size is therefore the length of ppb.   
         float [] ppb = moveGridalloverCountPixels  ( b, x, y,  LOC,  ipc);                                            
         if (ppb==null)return false;
         float boxes = ppb.length;                        
         Pix[LOC][b]=new float [(int)boxes];
         for (int i=0; i<boxes;i++)Pix[LOC][b][i]=ppb[i];  
         Empties[LOC][b]=empties;//global variable filled 
         //elsewhere                      
    }
    return true;
}//end StartBoxCounts



/**Calculates multifractal data for the passed array. Is called by 
* {@link #MultifractalDataProcessor multifractal data getter}. The
* essential algorithm is:<OL><li>initialize arrays</li><li>convert
* the passed masses into 
 * {@link FLUtilities.FLutil#ElementOverSumOfArray densities}
 * </li><li>at each &epsilon;
* {@link #SummedPAtEpsilon sum probabilities to the Q}: 
* &#8721: (P<sub>i</sub>)<sup>Q</sup></li><li>store the 
* values for &tau;,D<sub>q</sub>, &alpha;, and &#402;(&alpha;)
* </li><li>make the appropriate &epsilon; array</li><li>Find  the
* {@link #FinishQslopes log-log regression} slopes</li><li>print 
* columns of formatted data corresponding to headings:Q, 
* D(<sub>Q</sub>), &tau;, &mu; &tau;, &alpha;, and &#402;
* (&alpha;)<li>depending on {@link FLVars#GraphFA options},
* {@link #PlotFAndAlpha graph &#402;(&alpha;)}</li><li>or 
* {@link #plotXY plot} other relationships</li></OL>
* @param dataoption int for the type of analysis to do
* @param newBoxes array of box sizes to use
* @param masses array of masses to use
* @param LOC int for the current grid position
* @return String of multifractal data
* @see FLUtilities.FLutil#ElementOverSumOfArray
* @see #SummedPAtEpsilon
* @see #FinishQslopes
* @see #PlotFAndAlpha
* @see #plotXY*/
public String doMF (float [][] masses, int [] Boxes,
            int LOC, int dataoption, int thisslice)
{
//find the minimum or slope corrected
//arrays for multifractal analyses
//calculate the multifractal data after converting
//the passed arrays to slope removed                     
    int es = Boxes.length;
    String title = (LOC==-1)?title=V.TitleAndSlice:  Titles[LOC];
    float[] []slopeDqs= FLutil.new2dArray (es, V.Qs.length,0.0f);                     
    float[] []slopeMeanTau=FLutil.new2dArray (es,  V.Qs.length,0.0f);                     
    float[] []slopeAlphas=FLutil.new2dArray (es, V.Qs.length,0.0f);                     
    float[] []slopeFs=FLutil.new2dArray (es,  V.Qs.length,0.0f);                     
//now get the multifractal calculations done
//once for each epsilon in the new list
    for (int i = 0; i < es; i++)
    {
        float [] Densities = 
            FLutil.ElementOverSumOfArray(masses[i], masses[i].length);
//                         at each epsilon, sum the probabilities to the Q
//                         for calculating the multifractal spectrum
//                         if the slope removed cover is being sought, skip any
//                         epsilon that had the same count as the previous
        float[][] A= SummedPAtEpsilon (Densities, V.Qs, masses[i]);                         
        slopeDqs[i]=A[0];
        slopeMeanTau[i]=A[1];
        slopeAlphas[i]=A[2];slopeFs[i]=A[3];
      }
    float [] E = new float[es];
    for (int i=0; i < es; i++)
        E[i]=((float)Boxes[i])/(float)V.GreaterOfHtAndWd;
    float  [][] GD =FinishQslopes (slopeDqs, slopeMeanTau,
            slopeAlphas, slopeFs, LOC, E, es, V.Qs);                     
    String type=
        (dataoption==0) ? "standard" :
        (dataoption==1) ?  "slope corrected"  :
        (dataoption==2) ? "minimum cover" :
        "Slope Corrected Minimum Cover";

    //print columns of q and the corresponding Dq
    String B= title+newline;
    String qstring = "";
    for (int q = 0; q<V.Qs.length; q++)
                qstring =   qstring+ V.Qs[q] +tab+
                FLutil.fnum (GD[DQindex][q])                +tab+
                FLutil.fnum (GD[dDQindex][q])              +tab+
                FLutil.fnum (GD[TAUindex][q])              +tab+
                FLutil.fnum (GD[ALPHAQindex][q])      +tab+
                FLutil.fnum (GD[FALPHAQindex][q])    + newline;                     
    B=B+qstring;  
    if(V.GraphFA)PlotFAndAlpha (GD[ALPHAQindex], 
            GD[FALPHAQindex], "",  title, V.Qs);                     
    if(V.GraphQs) 
        plotXY (true, V.Qs, GD[DQindex],  "",   title, "Q", "D(q)");                     
    if(V.GraphT)
    {
        plotXY (false, V.Qs, GD[TAUindex], "", title, "Q", tau);      
        plotXY (false, V.Qs, GD[dDQindex], "", title, "Q", tau);      
    }
    //              write data and graphs if the user wants the 
    //       optimum grid chosen
    float thiscv=0f;
    if(V.WriteQslopeandCV||V.SelectBestCV)
    {
        FracStats F = new FracStats();
        thiscv=FLstatistics.CoefVar(GD[DQindex], V.Qs.length);
        float [] f=
                F.PlainLinearRegression(GD[DQindex], V.Qs, V.Qs.length);
 //record the slope and cv in the data string
        B=B+newline+TabbedStats(GD, V.Qs)+newline+" "+tab+
                "Coefficient of Variation"+ tab+cv+tab+rsq+ newline+
                " "+tab+FLutil.fnum(f[F.SLOPEindex])+tab+
                FLutil.fnum(thiscv)+tab+FLutil.fnum(f[F.RSQindex])
                +newline+newline;
        if(V.SelectBestCV)
        {
            if (thiscv > bestCV)
            {
                bestCV=thiscv;
                bestLOC=LOC;
                bestGD = GD;
            }
            if(LOC==V.GridPositions-1)B=B+ShowOptimizedMF(GD);
        }
    }
    FLGraphics.drawsGeneralizedDimensions ( V, GD,  title);
    return B; 
}

/**Returns an array of pixel counts after setting variables and 
* selecting a scanner. If  {@link FLVars#RandomMassSample} is 
* true, a block of the image is {@link #Scan scanned} for each 
* {@link FLVars#subsamples subsample}, from origins stored in 
* {@link #RMxy an array filled earlier}, using a height and width  
* from {@link FLVars#Subboxsize the sub size} option.  
* Otherwise, widthOfTotalScan=rt-xstart and heightOfTotalScan 
* = bot-ystart. If the {@link FLVars#SLAC sliding lacunarity} 
* option  is true, {@link #SLACScan the sliding scanner} is called 
* once for the given boxsize, xstart, ystart, and dimensions. If a 
* {@link FLVars#BLOCKTEXTUREANALYSIS block}, {@link 
* FLVars#isRelative relative series}, or {@link FLVars#isGray 
* grayscale} scan is chosen, {@link #ScanBlockOrRelativeOrGray the 
* appropriate scanner}is used for this box size, xstart, ystart, and 
* dimensions (xstart to rt and ystart to bot). Otherwise, the {@link 
* #Scan scanner} for a regular scan is called at this boxsize, xstart,
* ystart, and dimensions (xstart to rt, ystart to bot).  If {@link 
* FLVars#showGrids the grids are to be drawn}, updates {@link 
* #GRIDIMAGEipl the grid image} which was drawn on by the 
* called scanner.  <h3>Starting Position</h3><p>The first box is 
* laid at the passed x, y coordinates, except in some cases where 
* these are adjusted. <ol><LI>Multifractals:<ul><LI> The first 
* four grids of a multifractal  scan using more than 1 
 * {@link FLVars#GridPositions location} are adjusted. They 
* fall so that each scan has a box that falls fully on respectively 
* the four corners of  the rectangle enclosing the image's 
 * {@link FLVars#foreground} pixels.</li><LI>This means 
* each epsilon is shifted to fall on the corner for a scan series.</LI>
* <LI>The coordinates delivered to the final file are for the top right
* corner, of the last scan at a location, though, so do not indicate 
* where the other scans started, only where the largest box started.
* </LI><LI>The effect is a 90&deg; rotation at each scan, where x 
* and y are changed:<ul><LI>If 
 * {@link FLAnalyzer.FLVars#slipGrid slipGrid} is true, random x,y 
 * values for starting are set
* within this method. </LI>The {@link #PPBr array} for counitng
* pixels per box is remade on each call, for a size of imap.getWidth
* ()*imap.getHeight () or, if this is a 
 * {@link FLVars#RandomMassSample random mass}sample, to 
 * {@link FLVars#subsamples subsamples} times 
 * widthOfTotalScan*heightOfTotalScan. The variable for empties is
 * reset to 0 on each run, and {@link FLAnalyzer.FLVars#actualx} 
 * and {@link FLAnalyzer.FLVars#actualy}.
* @return int [] integer array of the number of pixels per box at 
* this size skipping zero entries if binary, and truncated so there are
* no null entries at the end.              
* @param boxnum integer for the number that this box size is in 
* the array of box sizes; used for storing data
* @param xstart integer for x coordinate to start scanning at
* @param ystart integer for y coordinate to start scanning at                     
* @param thislocation int for this grid location
* @param imap ij.process.ImageProcessor for processing
* @see FLVars#showGrids option to show grids */
public float [] moveGridalloverCountPixels (                      
     int boxnum,
     int xstart,
     int ystart,                         
     int thislocation,                       
     ImageProcessor imap  )                         
{                     
 //for main analysis, set the 
 //right edge of the total pixelated area
 int rt = V.Margins[1]; 
 int bot = V.Margins[3];   
 if(changedSeries)
 {
     xstart=V.Margins[0];
     ystart=V.Margins[2];
 }

 int []  H =null;
 if(V.showGrids) 
 {
    GRIDIMAGEimg= imap.createImage ();
    GRIDIMAGEipl =  new ImagePlus ("Grids", GRIDIMAGEimg);
    GRIDIMAGEipr =  GRIDIMAGEipl.getProcessor ();
 }//end of preparation to show grids in a graphic         

 String name = V.TitleAndSlice;
 if (V.TitleAndSlice.length ()>12) 
     name=V.TitleAndSlice.substring (0, 11);                     
  IJ.showStatus ("Scan "+(thislocation+1)+"/"+V.GridPositions +
         " for "+name+" "+ " " +progress++);  
 int largestBox = BOXSIZES[0][BOXSIZES[0].length-1];
 int thisBox=BOXSIZES[0][boxnum];
 float widthOfTotalScan=rt-xstart;
 float heightOfTotalScan = bot-ystart;
 float CountBoxes=0f, CountGray=0f;
 if( V.BottomRight&&!V.RandomMassSample &&thislocation>0)
 {
     int [] U=SetFourCorners (thislocation, thisBox);
     xstart = U[0];
     ystart = U[1];                         
 }
 if(V.slipGrid&&!V.SLAC)
 {
         Random R = new Random ();
         xstart=RectEnclosingForeground.x-R.nextInt (V.border);
         ystart=RectEnclosingForeground.y-R.nextInt (V.border);
 }
 int possiblepix= imap.getWidth ()*imap.getHeight (); 
 if(V.RandomMassSample){possiblepix =(int)(V.subsamples*
         widthOfTotalScan*heightOfTotalScan);}
 PPBr = new float[possiblepix];
 empties=0;
  if(V.RandomMassSample)
  {                         
     for (int  i = 0; i < V.subsamples; i ++)
     {
         int subx=RMxy[0][i]; 
         int suby=RMxy[1][i];
         int bottom = suby+V.Subboxsize;
         int right = subx+V.Subboxsize;                             
          if(i==0){CountBoxes=0;CountGray=0;} 
         float oldcount =CountBoxes;
         float [] starts =Scan( boxnum,  subx, suby,                         
         thislocation,  bottom,right,
         imap, thisBox, CountBoxes, CountGray); 
         float newCount = starts[2];
         CountBoxes = newCount; 
         CountGray=starts[3];
     }
  }
 else if(V.SLAC)
 {
     float [] starts =SLACScan (boxnum, xstart, ystart,                                
             thislocation,  bot, rt,    imap,   thisBox); 
     xstart=(int)starts[0]; ystart=(int)starts[1];                    
     CountBoxes=starts[2];  CountGray=starts[3];
 }
 else if(changedSeries   ||  V.BLOCKTEXTUREANALYSIS
     ||   V.isRelative() ||   V.isGray() )
{
         float [] starts = ScanBlockOrRelativeOrGray(
          boxnum,xstart,  ystart, thislocation,
          bot,  rt,   imap,    thisBox);
         xstart=(int)starts[0]; ystart=(int)starts[1];                    
         CountBoxes=starts[2];CountGray=starts[3];
 }                     
 else  
 {
    float [] starts = Scan( boxnum, xstart, ystart,thislocation, bot,rt,
        imap, thisBox, 0, 0); 
    xstart=(int)starts[0]; ystart=(int)starts[1];
    CountBoxes=starts[2];
 }                     

 if(V.showGrids)GRIDIMAGEipl.updateAndDraw();
 if (boxnum!=BOXSIZES[0].length-1) IJ.showStatus (
         "Finished grid "+boxnum+1 +" of "+BOXSIZES[0].length);
 else IJ.showStatus("Finished last box size");
 if ((!V.ScanSeparateRandomSubAreas)&&
         (!V.isUseParticleAnalyzer ())&&
         (!V.UseRectAnalyzer))
 {
     V.setActualx (xstart);
     /*set these for printing the box locations*/
     V.setActualy (ystart);
     /*for printing the box locations*/
 }                     
 if(V.showGrids) 
 {
        if(boxnum==0)
        {
            GridStack = new ImageStack (GRIDIMAGEipl.getWidth (),
              GRIDIMAGEipl.getHeight ());
        }
        GridStack.addSlice ((thislocation+1)+ "."+ 
                BOXSIZES[0][boxnum]
                +1+" pixels",  GRIDIMAGEipl.getProcessor ());
        if(boxnum==BOXSIZES[0].length-1)
        {
            StackWindow iww = new StackWindow (new ImagePlus
                    ("Location "+ (thislocation+1), GridStack));                          
            iww.showSlice (GridStack.getSize ());
            iww.setVisible (true);
        }
        IJ.showStatus("Done drawing grids.");
 }
 V.actualbot=bot;
 V.actualrt=rt;
 int holder = V.isGray()?(int)CountGray: (int)CountBoxes;
 float [] PPBFinal=new float [holder];
 for (int a = 0; a <holder; a++)
 {
     PPBFinal[a]=PPBr[a];
 }
 PPBr = null;
 return PPBFinal;                     
}

/**Returns randomly generated coordinates on the passed 
* ImageProcessor.  The coordinates are based on {@link 
* #RectEnclosingForeground a previously calculated rectangle}.  
* The number of pairs is {@link FLVars#subsamples the number of
* subsamples}, but this number may be reduced by a dialog that 
* comes up if the pixels in the sample areas are 
 * {@link FLVars#FACTOR too sparse}.
* @param ip ImageProcessor to work on
* @return int [][] of randomly generated (x,y) coordinates */                       
public int[] [] MakeRandomMassLocations (ImageProcessor ip)
{//make sure to only call it on the first slice
    int [][] xy = new int [2][V.subsamples];
    Random rn = new Random();
        int z = 0, subx=0, suby=0;boolean skip = false;
      for (int j = 0; j<V.subsamples; j++)             
      {
          skip = false;                  
          int checkiterations = 0;
          do
          {  
              {subx = RectEnclosingForeground.x-(int)
               (.5*(float)V.Subboxsize)+ 
                       rn.nextInt (RectEnclosingForeground.width+1);
               suby = RectEnclosingForeground.y-(int)
               (.5*(float)V.Subboxsize) + 
                       rn.nextInt (RectEnclosingForeground.height+1);
              }
              if(!V.isGray())
              {//                               Generate a random starting point,
              //                                  then set a square roi with
              //                                  top left at that (x,y)
              ip.setRoi (subx, suby, V.Subboxsize, V.Subboxsize);
              int [] histogram = ip.getHistogram ();
              //Count its foreground pixels.
            //As long as the number of pixels is too low
            //relative to the number of
            //background pixels, try again.
            //After 150 tries, ask the user to decide
            //if they want to try
            //again or use this roi anyway.
              z = histogram[V.getForeground ()];                      
              if ((z==0) ||   (((float)z/Math.pow (V.Subboxsize, 2))
                  <V.getFACTOR ()))    { z=0;}
              checkiterations++;
              Object[] options = {"Click YES to ignore limit."};
              if (checkiterations>5000)
              {
                  if (JOptionPane.showConfirmDialog (null,
                          options, "Pixel ratio is too low. Ignore limit?",
                          JOptionPane.YES_NO_OPTION)
                          ==JOptionPane.YES_OPTION)
                  z=2;
                  skip=true;
              }
              }else z=2;                    
          } while (z==0) ;
          //                        Now find out if this is the last
          //                        subarea to do, and inform if so.
          boolean islastsub=false;
          if (j==(V.subsamples-1))islastsub=true;
          //                           Do a full analysis on this subarea.
          if (skip==false) 
          {
              xy[0][j]=subx;
              xy[1][j]=suby;
          }else {V.subsamples--;}   
      }
    return xy;
}

/**Returns false or else true if the ratio of foreground pixels (z)
 * in the box of size boxsize is less than {@link FLVars#FACTOR 
 * the set value}.
 * @param z int for number of foreground pixels
 * @param boxsize int for box's side length*/
boolean CheckPixelRatio(float z, float boxsize)
{
    boolean toosparse=false;
    if ( (float)(z/Math.pow (boxsize, 2)) <V.getFACTOR ())
    toosparse=true;                  
    return toosparse;
}
/**Returns a set of origins coded by the passed thislocation to be
* equivalent to scanning from each of the four corners.  Assumes 
* the first location is already at the top left.
* @param thislocation int for grid number 1, 2, or 3
* @param Box int for box size index
* @return int [] of origins that makes the grids fall at each of the 
* four corners of the image rectangle*/               
public int [] SetFourCorners(int thislocation, int Box)
{//adjust multifractal scans that are not random mass
    int X=0, Y=0;                        
     {
         float w = V.Margins[1]-V.Margins[0];
         float h = V.Margins[3]-V.Margins[2];
         if(thislocation==1)
         {
             X=(int)(V.Margins[1]-(Math.ceil (w/(float)Box)*(float)Box));
             Y=(int)(V.Margins[3] -(Math.ceil (h/(float)Box)*(float)Box));
         }
         else   if(thislocation==2)
         {
             X   =(int)(V.Margins[0]);
             Y   =(int)(V.Margins[3]-
                     (Math.ceil (h/(float)Box)*(float)Box));
         }
         else   if(thislocation==3)
         {
             X  =(int)(V.Margins[1]-
                     (Math.ceil (w/(float)Box)*(float)Box));
             Y=(int)(V.Margins[2]);
         }
     }
 return new int []{X,Y};
}


/**Scans the image for maxXBlocks and maxYBlocks, in increments 
* of {@link FLVars#SLIDEX} and {FLVard#SLIDEY}, counting 
* pixels and Intensity or pixel values over an area of <i>size</i>. 
* Returns an array of values that are {xstart, ystart, countboxes, 
* countgray}. Counts pixels per box for binary scans, and calls 
* {@link #GrayScaleCountAtThisSpot gray scale counter} for 
* grayscale images. Draws {@link #GRIDIMAGEipr grids} on the 
* image if this {@link FLVars#showGrids option} is set.
* @see #getMaxBlockBlocks
* @see #getMaxRelativeBlocks
* @see #getMaxBlocksWithin
* @see #getMaxBlocksBeyond
* @return float [] = {xstart, ystart, countboxes, countgray}
* @param boxnum index for this box size in the box sizes array
* @param xstart int for starting coordinate
* @param ystart int for y starting coordinate 
* @param thislocation int for this grid location
* @param bot int for bottom margin
* @param rt int for right margin
* @param imap ImageProcessor to scan                   
* @param size int for box size to analyze*/
public float [] ScanBlockOrRelativeOrGray(
    int boxnum,
    int xstart,
    int ystart,                         
    int thislocation,
    int bot,
    int rt,
    ImageProcessor imap,                                                  
    int size)
{
    int [] M = V.BLOCKTEXTUREANALYSIS?
        getMaxBlockBlocks(xstart, ystart, bot, rt,  size, size,
            BOXSIZES[0][BOXSIZES[0].length-1]):
        changedSeries||V.isRelative ()?
            getMaxRelativeBlocks(xstart, ystart, bot, rt, size, 
            BOXSIZES[0][BOXSIZES[0].length-1]):
            V.CheckPixRatio?
                getMaxBlocksWithin(xstart, ystart, bot, rt, size):
                getMaxBlocksBeyond(xstart, ystart, bot, rt, size);
    int maxXBlocks=M[0];
    int maxYBlocks = M[1];
    int szx = size, szy=size;
    int xblocks=0, yblocks=0;
    boolean foundone=false;
    float numPix=0f, countboxes=0f, countgray=0f;
    int [] H = null; 
    for (int y =ystart; yblocks <maxYBlocks; y=y+size)
    {//go through every row                             
         //restart the columns and count a row
         yblocks++;
         szx=size;
         szy=size;
         xblocks=0;
         for (int x = xstart;  xblocks<maxXBlocks; x=x+size)
         {
             xblocks++;//count one column                                 
             {
                 //first mark the starting place
                 if(y==ystart&&x==xstart){V.StartX=x;V.StartY=y;}
                 else{V.EndX=x+szx;V.EndY=y+szy;} 
                 //set the roi to the box to test  
                 if(V.BLOCKTEXTUREANALYSIS)
                 {//make sure the last box is within the block
                     if(rt-x<szx)szx=rt-x;
                     if(bot-y<szy)szy=bot-y;
                 }
                 imap.setRoi (x, y, szx, szy);
                 if(!V.isGray())
                 {
                     imap.setRoi (x, y, szx, szy);
                     H=imap.getHistogram ();
                     numPix = H[V.getForeground ()];
                 }
                 else
                 {
    //make a histogram that excludes nongray values
                     H=new int[256];
                     for (int i = 0; i<256; i++)H[i]=0;                                       
                     foundone =false; 
                     boolean foundtwo=false;
                     for (int h=x; h<x+szx; h++)
                     {
                         for (int v=y; v<y+szy; v++)
                         {
                             int [] P=new int[3];
                             if((h<imap.getWidth ()&&v<imap.getHeight ())
                                &&(h>=0)&&v>=0)
                             {
                                 P=imap.getPixel (h, v, null);
                                 if(P[0]==P[1]&&P[1]==P[2])
                                 {
                                     H[P[0]]++;
                                     if(foundone)foundtwo=true;
                                     foundone=true;
                                 }
                             }
                         }
                         }
                     if(foundtwo)foundone=true;
                     else foundone=false;
                     if (foundone)
                         numPix=GrayScaleCountAtThisSpot (H, size); 
                 }
                 if ((!V.isGray ()&&numPix>0) ||(V.isGray()&&foundone))
                 {
                     if(V.showGrids)
                     {//draw the grid if option selected
                         Color C = Color.blue;
                         if(V.isGray())
                         {
                             float n =(V.grayscantype==V.DIFFERENTIALBC)?
                                 numPix-1: numPix/(size*size); 
                             float r=n/(float) (imap.getHistogramSize ()-1);
                             float b = .6f;
                             C=new Color (r, .2f, b);//.8f);
                         }
                         //                     //correct for edge effects
                     if(V.CheckPixRatio && CheckPixelRatio(numPix, size)) ;
                     else
                         //only draw a box if it had more than the pixel ratio
                     {
                            GRIDIMAGEipr.setColor (C);                                               
                            GRIDIMAGEipr.drawRect(x, y, szx, szy);
                     }
                     }
                     if(V.isGray())
                     {
                         countboxes=countboxes+numPix;
                         countgray++;
                         PPBr[(int)countgray-1]=numPix;
                     }
                     else
                     {
                         countboxes++;//count if there were pixels in the box
                           PPBr[(int)countboxes-1]=numPix;
                     }
                 }
                 else {empties++;}
                 }
             }
         }
        return new float []{xstart, ystart, countboxes, countgray};
}

/**Returns the number of blocks along the x and the number along 
* the y for a relative series. The maximum number of blocks 
* depends on the type of scan: if its a relative scan, the blocks all fall
* at the end of the last box of the biggest size, so find where that box
* falls and divide the number of boxes into it. 
* @param xstart int for starting x coordinate
* @param ystart int for starting y coordinate
* @param bot int for bottom
* @param rt int for right margin
* @param size int for block size
* @param biggestBox int for largest box in the series
* @return int [] of the number of blocks horizontally [0] and 
* vertically [1]*/   
public int [] getMaxRelativeBlocks(int xstart, int ystart, 
       int bot, int rt, int size, int biggestBox)
{
   int [] M = new int [2];                      
   int widthOfScan = rt-xstart;
   int heightOfScan = bot -ystart;
   int NumXBiggestBox =(int)Math.ceil((float)
   widthOfScan/(float)biggestBox);
   int NumYBiggestBox = (int)Math.ceil((float)
         heightOfScan/(float)biggestBox);
   M[0]=(int)Math.ceil((NumXBiggestBox*biggestBox)/size);
   M[1]=(int)Math.ceil((NumYBiggestBox*biggestBox)/size);
   if (V.isVerticalLine)M[0]=1;
   if (V.isHorizontalLine)M[1]=1;
   return M;
}

/**Returns the maximum number of blocks of <i>size</i> that will 
* fit within the largest square that fits in the given area and is made
* of multiples of the <i>biggestBox</i>.
* @param xstart int for x coordinate
* @param ystart int for y coordinate
* @param bot int bottom edge's y coordinate
* @param rt int right edges's x coordinate
* @param sizex int the size of the box/slide horizontally
* @param sizey int the size of the box/slide vertically
* @param biggestBox int size of the largest blocks
* @return int [] specifying the boxes [0] horizontally and 
* vertically [1]*/
public int [] getMaxBlockBlocks(int xstart, int ystart, 
               int bot, int rt, int sizex, int sizey, int biggestBox)
{
            //first define the block
            int BlockDimension = 
                    Math.min(bot-ystart, rt-xstart);
            if(changedSeries)BlockDimension=
                    Math.max(bot-ystart, rt-xstart);
           int [] M = new int [2];
          // the blocks all fall at the end of 
           //the last box of the biggest size, so find where that
           //box falls and divide the number of boxes into it
           //they are all supposed to be factors of that box,
           //so it should work out
     
           int NumberOfBiggestBoxes = (int)Math.floor
                   ((float)BlockDimension/(float)biggestBox);                      

           M[0]=(NumberOfBiggestBoxes*biggestBox)/sizex;
           if(M[0]<1)M[0]=1;
           M[1]=(NumberOfBiggestBoxes*biggestBox)/sizey;
           if(M[1]<1)M[1]=1;
           return M;
}

/**Returns the maximum number of slides of size <i>slidex</i> for 
 * a block of size <i>size</i> that will fit within the largest square 
 * that fits in the given area and is made of multiples of the 
 * <i>biggestBox</i>.  The basic calculation is N=1+((width-
 * boxsize)/slide) for the x and y directions.  The result is ceilinged to 
 * an integer.
 * @param xstart int for x coordinate
*  @param ystart int for y coordinate
*  @param bot int bottom edge's y coordinate
*  @param rt int right edges's x coordinate
*  @param size int for the size of the blocks
*  @param slidex int for the distance to slide each block horizontally
*  @param slidey int for the distance to slide  each block vertically
*  @param biggestBox int for the size of the largest blocks
*  @return int [] specifying boxes [0] horizontally and vertically [1]*/
public int [] getMaxSLacBlockBlocks(int xstart, int ystart, 
               int bot, int rt, int size, 
               int slidex, int slidey, int biggestBox)
{
    //first define the block
    int BlockDim =  Math.min(bot-ystart, rt-xstart);
   int [] M = new int [2];
  // the blocks all fall at the end of 
   //the last box of the biggest size, so find where that
   //box falls and divide the number of boxes into it
   //they are all supposed to be factors of that box,
   //so it should work out
 
   int NumBx=1+(int)Math.floor((BlockDim-biggestBox)/slidex);
   int NumBy=1+(int)Math.floor((BlockDim-biggestBox)/slidey);
   int width=(NumBx*slidex)+biggestBox;
   int height=(NumBy*slidex)+biggestBox;
   M[0]=1+(int)Math.ceil ((width-size)/slidex);
   M[1]=1+(int)Math.ceil ((height-size)/slidey);
   return M;
}
  
/**Returns the x and y margins to centre a square  
*of the greatest possible number of <i>biggestBox</i>
*within the rectangle (xstart, ystart), (bot, rt).
*@param xstart x coordinate of rectangle
*@param ystart ycoordinate of rectangle
*@param bot y of bottom of rectangle
*@param rt int for right side of rectangle
*@param biggest int for the boxes to make 
*into a square 
*@return int array [] where array[0]=the distance
*from x and array[1]=the distance from y to 
*start the block at*/ 
public int [] newLocation (int xstart, int ystart, 
       int bot, int rt, int biggest)
{
    //first define the block
    int BlockDim = Math.min(bot-ystart, rt-xstart);
    int [] M = new int [2];
    // the blocks all fall at the end of 
    //the last box of the biggest size, so find where that
    //box falls and divide the number of boxes into it
    //they are all supposed to be factors of that box,
    //so it should work out
 
    int NumBiggest=(int)Math.floor ((float)BlockDim/(float)biggest);                      
    M[0]=(int)(((float)(rt-xstart)/(float)NumBiggest)/2f);
    M[1]=(int)(((float)(bot-ystart)/(float)NumBiggest)/2f);
    return M;
}

/**Returns the number of blocks of <i>size</i> that will fit across
* the distance from (xstart, ystart) to (rt, bot) without exceeding
* that distance.
* @param xstart int starting x coordinate
* @param ystart int starting y coordinate
* @param bot int ending x coordinate
* @param rt int ending y coordinate
* @param size int for the blocks
* @return an integer [] where array[0]= number of horizontal 
* blocks and array[1] = number of vertical blocks*/
public int [] getMaxBlocksWithin(int xstart, int ystart, 
   int bot, int rt, int size)
{
    int [] M = new int [2];                       
    int widthOfScan = rt-xstart;
    int heightOfScan = bot -ystart;
    M[0]=(int)Math.floor((float)widthOfScan/(float)size);
    M[1]=(int)Math.floor((float)heightOfScan/(float)size);
    return M;
}

/**Returns the number of blocks of <i>size</i> that will fit across 
* the distance from (xstart, ystart) to (rt, bot) either fitting 
* exactly or going less than one full block over that distance.
* @param xstart starting x coordinate
* @param ystart starting y coordinate
* @param bot ending x coordinate
* @param rt ending y coordinate
* @param size for the blocks
* @return an integer [] where array[0]=number of horizontal 
* blocks and array[1] = number of vertical blocks */
public int [] getMaxBlocksBeyond(int xstart, int ystart, 
       int bot, int rt, int size)
{
   int [] M = new int [2];                       
   int widthOfScan = rt-xstart;
   int heightOfScan = bot -ystart;
   M[0]=(int)Math.ceil((float)widthOfScan/(float)size);
   M[1]=(int)Math.ceil((float)heightOfScan/(float)size);
   return M;
}
                     
/**Scans an image using overlapping and potentially exhaustive 
* sliding box scanning. Scans the image for {@link 
* #GetMaxBlocksforSLac maxXBlocks} and maxYBlocks, in 
* increments of {@link FLVars#SLIDEX slidex} and {@link 
* FLVars#SLIDEY slidey}, counting pixels and pixel values over an 
* area of side length <i>sz</i>. <em>Regular box counts scan by 
* moving a box one full box size over the image to the next position 
* that does overlap with the previous one, but a sliding box scan 
* slides the box by an amount that can be smaller than the box's 
* dimension, thus can overlap previous positions. </em> Counts 
* pixels for binary scans, or calculates for {@link 
* #GrayScaleCountAtThisSpot gray} scale image scans. Returns an
* array of values that are {xstart, ystart, countboxes, countgray}.
* @return float [] = {xstart, ystart, countboxes, countgray}.
* @param size int for the size of each block in this scan
* @param boxnum index for this box size in the box sizes array
* @param xstart int for starting coordinate
* @param ystart int for y starting coordinate            
* @param thislocation int for this grid location
* @param Bt int for bottom margin
* @param Rt int for right margin
 * @param imap ImageProcessor to scan*/
public float [] SLACScan(int boxnum,
     int xstart,
     int ystart,                                                                                              
     int thislocation,
     int Bt,
     int Rt,
     ImageProcessor imap,                          
     int size)
 {
   int [] marg =newLocation( xstart , ystart, Bt, Rt, 
           BOXSIZES[0][BOXSIZES[0].length-1]);                               
   int AdjustedX=V.BLOCKTEXTUREANALYSIS ?
       marg[0]: V.ScanEdges ? xstart-(size-1):xstart;
   int AdjustedY = V.BLOCKTEXTUREANALYSIS?
       marg[1]: V.ScanEdges ?  ystart-(size-1):ystart;
   int Adjustedrt = V.ScanEdges?Rt+(size-1):Rt;
   int Adjustedbot=V.ScanEdges?Bt+(size-1):Bt;
   int M [] = V.BLOCKTEXTUREANALYSIS?
       getMaxSLacBlockBlocks( AdjustedX, AdjustedY, Bt, Rt, size,
           V.SLIDEX, V.SLIDEY, BOXSIZES[0][BOXSIZES[0].length-1]):
       //for a block analysis, the start is inset and
       //the right and bottom are part of a square
       //defined by the biggest box size
       GetMaxBlocksforSLac(AdjustedX,  AdjustedY, Adjustedrt, 
           Adjustedbot,  size,  V.SLIDEX, V.SLIDEY);
   //for a scan edges analysis, the start is 
   //pulled up and left, and the end is pushed out and
   //down to get all possible positions
   //in the default analysis, the box starts at the
   //top left corner, goes across by slide y
   //until it reaches the edge,
   //then goes down by slide y and starts again
   float maxXBlocks = M[0];
   float maxYBlocks = M[1];
   int szx = size, szy=size;
   int xblocks=0, yblocks=0;
   boolean foundone=false;
   float numPix=0f, countboxes=0f, countgray=0f;
   int [] H = null; 
   for (int y =AdjustedY; yblocks <maxYBlocks; y=y+ V.SLIDEY)
   {
//go through every row                             
//restart the columns and count a row
       yblocks++;
       xblocks=0;
       for (int x = AdjustedX;  xblocks<maxXBlocks; x=x+V.SLIDEX)
       {
           xblocks++;//count one column                                 
           {
               szx=size;
               szy=size;                                  
//first mark the starting place
                if(y==AdjustedY&&x==AdjustedX)
                {V.StartX=x;V.StartY=y;}
                else{V.EndX=x+szx;V.EndY=y+szy;} 
//set the roi to the box to test  
                if(!V.ScanEdges)
                {//make sure the last box is within the block
                   if(szx>(Adjustedrt-x)) szx=Adjustedrt-x;
                   if(szy>(Adjustedbot-y)) szy=Adjustedbot-y;
                }
               imap.setRoi (x, y, szx, szy);
               if(!V.isGray())
               {
                   imap.setRoi (x, y, szx, szy);
                   H=imap.getHistogram ();
                   numPix = H[V.getForeground ()];
               }
               else
               {//make a histogram that excludes
                       //nongray values
                   H=new int[256];
                   for (int i = 0; i<256; i++)H[i]=0;
                   foundone =false; 
                   boolean foundtwo=false;
                   for (int h=x; h<x+szx; h++)
                   {
                       for (int v=y; v<y+szy; v++)
                       {
                           int [] P=new int[3];
                           if((h<imap.getWidth () &&v<imap.getHeight ())
                            &&(h>=0)&&v>=0)
                           {    
                               P=imap.getPixel (h, v, null);
                               if(P[0]==P[1]&&P[1]==P[2])
                               {
                                   H[P[0]]++;
                                   if(foundone)foundtwo=true;
                                   foundone=true;
                               }
                           }
                       }
                   }
                   if(foundtwo)foundone=true;
                   else foundone=false;
                   if(foundone)numPix=GrayScaleCountAtThisSpot (H,size); 
               }
               if ((!V.isGray ()&&numPix>0) || (V.isGray()&&foundone))
               {
                   if(V.showGrids)
                   {//draw the grid if option selected
                       Color C = Color.blue;
                       if(V.isGray())
                       {
                           float n=(V.grayscantype==V.DIFFERENTIALBC)?
                               numPix-1:numPix/(size*size);
                           C=new Color (n/(float)(imap.getHistogramSize ()-1),
                                   .2f, .8f);
                       }
                       GRIDIMAGEipr.setColor (C);
                       GRIDIMAGEipr.drawRect(x, y, szx, szy);
                   }
                   if(V.isGray())
                   {
                       countboxes=countboxes+numPix;
                       countgray++;
                       PPBr[(int)countgray-1]=numPix;
                   }
                   else 
                   {
                       countboxes++;//count if were pixels in the box
                       PPBr[(int)countboxes-1]=numPix;
                   }}
               else {empties++;}
           }
       }
   }
   return new float []{AdjustedX, AdjustedY, countboxes, countgray};
 }    

/**Returns the number of blocks horizontally and vertically.
* @return int [] array[0]=xblocks, array[1]=yblocks.
* @param startX int for starting x coordinate
* @param startY int for starting y coordinate
* @param endX int for right margin
* @param endY int for bottom margin
* @param boxsize int for box size
* @param slidex int for distance to slide each box in the x direction
* @param slidey int for distance to slide each box in the y direction*/               
public  int [] GetMaxBlocksforSLac (int startX, int startY, int endX,
        int endY, int boxsize, int slidex, int slidey)
{
   int [] XY = new int [2];
   //from the beginning and ending values, calculate the
   //number of boxes to go
   int width =  endX-startX;
   int height = endY-startY;
   XY[0]=1+ (int)Math.ceil((float)(width-boxsize)/(float)slidex);
   XY[1]=1+ (int)Math.ceil((float)(height-boxsize)/(float)slidey); 
   if(V.isVerticalLine)XY[0]=1;
   if(V.isHorizontalLine)XY[1]=1;
   return XY;
}

/**Does a regular nonoverlapping box count scan by moving a box 
* over an image in increments equal to the box's side size, so that 
* each placement does not overlap the previous ones in any 
* particular scan over the image at a box size.  (This is in contrast 
* to a {@link #SLACScan sliding box scan}.) Scans the area from 
* (xstart, ystart) to (rt, bot) in blocks of dimension <i>sz</i> and 
* counts pixels of {@link FLVars#foreground} that fell in the boxes 
* as well as the number of boxes that had pixels.  Fills the {@link 
* #PPBr array of pixel counts} at each successive index with the 
* number of pixels at each box that was greater than 0, and returns 
* the total number of boxes that had any pixels of the foreground  
* colour. Also increments {@link #empties} if there were no pixels 
* in a box. If {@link FLVars#showGrids grids are being shown}
* draws all boxes that had pixels in them on {@link 
* #GRIDIMAGEipr the grid image}.
* @return float array: [0]=xstart, [1]=ystart, [2]=count of boxes 
* that had pixels, [3] count of boxes in a grayscale scan
* @param countgray float for counting boxes in grayscale scans
* @param boxnum index of the current box size in the {@link 
* #BOXSIZES array of sizes}
* @param xstart int for starting x coordinate
* @param ystart int for starting y coordinate
* @param thislocation int for this grid
* @param bot int for bottom margin of scan area
* @param rt int for right margin of scan area
* @param imap ImageProcessor to scan
* @param sz int for sz of blocks
* @param count int to keep track of boxes counted */
public float [] Scan(
int boxnum,
int xstart,
int ystart,                         
int thislocation,
int bot,
int rt,
ImageProcessor imap,  
int sz,
float count, float countgray)
{      
    boolean foundone=false;
    int []H = null;                            
    float numPix=0;
    for (int y =ystart; y <=bot; y=y+sz)
    {                         
        //count at all the boxes
        for ( int x = xstart;x<=rt; x=x+sz)
        {
            int tempw=sz, temph=sz;
            if ((x+sz)>rt)tempw=(rt-x);
            if((y+sz)>bot)temph=(bot-y);
            {
                if(y==ystart&&x==xstart){V.StartX=x;V.StartY=y;}
                else {V.EndX=x+tempw;V.EndY=y+temph;}    
                if(!V.isGray())
                {
                    imap.setRoi (x, y, tempw, temph);
                    H = imap.getHistogram ();
                    numPix = H[V.getForeground ()];
                }
                else//if it is a gray scale image
                {//make a histogram that excludes nongray values
                   H=new int[256];
                   for (int i = 0; i<256; i++)H[i]=0;                                       
                   foundone =false; 
                   boolean foundtwo=false;
                       for (int h=x; h<x+tempw; h++)
                       {
                          for (int v=y; v<y+temph; v++)
                           {
                               int [] P=new int[3];
                               if((h<imap.getWidth ()
                               &&v<imap.getHeight ())
                               &&(h>=0)&&v>=0)
                               {    
                                  P=imap.getPixel (h, v, null);                                          
                                  if(P[0]==P[1]&&P[1]==P[2])
                                  {
                                           H[P[0]]++;
                                           if(foundone)foundtwo=true;
                                           foundone=true;
                                  }
                               }
                            }
                          }
                   if(foundtwo)foundone=true;
                   else foundone=false;
                   if (foundone)numPix=
                           GrayScaleCountAtThisSpot (H, sz); 
                 }  
                if ((!V.isGray()&&numPix>0)||(V.isGray()&&foundone))
                {
                    if(V.showGrids)
                    {//draw the grid if option selected                   
                        Color C = Color.blue;
                        if(V.isGray())
                        {
                            float n=
                                    (V.grayscantype==V.DIFFERENTIALBC)?
                            numPix-1:numPix/(sz*sz);
                            C=new Color (n/(float)                                         
                            (imap.getHistogramSize ()-1), .2f, .8f);
                        }
                        if(V.CheckPixRatio && CheckPixelRatio(numPix, sz)) ;
                     else
                         //only draw a box if it had more than the pixel ratio
                     {
                        GRIDIMAGEipr.setColor (C);
                        GRIDIMAGEipr.drawRect(x, y,tempw,temph);
                        }
                    }
                    if(V.isGray())
                    {
                        count=count+numPix;
                        countgray++;
                        PPBr[(int)countgray-1]=numPix;
                    }
                    else 
                    {
                        count++;//count if there were pixels
                        PPBr[(int)count-1]=numPix;
                    }
                } else {empties++;} 
            }
        }
    }
    return new float []{xstart, ystart,  count, countgray};
}                             


/**For calculating the fractal dimension of gray scale images. 
* Returns values based on the maximum and minimum (d=max-
* min) indices of the passed array.  For instance, if the passed array 
* is  {0, 2, 0, 4}, d=3-1=2.  Calls {@link #MinPositiveIndex 
* minimum} and {@link #MaxIndex maximum} index methods.
* If the value in {@link FLVars#grayscantype} is 
* {@link FLVars#DIFFERENTIALBC}, then it is assumed that a
* differential box count scan is being done and the value returned is
* d+1. Otherwise, it is assumed a 2d variation box count is being 
* done, and d*r<sup>2</sup> is returned.
* @return float for the difference between max and min, as either 
* d+1 or dr<sup>2</sup>
* @param r int for box size, used in return value
* @param A int [] of grayscale values arranged in a 256 histogram*/
public float GrayScaleCountAtThisSpot(int [] A, int r)
{
    float min = MinPositiveIndex(A);
    float max = MaxIndex(A);
    float d=(V.grayscantype==V.DIFFERENTIALBC)?
    max-min+1:r*r*(max-min);                  
    if (d<0)IJ.showMessage ("Weird image");
    return d;
}                 

    
}/*end plugin*/ 
