// Notice
//
// The contents of this file are subject to the
//  same license as the source code for ImageJ.  
//FracLac is in the public domain.
// If you use it to publish research, please cite the source.
// The initial developer of the original
// code is Audrey Karperien, Charles Sturt University.
package FLAnalyzer;

import ij.ImagePlus;
import ij.gui.PolygonRoi;
import ij.process.ImageProcessor;
import ij.process.ImageStatistics;
import java.awt.Color;

/**Holds the variables for running FracLac. 
* <p><b>IDE</b>: NetBeans IDE 4.1</p>
* @version February 2006
* @version 2.4e, April 2006
* @author Audrey Karperien
* (akarpe01@postoffice.csu.edu.au)
* @since jdk 1.5.0_04
*/
public class FLVars
{
/**Array of ImageProcessors for drawing the hull and circle and 
*colour-coded subscans.*/
public ImageProcessor [] [] HCImageProcessors;
public ImageProcessor [] IpArray;      
//===========================//
/**True scans random rectangular areas that are either on an 
* entire image or within the roi set on the image.
* @see FLsetup#GetSubScanInputs setup to set this option
* @see GUI.FL_#DoARectangularRoi rectangular roi scanner
* @see GUI.FL_#DoIrregularRoi irregular roi scanner
* @see GUI.FL_#DoEntireImage image scanner*/
public boolean ScanSeparateRandomSubAreas=false;
public int ChooseBackground=0;
public Color filler = new Color(23, 142, 29);
public static int BINARY = 0,  GRAY = 2,  THRESHHOLD = 1,
    SLIDINGLAC=32;
public boolean DisableRoiManager=false;

public static int DIFFERENTIALBC=6, VARIATION2DBC=8;
public int grayscantype=VARIATION2DBC;
/**True shows graphics showing the positions of grids applied
*in analyzing a pattern.*/
public boolean showGrids = false;
public boolean UseRoiManager=false;
/**True analyzes a binary image as is.*/
public int ImageType = BINARY;        
public static int CHOOSEAUTOMATICALLY = 0, USEBLACK=1,
    USEWHITE=2, BLACK=0, WHITE=255;
/**True makes max and min limits to the same max and min
 * for all log log plots for sliding lacunarity scans.*/
public boolean SameScale = true;    
/**Returns a formatted string array "Lacunarity=L(J)=
* (Sum[iLambda])/GRIDS" where Lambda = &Lambda; and 
* "sigma/mu for L(i)" (i.e., &sigma;/&mu;).
*@param J String to use in L(J)
*@param i String to use in second part */
String [] L (String J, String i)
{
    return new String [] 
 {
     "Lacunarity =L("+J+")=("+FLMain.sum+      
            "["+i+FLMain.Lambda+"])/GRIDS",
      cv+" for L("+i+")"
 };    
}

String  mean (String J)
    {return new String (FLMain.sum+ "("+J+")/GRIDS");}

String [] boxcountheadings=
    {
    "FracLac v2.4e: file/slice/roi",
    "Foreground pixels",
    "Total pixels",

    "Mean D =" +mean("D")+
            " (D=slope [ln(Boxes with Foreground Pixels)/ln("
            +FLMain.epsilon+")]",
    "Mean "+FLMain.Lambda+"D", 

    FLMain.rsq, 
    "Standard Error (SE)", 
    "CV =standard deviation/mean"+cv+" for D",                    
    "Mean Smoothed D",
    FLMain.rsq,
    "SE",
    "Minimum Cover D",
    FLMain.rsq,
    "SE",
    "Minimum Cover Smoothed D",
    FLMain.rsq,
    "SE",
    "Mean Y-intercept for D",
    FLMain.Lambda +
            "Mean Y-intercept Prefactor for D (Count vs Mass)",
    cv+" for Y-Intercepts for D",
    L("F (foreground mass)", "F")[0],
    L("F", "F")[1],
    L("E (foreground and empty space)", "E")[0],
    L("E", "E")[1],
    L("binned probability distribution for foreground", "FBPD")[0],
    L("FBPD", "FBPD")[1],    
    L("EBPD (foreground and empty boxes)",  "EBPD")[0],
    L("EBPD", "EBPD")[1],           
    "Variation in Count="+mean("Count"+cv),
    mean("Foreground Count CV/Image Count CV"),
    "Box Sizes for D",
    "Box Sizes for Smoothed D ",
    "GRIDS = Number of origins",
    "Mean Max box size for D",
    "Mean Min box size for D",
    };       
    
String [] HullAndCircleDataHeadings ={       
                       
        "Total Foreground Pixels",
        "Total Pixels",
        "Density=Foreground Pixels/Hull Area",       
        "Span Ratio (major/minor axis)",
        "Hull's Centre of Mass",
        "Maximum Span Across Hull",
        "Area",
        "Perimeter",
        "Circularity",
        "Width of Bounding Rectangle",
        "Height of Bounding Rectangle",
        "Maximum Radius from Hull's Centre of Mass",
        "Max/Min Radii",
        "CV for all Radii",
        "Mean Radius",
        "Circle's Centre",
        "Diameter of Bounding Circle",
        "Maximum Radius from Circle's Centre",
        "Max/Min Radii from Circle's Centre",
        "CV for all Radii from Circle's Centre",
        "Mean Radius from Circle's Centre",
        "Method Used to Calculate Circle" };
        
public String GDataHeadings="";
public static String cvsq =
        "("+FLMain.sigma+"/"+FLMain.mu+")"+FLMain.sq;
public static String cv = "("+FLMain.sigma+"/"+FLMain.mu+")";

String [] gboxcountheadings=
    { 
        "Grayscale Image Data ",
        "Grayscale Pixels",
        "Mean FRACTAL DIMENSION (D)=slope ln[I]/ln "
        +FLMain.epsilon,
        "Mean ("+FLMain.mu+") "+FLMain.rsq,
        FLMain.mu+" Std Error",
        "Coefficient of Variation "+cv+" for Ds",
        FLMain.mu+" Y-intercept for Regression Line",                             

        "Mean "+FLMain.Lambda+
        " across grids (1+"+cvsq+" for [I])",       
        "Std Dev "+FLMain.sigma+
        " for Mean "+FLMain.Lambda+" across grids",

        "Mean "+" Binned Probability Distribution "
        +FLMain.Lambda+" across grids",
        FLMain.sigma+
        " for Mean BPD "+FLMain.Lambda+" across grids",

        "Formula for Intensity",
        "Formula for Df",
        "Mean Prefactor Lacunarity",
        "CV for Y-Intercepts",                
        "Number of origins",
        "Mean Box Sizes",
        "Mean Maximum box size",
        "Mean Minimum box size"
    };

public String GBoxCountHeadings=
    MakeHeadings (gboxcountheadings);

public String BoxCountHeadings=null;
        
String [] lacheadings = {
    "FracLac v2.4e: file/slice/roi",
    "Box Size",             
    FLMain.epsilon +"=Box Size/Larger Image Dimension",

    FLMain.mu+" foreground pixels per box",
    FLMain.sigma,
    "Boxes containing foreground pixels",                
    "F"+FLMain.lambda+"=1+"+cvsq,

    FLMain.mu+" image pixels per box",
    FLMain.sigma,               
    "Total boxes",
    "E"+FLMain.lambda+"=1+"+cvsq,

    FLMain.mu 
            +"foreground pixels in binned " +
            "probability distribution (BPD)",
    FLMain.sigma,
    "FBPD"+FLMain.lambda+"=1+"+cvsq,

    FLMain.mu +"image pixels per box for BPD", 
    FLMain.sigma,                
    "EBPD"+FLMain.lambda+"=1+"+cvsq
}; 

/**String of headings for sliding lacunarity scans of grayscale 
* images: Image Information, Box Size,&epsilon;, Number of Boxes,
* &#8721; I, &mu; I, &sigma; for I, &lambda; = 1+(&sigma;/&mu;)
* <sup>2</sup>, &mu; for Binned Probability Distribution, 
* &sigma; I for BPD, &lambda = 1+(&sigma;/&mu;)<sup>2</sup>
* for BPD*/
String [] grayslacheadings = 
{
            "Image Information",
            "Box Size",
            (FLMain.epsilon),
            "Number of Boxes",
            FLMain.sum+"I",
            FLMain.mu+" mean I",
            FLMain.sigma +"for I",
            FLMain.lambda+" =1+"+cvsq,
            FLMain.mu+"I for Binned Probability Distribution",
            FLMain.sigma +"I for BPD",          
            FLMain.lambda+"=1+"+cvsq+"for BPD"
}; 
public String SLACHeadings= MakeHeadings (lacheadings);
public String GRAYSLACHeadings
        =MakeHeadings (grayslacheadings);
public String DataFileHeadings="";
public String MFHeadings="";        
//      PARAMETERS FOR Q VALUES FOR 
//CALCUATING GENERALIZED DIMENSION
//      AND MULTIFRACTAL SPECTRUM.
/**String for data written to ouput file.*/
public String BCdata="", GBCdata="", SLACdata="",
GSLACdata="",   MFdata="", FrequencyData, 
BinnedFrequencyData;
float qmin = -10.00f;       
//minimum and maximum exponents in range
float qmax = 10.00f;        //used for calculating
float qinc = 0.25f;          //the generalized dimension
int dataoption = 0;
String DS [] ={"Standard","Slope-Corrected","Minimum Cover", 
        "Slope-Corrected/Minimum Cover"};
public static int STANDARD=0, SMOOTHED=1,MINCOVER=2, 
        SMOOTHEDMINCOVER=3;
/**Array of values to calculate
* the generalized dimension over.*/
public float [] Qs;          
public boolean DRAWHull=true;
public boolean DRAWCircle=true;
public boolean IsASingleRectangularRoi=false;
public boolean IsIrregularRoi=false;                        
public boolean notsetup=true;
public boolean isVerticalLine=false, isHorizontalLine=false;
public float LesserOfHtAndWd, GreaterOfHtAndWd;          
//      UTILITY VARIABLES
public ImagePlus DrawingForRectangularRoi;
public int NonRectX, NonRectY;
public String TitleAndSlice=" ";
public int [][] ParticleRectangleXY;
public PolygonRoi [] PAHulls;
public float [][] PACircs;
/**Value of pixels that will be assessed.  Is determined in 
* {@link FLAnalyzer.FLMain#SetImage}.*/
public int foreground=BLACK;
public int background=WHITE;
/**Size of border to add around images.*/
public int border = 0;
public float ScaledMaxBoxSize=0;
public boolean UseGreater=false;

/**Means {@link FLMain#BOXSIZES the box sizes} are {@link 
* FLMain#MakeNewBoxSizes calculated} as a linear series.
* @see #Series*/
final public static int DEFAULTBOXSIZES=0;
/**Means {@link FLMain#BOXSIZES the box sizes} are {@link 
* FLMain#MakeNewBoxSizes calculated} as a power series.
* @see #Series*/
final public static int POWERSERIES=1;
/**Means {@link FLMain#BOXSIZES the box sizes} are {@link 
* FLMain#MakeNewBoxSizes calculated} as a scaled series 
* according to the {@link FLMain#MakeScaledSeries box maker}
* @see #BoxSizeScaleDenominator
* @see #BoxSizeScaleNumerator
* @see #Series*/
final public static int SCALEDSERIES=2;
/**Means {@link FLMain#BOXSIZES the box sizes}
*are {@link FLMain#MakeNewBoxSizes calculated}
*as a relative series defined by the largest, 
* so that a whole number of
* each box fits into the largest box                        
@see #Series*/
final public static int RELATIVESERIES=3;
/**User option that determines how the series of box sizes is {@link 
* FLMain#MakeNewBoxSizes calculated} for each 
* {@link FLMain#StartBoxCounts scan}.
* @see #DEFAULTBOXSIZES
* @see  #POWERSERIES
* @see  #SCALEDSERIES
* @see  #RELATIVESERIES*/
public int Series=DEFAULTBOXSIZES;
int MaxFrequencies = 40;
int  TotalForegroundPixels ,
TotalAllPixels;
int PixelRatio = 1;
/**For ensuring a sufficient ratio of pixels to background for random
* subsamples.  Default = 0.001*/
public float FACTOR=0.001f;
public boolean dontclose=false;
public String methodused=null;
int xOffset, yOffset;
int StartX, EndX, StartY, EndY;

//////////////////////////////////////////////////////////////////////
//      VARIABLES FOR THE 
///PARTICLE ANALYSIS SCAN
public  int particlenumber=0;
public  int minparticle=30, maxparticle=999999;
///////////////////////////////////////////////////////////////////////
//      VARIABLES DEFINING THE BOX SIZES USED
/**Unaccessible to user; for the lowest permitted number of boxes 
* in a series.
* @see FLAnalyzer.FLMain#MakeNewBoxSizes sizing method*/
public int leastpossibleboxes=4;
public float BoxSizeScaleNumerator=1f;
public float BoxSizeScaleDenominator=3f;
/**Unaccessible to user; is the most box sizes that can be used 
* in a scan.*/
public int absolutemaximumnumberofboxes=500;

/**Integer that limits the number of box sizes to use.  Is set in the
 * {@link FLAnalyzer.FLMain#MakeNewBoxSizes box size limit 
 * setting method}.*/
public int limit = 4;

/**Holds values for colours for each square for doing sub areas.*/
public float [] colourarray;
/**Specifies the transparency of the graphic for subareas in {@link 
 * FLGraphics}. Default is 190; max is 255.*/
public int LocalAlpha = 190;
public boolean ScanEdges=false;
public boolean CheckPixRatio=false;
///////////////////////////////////////////////////////////////////////
//      VARIABLES FOR GATHERING BOX COUNT 
//       RESULTS.  These hold
//      the statistics per box size.                         
int [][] Globalxy=null;                         

/**Coordinates marking where a grid started.*/
public int actualx = 0,  actualy = 0, actualbot = 0, actualrt=0;

/**For the type of scan to do.  This values is used along with {@link 
 * #RoiColouringMethod},and is based on the values in the string 
 * array stored in FLAnalyzer.FLsetup:
* <pre>
* public static String [] SubScanTypes=
*
* "Analyzer Scan-contours",
* "Analyzer Scan-filled",
* "Analyzer Scan-display dimension",
*
* "Rectangular Scan-display dimension",
* "Rectangular Scan-blocks",
* "Rectangular Scan-pixels",
*
* "Random Block Scan-filled",
* "Random Block Scan-colour pixels",
//* "Block Scan-no colour",
* "Random Block Scan-no colour",
//* "Block Scan-colour pixels",
//* "Block Scan-fill",};
 * </pre>*/
public int SubScanType =5;

///////////////////////////////////////////////////////////////////////
//      IMAGE STUFF
/**Is reused for each image processed.
 * @see FLAnalyzer.CircStats#measureHullandCircle*/
public ImagePlus originaldrawing;

/////////////////////////////////////////////////////////////
/////  VARIABLES SET BY getBCInputs METHOD   /////
/////////////////////////////////////////////////////////////

/**User set variable for the minimum box size. Default = 1
* @see FLAnalyzer.FLMain#MakeNewBoxSizes*/
private int UserMinBoxSize=2, UserMaxBoxSize=0;

/**Number of times to do an entire series of grids (i.e., if multiple 
* origins are being used).  If set to 1,only one scan is done for each 
* &epsilon;, where each scan at a different grid size starts at the 
* same location. If the value is more than 1, the series of grid sizes
* is restarted several times from a different location. A special case
* is seen with {@link #GridPositions multifractal scans}. Subscans 
* cannot use more than one location. Is set by {@link 
 * FLAnalyzer.FLsetup#getBCInputs}  The number of grid 
 * positions used in the multifractal analysis as set in the {@link 
 * FLAnalyzer.FLsetup#getMFInputs get multifractal inputs} 
 * method, to temporarily 
* supersede {@link #GridPositions}. The default is 4; these first 
* four values are different than in the regular analysis.  The position 
* changes only if a {@link 
 * FLAnalyzer.FLMain#moveGridalloverCountPixels multifractal} 
 * analysis is being done.
* The change orients the grid so that it falls completely at each of 
* the  four corners of the rectangle encompassing the image. This 
* emulates rotating the image 90&deg; to all four positions,
* whereas the usual shift is a translation.  For calculations of the 
* positions, see
 * {@link FLAnalyzer.FLMain#moveGridalloverCountPixels moving} 
 * function.*/
public int GridPositions = 4;     
/**Maximum number of box sizes to use in a series.  Is set by {@link 
* FLAnalyzer.FLsetup#getBCInputs}. Default is 0.
* @see FLAnalyzer.FLMain#MakeNewBoxSizes */
private int MaxNumBoxSizes = 0;   
public int Subboxsize = 40;
/**Maximum percent of subbox to use as a box size. Is {@link 
* FLAnalyzer.FLMain#MakeNewBoxSizes applied} when doing 
* subareas. Is set by {@link FLAnalyzer.FLsetup#getBCInputs}.*/
public float percentsubboxsize = 100;

/**True switches execution to calculate the lacunarity using a {@link 
* FLAnalyzer.FLMain#GetSLACData sliding box}algorithm and also
* turns off the regular box counting and multifractal scans.*/
public boolean SLAC=false;

/**
* Value by which to slide the sliding box in
* {@link 
* FLAnalyzer.FLMain#GetSLACData 
* sliding lacunarity}
* method.
**/
public int SLIDEX = 5;

/**
* Value by which to slide the sliding box in
* {@link 
* FLAnalyzer.FLMain#GetSLACData 
* sliding lacunarity}
* method.
**/
public int  SLIDEY=5;

/**
* True shows graphs of regression lines.
* Is set by 
* {@link FLAnalyzer.FLsetup#getMFInputs}.
*/
public boolean GraphQs=false;

/**True moves each grid randomly at each &epsilon;, which is an
* additional shift to the shift at the beginning of each series of 
* &epsilon;s.  The effect is that in a scan, each new box size does not
* start at the same position as the previous.  This variable is used in 
* the {@link  FLAnalyzer.FLMain#moveGridalloverCountPixels the
* scanning}function.*/
public boolean slipGrid=false;
public boolean abort=true;

/**Variable for composite type.  Default is 5*/
public int comptype = 5;
/**True specifies whether or not to use the ImageJ particle analyzer.
*Is set in the @link getBCInputs method. Only applies when doing 
* subareas.*/
public boolean UseParticleAnalyzer = false;
/**Boolean that if true triggers a change in how the grid is positioned
* for a multifractal analysis. That is, for the first four scans in a 
* multifractal analysis using multiple origins, the grid is anchored at
* each of the four corners of the foreground pixel area's enclosing 
* rectangle, instead of  at the top left as is done with the standard 
* count.  This is akin to rotating 90&deg; each time, as opposed to 
* translating some random amount.
* @see #GridPositions
* @see FLAnalyzer.FLMain#moveGridalloverCountPixels moving 
* method*/
public boolean BottomRight=true;

/**Maximum percent of image area to use as box size. Is {@link 
* FLAnalyzer.FLMain#MakeNewBoxSizes applied} to the smallest
* {@link FLAnalyzer.FLMain#RectEnclosingForeground rectangle}
* enclosing the pixelated part of the image.  Is set by {@link 
* FLAnalyzer.FLsetup#getBCInputs}.*/
public float maxpercent=45;                         
public float base=2.00f;
public float exponent=0.50f;
/**Maximum percent of image area to use as sub box size for 
* multifractal random mass scans.  Based on smallest rectangle
* enclosing the pixelated part of the image. Is set by {@link 
* FLAnalyzer.FLsetup#getMFInputs} and used in the {@link 
* FLAnalyzer.FLMain#MakeNewBoxSizes box making} method.*/
public float RandomMassMaxSubpercent=0f;
/**Number of  subareas to analyze if doing a random sample. Is set 
* by {@link FLAnalyzer.FLsetup#GetSubScanInputs}.*/
public int subsamples = 3;
/**True prints summary data, and false prints the {@link GUI.FL_
* #WriteScanData  data}, including the mass information.  The
* mass data (mean pixels per box at each &epsilon;) can be used to 
* find the fractal dimension, as the slope of the regression line of the
* ln-ln plot of mean and &epsilon; (see {@link 
 * FLAnalyzer.FLMain#FinishQslopes Generalized Dimension}). 
 * Is set by {@link FLAnalyzer.FLsetup#getBCInputs}.*/
public boolean ShowData = true;
public boolean BLOCKTEXTUREANALYSIS=false;
/**If true, runs {@link FLAnalyzer.CircStats#measureHullandCircle
*circularity} calculations. Default is to run them. Is set by {@link 
* FLAnalyzer.FLsetup#getBCInputs}.*/
public boolean docircul = true;
/**True fills coloured squares if subareas are being done.  Is set by 
* {@link FLAnalyzer.FLsetup#GetSubScanInputs  subscanner}.*/
public boolean fill=false;
/**Integer sets the colour coding scheme according to the internal 
* ColourRule class in the {@link #ScanSubAreas subsampling 
* methods}.*/
public int colourcode=0;


/**True shows a graphic colour coded for the fractal dimension of 
* each subarea. Is set by {@link FLAnalyzer.FLsetup#getBCInputs}.
*/
public boolean showcolours = false;

/**True if the user wants to set the number of boxes rather than use 
* a value {@link FLAnalyzer.FLMain#MakeNewBoxSizes optimized}
* for each image size. Is set by {@link 
 * FLAnalyzer.FLsetup#getBCInputs}.*/
private boolean UserWantsToSpecifyNumberOfBoxes=false;
private boolean MF=false;
public boolean GraphAQ=false,
GraphFA=false,GraphFQ=false, GraphT=false,
GraphRegression=false, 
GraphLac=false, SelectBestCV=false,
WriteQslopeandCV=false;
/**True writes a great deal of data-each pixel count for each box at 
 * each box size for each location.
 * @see FLAnalyzer.FLDataProcessor#recordRawData*/
public boolean WriteRawData=false;
/**True prints pixel mass and probability frequency distributions.*/
public boolean printFrequencies=false;
/**True enables a random mass sample that accumulates with each 
 * scan, rather than terminates as otherwise happens.*/
public boolean RandomMassSample = false;
/**True enables the opening of many files at once.*/
public boolean MultiFiles = true;
///////////////////////////////////////////////////////////////
///////////  VARIABLES RETURNED 
///////FOR MORPHOMETRICS /////////////
////////////////////////////////////////////////////////////
/**Boolean is checked before an image or roi is processed. Initialized 
 * to false; Is changed in the input getting methods of {@link 
 * FLsetup set up method}.*/
public boolean proceed = false;

/**If true, a subscan is run using rectangular areas. Is set to false 
 * automatically in all non subscan calls -- i.e.,
 * {@link FLsetup#getBCInputs standard inputs}
 * {@link FLsetup#getSLacInputs  sliding lacunarity}, and
 * {@link FLsetup#getMFInputs multifractal inputs}--but is
 * manipulated in{@link FLsetup#GetSubScanInputs subs}.*/
public boolean UseRectAnalyzer = true;

/**Specifies the method for colouring subscans.
 *@see FLsetup#GetSubScanInputs*/
public int RoiColouringMethod = 0;
public static float[] DefaultColours = 
{0.19f, 0.69f, 1.09f, 1.29f,
1.49f, 1.69f, 1.89f, 2.19f, 
  2.39f, 2.69f, 2.99f};
/**Array of floats specifying the colours to use if 11 is selected in the
* colour selector for sub scans.
@see FLsetup#getUserColourGuides*/
public float[] UserColours = 
DefaultColours;

/**Perimeter of the {@link FLAnalyzer.CircStats#getHull convex 
* hull} that is set in{@link 
 * FLAnalyzer.CircStats#measureHullandCircle circularity}
 * method.*/
public float perimeterOfHull;
float major, minor;
/**Area of the {@link FLAnalyzer.CircStats#getHull convex hull} as
 * set in the  {@link FLAnalyzer.CircStats#measureHullandCircle 
 * circularity} method.*/
public float AREAofHull = 0;

/**Circularity of {@link FLAnalyzer.CircStats#getHull convex hull}
 *= 4.0*Math.PI*({@link FLAnalyzer.FLVars#AREAofHull}/
 * ({@link FLAnalyzer.FLVars#perimeterOfHull}<sup>2</sup>)
 * <li> which is 1.00 for a circle</li>;*/
public float circularity;

/**Greatest distance from the {@link FLVars#MassCentrex centre 
 * of mass} to the hull's points  found using {@link 
 * FLAnalyzer.CircStats#FindDistancesFromCentre the
 * distances method}.*/
public float MaxRadius;
/**This value is set in the {@link 
 * FLAnalyzer.CircStats#measureHullandCircle measuring} 
 * method, then is later made 
 * into an {@link FLAnalyzer.CircStats#MaxOverMinRadiiStr
 * element in the data string }and reported in the 
 * {@link GUI.FL_#BoxCountWindow results window}. 
 * </p>MaxOverMinRadii is changed using a
 * {@link FLAnalyzer.CircStats#FindDistancesFromCentre method} 
 * to find the distances from 
 * the centre of mass for the roi for all the points of the convex hull.
 * MaxOverMinRadii is set to the ratio of the largest to the smallest 
 * of these radii.*/
public float MaxOverMinRadii;

/**CV for the radii from the centre of mass to the hull's points.*/
public float CVRadii ;
/**Mean radii from the centre of mass to the hull's points.*/
public float MeanRadii ;
/**Greatest distance from the {@link FLVars#CircleMiddlex centre
 * of the circle} to the hull's points  found using 
 * {@link FLAnalyzer.CircStats#FindDistancesFromCentre 
 * dedicated method}.*/
public float MaxRadiuscircle;

/**This value is set in the 
 * {@link FLAnalyzer.CircStats#measureHullandCircle measuring}
 * method, then is later made 
 * into an {@link FLAnalyzer.CircStats#MaxOverMinRadiiStr 
 * element in the data string }and reported in the {@link 
 * GUI.FL_#BoxCountWindow results window}.</p>MaxOverMinRadii is 
 * changed using {@link CircStats#FindDistancesFromCentre a 
 * method} to find the distances from the centre of the circle for the
 * roi for all the points of the convex hull.  MaxOverMinRadiicircle is 
 * set to the ratio of the largest to the smallest of these radii.*/
public float MaxOverMinRadiicircle;
/**CV for the radii from the centre of the circle to the hull's points.*/
public float CVRadiicircle ;

/**Mean radii from the centre of the bounding circle to the hull's 
 * points.*/
public float MeanRadiicircle ;
//    /**True resizes the image so that the circle can be seen.*/
//  public boolean resize=false;
/**If true, the span method is used, otherwise the method is 
 * optimized.
 * @see FLsetup#getBCInputs getBCInputs
 * @see FLAnalyzer.CircStats CircStats*/
public boolean UseSpanMethod = false;
/**If true, the triangle method is used, otherwise the method is 
 * optimized.
 * @see FLsetup#getBCInputs getBCInputs
 * @see FLAnalyzer.CircStats CircStats*/
public boolean UseTriMethod=false;
/**This is the x coordinate for the centre of mass for the convex hull, 
* found using IJ's measurements after the convex hull is made the 
 * roi  in {@link FLAnalyzer.CircStats#measureHullandCircle 
 * method to make hull and circle}.*/
public float MassCentrex=0;
/**This is the y coordinate for the centre of mass for the convex hull,
 * found using IJ's measurements after the convex hull is made the 
 * roi  in {@link FLAnalyzer.CircStats#measureHullandCircle 
 * method to make hull and circle}.*/
public float  MassCentrey=0;
/** As set in {@link FLAnalyzer.CircStats#measureHullandCircle 
* method to make hull and circle}.*/
public float  CircleDiameter;
/** As set in {@link FLAnalyzer.CircStats#measureHullandCircle 
* method to make hull and circle}.*/
public float  CircleMiddlex;
/** As set in {@link FLAnalyzer.CircStats#measureHullandCircle
* method to make hull and circle}.*/
public float  CircleMiddley;
/**The greatest distance between any two points in the convex hull,
* as set in {@link FLAnalyzer.CircStats#measureHullandCircle 
* method to make hull and circle}.*/
public float MaximumSpanAcrossHull;
/**The shortest distance between any two points in the convex hull,
* set in {@link FLAnalyzer.CircStats#measureHullandCircle method
* to make hull and circle.}*/
public float MinimumSpanAcrossHull;
public float //bdiam = 0, vdiam=0, hdiam=0, tdiam=0,
GreaterHorizontalOrVerticalDimension=0;
/**Margins[0]=left margin, Margins[1]=right margin, 
* Margins[2]=top margin, Margins[3]=bottom margin*/
public int [] Margins=null;
/**Perimeter of the {@link FLAnalyzer.CircStats#getHull convex 
 * hull} that is set in 
 * {@link FLAnalyzer.CircStats#measureHullandCircle circularity} 
 * method.*/
public float perimeter;
public String GDataString="";
public String DataString="";
/**Area of the {@link FLAnalyzer.CircStats#getHull convex hull} as
 * set in the {@link FLAnalyzer.CircStats#measureHullandCircle 
 * circularity}method.*/
public float AREA = 0;           
/////////////////END VARIABLE DECLARATIONS////////////  

//============================//
/////////////////////          METHODS         ///////////////////      
/**Constructor that accepts no parameters*/
public FLVars ()
{}
/**@return true if the scan is to be {@link #abort aborted}*/
public boolean isAbort ()
{
return this.abort;            }

/**@param abort*/
public void setAbort (final boolean abort)
{
this.abort = abort;
}

/**@return an int for the value of {@link #actualx x} the scan 
 * started at*/
public int getActualx ()
{
return this.actualx;
}

/**
*
* @param actualx
*/
public void setActualx (final int actualx)
{
this.actualx = actualx;
}

/**
*
* @return an int for the value of {@link #actualy y} 
* the scan started at
*/
public int getActualy ()
{
return this.actualy;
}

/**
*
* @param actualy
*/
public void setActualy (final int actualy)
{
this.actualy = actualy;
}

/**
*
* @return int for background (255 or 0)
*/
public int getBackground ()
{
return this.background;
}

/**
*
* @param background
*/
public void setBackground (final int background)
{
this.background = background;
}



/**
*
* @return an int for the {@link #colourcode}
*/
public int getColourcode ()
{
return this.colourcode;
}

/**
*
* @param colourcode
*/
public void setColourcode (final int colourcode)
{
this.colourcode = colourcode;
}

/**
*
* @return an integer for the
 * {@link #comptype composite type}
*/
public int getComptype ()
{
return this.comptype;
}

/**
*
* @param comptype
*/
public void setComptype (final int comptype)
{
this.comptype = comptype;
}

/**
*
* @return true if {@link #docircul} 
 * circularity is being calculated
*/
public boolean isDocircul ()
{
return this.docircul;
}

/**
*
* @param docircul
*/
public void setDocircul (final boolean docircul)
{
this.docircul = docircul;
}



/**
*
* @return a float for {@link #FACTOR}
*/
public float getFACTOR ()
{
return this.FACTOR;
}

/**
*
* @param FACTOR
*/
public void setFACTOR (final float FACTOR)
{
this.FACTOR = FACTOR;
}



/**
*
* @param fill
*/
public void setFill (final boolean fill)
{
this.fill = fill;
}

/**
*
* @return an integer for foreground (either 0 or 255)
*/
public int getForeground ()
{
return this.foreground;
}

/**
*
* @param foreground
*/
public void setForeground (final int foreground)
{
this.foreground = foreground;
}

/**
*
* @return true if graphing &alpha; vs. Q
*/
public boolean isGraphAQ ()
{
return this.GraphAQ;
}

/**
*
* @param GraphAQ
*/
public void setGraphAQ (final boolean GraphAQ)
{
this.GraphAQ = GraphAQ;
}

/**
*
* @return true if graping &#402;(&alpha;) vs. &alpha;
*/
public boolean isGraphFA ()
{
return this.GraphFA;
}

/**
*
* @param GraphFA
*/
public void setGraphFA (final boolean GraphFA)
{
this.GraphFA = GraphFA;
}

/**
*
* @return true if graphing &#402;(&alpha;) vs Q
*/
public boolean isGraphFQ ()
{
return this.GraphFQ;
}

/**
*
* @param GraphFQ
*/
public void setGraphFQ (final boolean GraphFQ)
{
this.GraphFQ = GraphFQ;
}

/**
*
* @return true if graphing any sliding box lacunarity graphs
*/
public boolean isGraphLac ()
{
return this.GraphLac;
}

/**
*
* @param GraphLac
*/
public void setGraphLac (final boolean GraphLac)
{
this.GraphLac = GraphLac;
}

/**
*
* @return true if graphing any multifractal graphs
*/
public boolean isGraphQs ()
{
return this.GraphQs;
}

/**
*
* @param GraphQs
*/
public void setGraphQs (final boolean GraphQs)
{
this.GraphQs = GraphQs;
}

/**
*
* @return true if graphing regression lines
*/
public boolean isGraphRegression ()
{
return this.GraphRegression;
}

/**
*
* @param GraphRegression
*/
public void setGraphRegression
(final boolean GraphRegression)
{
this.GraphRegression = GraphRegression;
}

/**
*
* @return true if Graphing Tau
*/
public boolean isGraphT ()
{
return this.GraphT;
}

/**
*
* @param GraphT
*/
public void setGraphT (final boolean GraphT)
{
this.GraphT = GraphT;
}



/**
*
* @return true if this is a multifractal scan
*/
public boolean isMF ()
{
return this.MF;
}



/**
*
* @return float for the increment between q values
*@see #qinc
*/
public float getQinc ()
{
return this.qinc;
}

/**
*
* @param qinc
*/
public void setQinc (final float qinc)
{
this.qinc = qinc;
}

/**
*
* @return float for the maximum {@link #qmax q} value
*/
public float getQmax ()
{
return this.qmax;
}

/**
*
* @param qmax
*/
public void setQmax (final float qmax)
{
this.qmax = qmax;
}
static String tab = "\t";
/**
* Returns the passed array of strings as a single
* string separated by tabs.
* Is used to list the headings for the results file as a string. 
* Is called in the
*     {@link GUI.FL_#run} method of {@link GUI.FL_}</p>
* @return string of headings.
* @param arrayofheadings an array of strings
*/
public static String MakeHeadings (String [] 
arrayofheadings)
{
String headings ="";
{
headings=arrayofheadings[0];
for (int i = 1; i<arrayofheadings.length;i++)
headings=headings+tab+arrayofheadings[i];

}
return headings;
}
/**
*
* @return float for the {@link #qmin minimum q value}
*/
public float getQmin ()
{
return this.qmin;
}

/**
*
* @param qmin
*/
public void setQmin (final float qmin)
{
this.qmin = qmin;
}






/**
*
* @return true if the particleanalyzer is being used
*/
public boolean isUseParticleAnalyzer ()
{
return this.UseParticleAnalyzer;
}

/**
*
* @param UseParticleAnalyzer
*/
public void setUseParticleAnalyzer
(final boolean UseParticleAnalyzer)
{
this.UseParticleAnalyzer = UseParticleAnalyzer;
}



/*Returns true if the {@link #imagetype} is {@link GRAY}
or false otherwise.
* 
* @return true if imagetype is gray, otherwise false
*/
public boolean isGray()
{
return ImageType==GRAY?true:false;
}
/*Returns true if the {@link #imagetype} is {@link BINARY}
or false otherwise.
* @return true if imagetype is binary, otherwise false
*/
/**
* 
* @return boolean
*/
public boolean isBinary()
{
return ImageType==BINARY;
}
/**Returns true if the {@link #ImageType} is 
* {@link #THRESHHOLD}
* or false otherwise.                           * 
* @return boolean true if {@link #ImageType} = 
*{@link #THRESHHOLD}.
*/
public boolean isAutoThresh()
{
return ImageType==THRESHHOLD;
}
/*Returns the value stored in {@link #ImageType}.*/
/**
* 
* @return int for {@link #ImageType}.
*/
public int getImageType()
{
return this.ImageType;
}
/*Sets the {@link #ImageType} to the 
passed value.
@param imagetype int to set the variable to.*/
/**
* 
* @param imagetype 
*/
public void setImageType(final int imagetype)
{
this.ImageType = imagetype;
}



/**
* Sets the values of {@link #background} 
* and {@link #foreground}
* according to the passed ImageStatistics or the value in
* {@link #ChooseBackground}. 
* If the {@link #ChooseBackground option}
* is set to 
* {@link #USEBLACK use black} then the 
* background is set to
* {@link #BLACK} and the {@link #foreground}
* to {@link #WHITE};
* if it is set to use white, then the reverse are set.
* But if the {@link #ChooseBackground option} is 
* set to {@link #CHOOSEAUTOMATICALLY let the
* program choose},
* then if there are more white pixels than 
* black in the passed
* ImageStatistics, then the background is 
* white and foreground black,
* or if there are more black than white, 
* then the foreground is white
* and the background black.
* @param ss ImageStatistics for determining background
*/         
public void setForegroundandBackground
(ImageStatistics ss)
{//set the foreground to the colour
//that is least prevalent

if (ChooseBackground==USEBLACK)
{
background=BLACK;foreground=WHITE;
}
else if (ChooseBackground==USEWHITE)
{
background=WHITE;foreground=BLACK;
}
else if
(ChooseBackground==
CHOOSEAUTOMATICALLY)
{if (ss.histogram[0]>ss.histogram[255])
{
setForeground (255);
setBackground (0);
}
else
{
setForeground (0);
setBackground (255);
}}

}

/**
* Returns true if the 
* {@link #Series} is 
* a {@link #RELATIVESERIES}.
* @return boolean that is true if 
* {@link #Series}=
* {@link #RELATIVESERIES}, otherwise false
*/
public boolean isRelative()
{
return (Series==RELATIVESERIES)?
true:false;
}



/**
* 
* @return int
*/
public int getUserMinBoxSize ()
{
return UserMinBoxSize;
}

/**
* 
* @param UserMinBoxSize 
*/
public void setUserMinBoxSize (int UserMinBoxSize)
{
this.UserMinBoxSize = UserMinBoxSize;
}

/**
* 
* @return int
*/
public int getUserMaxBoxSize ()
{
return UserMaxBoxSize;
}

/**
* 
* @param UserMaxBoxSize 
*/
public void setUserMaxBoxSize (int UserMaxBoxSize)
{
this.UserMaxBoxSize = UserMaxBoxSize;
}



/**
* 
* @return int
*/
public int getMaxNumBoxSizes ()
{
return MaxNumBoxSizes;
}

/**
* 
* @param MaxNumBoxSizes 
*/
public void setMaxNumBoxSizes (int MaxNumBoxSizes)
{
this.MaxNumBoxSizes = MaxNumBoxSizes;
}




/**
* 
* @return boolean
*/
public boolean isUserWantsToSpecifyNumberOfBoxes ()
{
return UserWantsToSpecifyNumberOfBoxes;
}

/**
* 
* @param UserWantsToSpecifyNumberOfBoxes 
*/
public void setUserWantsToSpecifyNumberOfBoxes 
(boolean UserWantsToSpecifyNumberOfBoxes)
{
this.UserWantsToSpecifyNumberOfBoxes =
UserWantsToSpecifyNumberOfBoxes;
}


/**
* 
* @param type 
* @return String
*/
public String StringForIntensityCalculation(int type)
{String I  =  "I (Intensity at i,j for " +FLMain.epsilon+
") = (max-min)*size"+FLMain.sq;
if (type==DIFFERENTIALBC)
I=   "I (Intensity at i,j for " +FLMain.epsilon+
") = max-min+1";
return I;

}

/**Returns a string describing the formula used to
*calculate the fractal dimension.  
* 
* @param type int for the type of scan
* @see #BINARY
* @see #VARIATION2DBC
* @see #DIFFERENTIALBC
* @return string describing fractal dimension formula used
*/
public String StringForDFFormula(int type)
{
String B = "";
if(type==BINARY)
B="D=slope of [ln "+FLMain.sum+"Count/ln "
+FLMain.epsilon+ "]";
if(type==SLIDINGLAC)
B="D=slope of [ln "+FLMain.sum+"Mass/ln "
+FLMain.epsilon+ "]";
if(type==VARIATION2DBC)
B="D=3-(s/2) where s=slope [ln "
+FLMain.sum+" I/ln "+FLMain.epsilon+"]";
if (type==DIFFERENTIALBC)
B=   "D=slope [ln " +FLMain.sum+
" I/ln (1/" +FLMain.epsilon+ ")]";
return B;

}


/**
* 
* @param MF 
*/
public void setMF(final boolean MF)
{
this.MF = MF;
}
}/*end plugin*/
